@@ -4,10 +4,11 @@
 use std::collections::HashMap;
 
 use pyo3::{exceptions::PyValueError, prelude::*, sync::PyOnceLock};
-use tensorzero_core::endpoints::dynamic_evaluation_run::DynamicEvaluationRunEpisodeResponse;
+use tensorzero_core::endpoints::workflow_evaluation_run::WorkflowEvaluationRunEpisodeResponse;
 use tensorzero_core::inference::types::pyo3_helpers::{deserialize_from_pyobj, serialize_to_dict};
 use tensorzero_rust::{
-    DynamicEvaluationRunResponse, FeedbackResponse, InferenceResponse, InferenceResponseChunk, Tool,
+    FeedbackResponse, InferenceResponse, InferenceResponseChunk, Tool,
+    WorkflowEvaluationRunResponse,
 };
 use uuid::Uuid;
 
@@ -64,43 +65,44 @@ pub fn parse_inference_chunk(py: Python<'_>, chunk: InferenceResponseChunk) -> P
     Ok(python_parsed.into_any())
 }
 
-pub fn parse_dynamic_evaluation_run_response(
+pub fn parse_workflow_evaluation_run_response(
     py: Python<'_>,
-    data: DynamicEvaluationRunResponse,
+    data: WorkflowEvaluationRunResponse,
 ) -> PyResult<Py<PyAny>> {
-    static PARSE_DYNAMIC_EVALUATION_RUN_RESPONSE: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+    static PARSE_WORKFLOW_EVALUATION_RUN_RESPONSE: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
     // This should never actually fail, since we're just importing code defined in our own Python
     // package. However, we still produce a Python error if it fails, rather than panicking
     // and bringing down the entire Python process.
-    let parse_dynamic_evaluation_run_response = PARSE_DYNAMIC_EVALUATION_RUN_RESPONSE
+    let parse_workflow_evaluation_run_response = PARSE_WORKFLOW_EVALUATION_RUN_RESPONSE
         .get_or_try_init::<_, PyErr>(py, || {
             let self_module = PyModule::import(py, "tensorzero.types")?;
             Ok(self_module
-                .getattr("parse_dynamic_evaluation_run_response")?
+                .getattr("parse_workflow_evaluation_run_response")?
                 .unbind())
         })?;
     let json_data = serialize_to_dict(py, data)?;
-    let python_parsed = parse_dynamic_evaluation_run_response.call1(py, (json_data,))?;
+    let python_parsed = parse_workflow_evaluation_run_response.call1(py, (json_data,))?;
     Ok(python_parsed.into_any())
 }
 
-pub fn parse_dynamic_evaluation_run_episode_response(
+pub fn parse_workflow_evaluation_run_episode_response(
     py: Python<'_>,
-    data: DynamicEvaluationRunEpisodeResponse,
+    data: WorkflowEvaluationRunEpisodeResponse,
 ) -> PyResult<Py<PyAny>> {
-    static PARSE_DYNAMIC_EVALUATION_RUN_EPISODE_RESPONSE: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
+    static PARSE_WORKFLOW_EVALUATION_RUN_EPISODE_RESPONSE: PyOnceLock<Py<PyAny>> =
+        PyOnceLock::new();
     // This should never actually fail, since we're just importing code defined in our own Python
     // package. However, we still produce a Python error if it fails, rather than panicking
     // and bringing down the entire Python process.
-    let parse_dynamic_evaluation_run_episode_response =
-        PARSE_DYNAMIC_EVALUATION_RUN_EPISODE_RESPONSE.get_or_try_init::<_, PyErr>(py, || {
+    let parse_workflow_evaluation_run_episode_response =
+        PARSE_WORKFLOW_EVALUATION_RUN_EPISODE_RESPONSE.get_or_try_init::<_, PyErr>(py, || {
             let self_module = PyModule::import(py, "tensorzero.types")?;
             Ok(self_module
-                .getattr("parse_dynamic_evaluation_run_episode_response")?
+                .getattr("parse_workflow_evaluation_run_episode_response")?
                 .unbind())
         })?;
     let json_data = serialize_to_dict(py, data)?;
-    let python_parsed = parse_dynamic_evaluation_run_episode_response.call1(py, (json_data,))?;
+    let python_parsed = parse_workflow_evaluation_run_episode_response.call1(py, (json_data,))?;
     Ok(python_parsed.into_any())
 }
 
