@@ -26,10 +26,8 @@ from tensorzero import (
     AsyncTensorZeroGateway,
     ChatDatapoint,
     ChatDatapointInsert,
-    ChatInferenceDatapointInput,
     JsonDatapoint,
     JsonDatapointInsert,
-    JsonInferenceDatapointInput,
     Template,
     TensorZeroError,
     TensorZeroGateway,
@@ -38,7 +36,7 @@ from tensorzero import (
 from uuid_utils import uuid7
 
 
-def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
+def test_sync_insert_delete_datapoints(sync_client: TensorZeroGateway):
     dataset_name = f"test_{uuid7()}"
     datapoints = [
         ChatDatapointInsert(
@@ -46,7 +44,10 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
             input={
                 "system": {"assistant_name": "foo"},
                 "messages": [
-                    {"role": "user", "content": [{"type": "text", "text": "bar"}]}
+                    {
+                        "role": "user",
+                        "content": [{"type": "text", "text": "bar"}],
+                    }
                 ],
             },
             output=[{"type": "text", "text": "foobar"}],
@@ -56,8 +57,7 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
             parallel_tool_calls=False,
             tags=None,
         ),
-        # Ensure deprecated ChatInferenceDatapointInput is still supported
-        ChatInferenceDatapointInput(
+        ChatDatapointInsert(
             function_name="basic_test",
             input={
                 "system": {"assistant_name": "Dummy"},
@@ -72,7 +72,11 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
                 {
                     "type": "tool_call",
                     "name": "get_temperature",
-                    "arguments": {"location": "New York", "units": "fahrenheit"},
+                    "id": "tool_call_id",
+                    "arguments": {
+                        "location": "New York",
+                        "units": "fahrenheit",
+                    },
                 }
             ],
             additional_tools=[
@@ -119,8 +123,7 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
             output_schema=None,
             tags=None,
         ),
-        # Ensure deprecated JsonInferenceDatapointInput is still supported
-        JsonInferenceDatapointInput(
+        JsonDatapointInsert(
             function_name="json_success",
             input={
                 "system": {"assistant_name": "foo"},
@@ -139,9 +142,7 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
             tags=None,
         ),
     ]
-    datapoint_ids = sync_client.bulk_insert_datapoints(
-        dataset_name=dataset_name, datapoints=datapoints
-    )
+    datapoint_ids = sync_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
     assert len(datapoint_ids) == 4
     assert isinstance(datapoint_ids[0], UUID)
     assert isinstance(datapoint_ids[1], UUID)
@@ -157,22 +158,14 @@ def test_sync_bulk_insert_delete_datapoints(sync_client: TensorZeroGateway):
     assert all(isinstance(dp, ChatDatapoint) for dp in listed_datapoints)
     assert all(dp.function_name == "basic_test" for dp in listed_datapoints)
 
-    sync_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[0]
-    )
-    sync_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[1]
-    )
-    sync_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[2]
-    )
-    sync_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[3]
-    )
+    sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+    sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[1])
+    sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[2])
+    sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[3])
 
 
 @pytest.mark.asyncio
-async def test_async_bulk_insert_delete_datapoints(
+async def test_async_insert_delete_datapoints(
     async_client: AsyncTensorZeroGateway,
 ):
     datapoints = [
@@ -181,7 +174,10 @@ async def test_async_bulk_insert_delete_datapoints(
             input={
                 "system": {"assistant_name": "foo"},
                 "messages": [
-                    {"role": "user", "content": [{"type": "text", "text": "bar"}]}
+                    {
+                        "role": "user",
+                        "content": [{"type": "text", "text": "bar"}],
+                    }
                 ],
             },
         ),
@@ -199,8 +195,12 @@ async def test_async_bulk_insert_delete_datapoints(
             output=[
                 {
                     "type": "tool_call",
+                    "id": "tool_call_id",
                     "name": "get_temperature",
-                    "arguments": {"location": "New York", "units": "fahrenheit"},
+                    "arguments": {
+                        "location": "New York",
+                        "units": "fahrenheit",
+                    },
                 }
             ],
             additional_tools=[
@@ -264,9 +264,7 @@ async def test_async_bulk_insert_delete_datapoints(
         ),
     ]
     dataset_name = f"test_{uuid7()}"
-    datapoint_ids = await async_client.bulk_insert_datapoints(
-        dataset_name=dataset_name, datapoints=datapoints
-    )
+    datapoint_ids = await async_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
     assert len(datapoint_ids) == 4
     assert isinstance(datapoint_ids[0], UUID)
     assert isinstance(datapoint_ids[1], UUID)
@@ -274,9 +272,7 @@ async def test_async_bulk_insert_delete_datapoints(
     assert isinstance(datapoint_ids[3], UUID)
 
     # Get a chat datapoint
-    datapoint = await async_client.get_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[0]
-    )
+    datapoint = await async_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
     print(datapoint)
     assert isinstance(datapoint, ChatDatapoint)
     assert datapoint.function_name == "basic_test"
@@ -290,9 +286,7 @@ async def test_async_bulk_insert_delete_datapoints(
     assert datapoint.output is None
 
     # Get a json datapoint
-    datapoint = await async_client.get_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[2]
-    )
+    datapoint = await async_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[2])
     assert isinstance(datapoint, JsonDatapoint)
     assert datapoint.function_name == "json_success"
     assert datapoint.input.system == {"assistant_name": "foo"}
@@ -326,18 +320,10 @@ async def test_async_bulk_insert_delete_datapoints(
     assert all(dp.is_custom for dp in listed_datapoints)
     assert all(dp.function_name == "basic_test" for dp in listed_datapoints)
 
-    await async_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[0]
-    )
-    await async_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[1]
-    )
-    await async_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[2]
-    )
-    await async_client.delete_datapoint(
-        dataset_name=dataset_name, datapoint_id=datapoint_ids[3]
-    )
+    await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+    await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[1])
+    await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[2])
+    await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[3])
 
 
 @pytest.mark.asyncio
@@ -350,9 +336,7 @@ async def test_list_nonexistent_dataset(async_client: AsyncTensorZeroGateway):
 async def test_get_nonexistent_datapoint(async_client: AsyncTensorZeroGateway):
     datapoint_id: UUID = uuid7()  # type: ignore
     with pytest.raises(TensorZeroError) as exc_info:
-        await async_client.get_datapoint(
-            dataset_name="nonexistent_dataset", datapoint_id=datapoint_id
-        )
+        await async_client.get_datapoint(dataset_name="nonexistent_dataset", datapoint_id=datapoint_id)
     assert "Datapoint not found for" in str(exc_info.value)
     assert "404" in str(exc_info.value)
 
@@ -383,9 +367,7 @@ def test_sync_render_datapoints(embedded_sync_client: TensorZeroGateway):
                 "messages": [
                     {
                         "role": "user",
-                        "content": [
-                            {"type": "text", "arguments": {"country": "France"}}
-                        ],
+                        "content": [{"type": "text", "arguments": {"country": "France"}}],
                     }
                 ],
             },
@@ -397,9 +379,7 @@ def test_sync_render_datapoints(embedded_sync_client: TensorZeroGateway):
         ),
     ]
 
-    datapoint_ids = embedded_sync_client.bulk_insert_datapoints(
-        dataset_name=dataset_name, datapoints=datapoints
-    )
+    datapoint_ids = embedded_sync_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
     assert len(datapoint_ids) == 2
 
     # List the inserted datapoints
@@ -415,15 +395,10 @@ def test_sync_render_datapoints(embedded_sync_client: TensorZeroGateway):
     assert len(rendered_samples) == 2
 
     # Verify the chat datapoint was rendered correctly
-    chat_sample = next(
-        rs for rs in rendered_samples if rs.function_name == "basic_test"
-    )
+    chat_sample = next(rs for rs in rendered_samples if rs.function_name == "basic_test")
     assert chat_sample.episode_id is None
     assert chat_sample.inference_id is None
-    assert (
-        chat_sample.input.system
-        == "You are a helpful and friendly assistant named TestBot"
-    )
+    assert chat_sample.input.system == "You are a helpful and friendly assistant named TestBot"
     assert len(chat_sample.input.messages) == 1
     assert chat_sample.input.messages[0].role == "user"
     assert len(chat_sample.input.messages[0].content) == 1
@@ -431,9 +406,7 @@ def test_sync_render_datapoints(embedded_sync_client: TensorZeroGateway):
     assert chat_sample.input.messages[0].content[0].text == "Hello, world!"
 
     # Verify the json datapoint was rendered correctly
-    json_sample = next(
-        rs for rs in rendered_samples if rs.function_name == "json_success"
-    )
+    json_sample = next(rs for rs in rendered_samples if rs.function_name == "json_success")
     assert json_sample.episode_id is None
     assert json_sample.inference_id is None
     assert json_sample.input.system is not None
@@ -442,20 +415,17 @@ def test_sync_render_datapoints(embedded_sync_client: TensorZeroGateway):
     assert json_sample.input.messages[0].role == "user"
     assert len(json_sample.input.messages[0].content) == 1
     assert isinstance(json_sample.input.messages[0].content[0], Text)
-    assert (
-        json_sample.input.messages[0].content[0].text
-        == "What is the name of the capital city of France?"
-    )
+    assert json_sample.input.messages[0].content[0].text == "What is the name of the capital city of France?"
 
     # Clean up
     for datapoint_id in datapoint_ids:
-        embedded_sync_client.delete_datapoint(
-            dataset_name=dataset_name, datapoint_id=datapoint_id
-        )
+        embedded_sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_id)
 
 
 @pytest.mark.asyncio
-async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGateway):
+async def test_async_render_datapoints(
+    embedded_async_client: AsyncTensorZeroGateway,
+):
     """Test rendering datapoints using experimental_render_samples (async version)."""
     dataset_name = f"test_render_async_{uuid7()}"
 
@@ -468,14 +438,15 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
                 "messages": [
                     {
                         "role": "user",
-                        "content": [
-                            {"type": "text", "text": "What's the weather like?"}
-                        ],
+                        "content": [{"type": "text", "text": "What's the weather like?"}],
                     }
                 ],
             },
             output=[
-                {"type": "text", "text": "I don't have access to current weather data."}
+                {
+                    "type": "text",
+                    "text": "I don't have access to current weather data.",
+                }
             ],
         ),
         JsonDatapointInsert(
@@ -485,9 +456,7 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
                 "messages": [
                     {
                         "role": "user",
-                        "content": [
-                            {"type": "text", "arguments": {"country": "Italy"}}
-                        ],
+                        "content": [{"type": "text", "arguments": {"country": "Italy"}}],
                     }
                 ],
             },
@@ -499,15 +468,11 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
         ),
     ]
 
-    datapoint_ids = await embedded_async_client.bulk_insert_datapoints(
-        dataset_name=dataset_name, datapoints=datapoints
-    )
+    datapoint_ids = await embedded_async_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
     assert len(datapoint_ids) == 2
 
     # List the inserted datapoints
-    listed_datapoints = await embedded_async_client.list_datapoints(
-        dataset_name=dataset_name
-    )
+    listed_datapoints = await embedded_async_client.list_datapoints(dataset_name=dataset_name)
     assert len(listed_datapoints) == 2
 
     # Render the datapoints using experimental_render_samples
@@ -519,15 +484,10 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
     assert len(rendered_samples) == 2
 
     # Verify the chat datapoint was rendered correctly
-    chat_sample = next(
-        rs for rs in rendered_samples if rs.function_name == "basic_test"
-    )
+    chat_sample = next(rs for rs in rendered_samples if rs.function_name == "basic_test")
     assert chat_sample.episode_id is None
     assert chat_sample.inference_id is None
-    assert (
-        chat_sample.input.system
-        == "You are a helpful and friendly assistant named AsyncBot"
-    )
+    assert chat_sample.input.system == "You are a helpful and friendly assistant named AsyncBot"
     assert len(chat_sample.input.messages) == 1
     assert chat_sample.input.messages[0].role == "user"
     assert len(chat_sample.input.messages[0].content) == 1
@@ -535,9 +495,7 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
     assert chat_sample.input.messages[0].content[0].text == "What's the weather like?"
 
     # Verify the json datapoint was rendered correctly
-    json_sample = next(
-        rs for rs in rendered_samples if rs.function_name == "json_success"
-    )
+    json_sample = next(rs for rs in rendered_samples if rs.function_name == "json_success")
     assert json_sample.episode_id is None
     assert json_sample.inference_id is None
     assert json_sample.input.system is not None
@@ -546,19 +504,16 @@ async def test_async_render_datapoints(embedded_async_client: AsyncTensorZeroGat
     assert json_sample.input.messages[0].role == "user"
     assert len(json_sample.input.messages[0].content) == 1
     assert isinstance(json_sample.input.messages[0].content[0], Text)
-    assert (
-        json_sample.input.messages[0].content[0].text
-        == "What is the name of the capital city of Italy?"
-    )
+    assert json_sample.input.messages[0].content[0].text == "What is the name of the capital city of Italy?"
 
     # Clean up
     for datapoint_id in datapoint_ids:
-        await embedded_async_client.delete_datapoint(
-            dataset_name=dataset_name, datapoint_id=datapoint_id
-        )
+        await embedded_async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_id)
 
 
-def test_sync_render_filtered_datapoints(embedded_sync_client: TensorZeroGateway):
+def test_sync_render_filtered_datapoints(
+    embedded_sync_client: TensorZeroGateway,
+):
     """Test rendering only specific datapoints by filtering function name."""
     dataset_name = f"test_render_filter_{uuid7()}"
 
@@ -595,24 +550,18 @@ def test_sync_render_filtered_datapoints(embedded_sync_client: TensorZeroGateway
                 "messages": [
                     {
                         "role": "user",
-                        "content": [
-                            {"type": "text", "arguments": {"country": "Spain"}}
-                        ],
+                        "content": [{"type": "text", "arguments": {"country": "Spain"}}],
                     },
                 ],
             },
         ),
     ]
 
-    datapoint_ids = embedded_sync_client.bulk_insert_datapoints(
-        dataset_name=dataset_name, datapoints=datapoints
-    )
+    datapoint_ids = embedded_sync_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
     assert len(datapoint_ids) == 3
 
     # List only the basic_test datapoints
-    chat_datapoints = embedded_sync_client.list_datapoints(
-        dataset_name=dataset_name, function_name="basic_test"
-    )
+    chat_datapoints = embedded_sync_client.list_datapoints(dataset_name=dataset_name, function_name="basic_test")
     assert len(chat_datapoints) == 2
 
     # Render only the chat datapoints
@@ -631,6 +580,182 @@ def test_sync_render_filtered_datapoints(embedded_sync_client: TensorZeroGateway
 
     # Clean up
     for datapoint_id in datapoint_ids:
-        embedded_sync_client.delete_datapoint(
-            dataset_name=dataset_name, datapoint_id=datapoint_id
-        )
+        embedded_sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_id)
+
+
+def test_sync_bulk_insert_datapoints_deprecated(sync_client: TensorZeroGateway):
+    dataset_name = f"test_{uuid7()}"
+    datapoints = [
+        ChatDatapointInsert(
+            function_name="basic_test",
+            input={
+                "system": {"assistant_name": "foo"},
+                "messages": [{"role": "user", "content": [{"type": "text", "text": "bar"}]}],
+            },
+            output=[{"type": "text", "text": "foobar"}],
+        ),
+    ]
+
+    # Test that the deprecated function still works
+    with pytest.warns(DeprecationWarning, match="Please use `create_datapoints` instead"):
+        datapoint_ids = sync_client.bulk_insert_datapoints(dataset_name=dataset_name, datapoints=datapoints)
+
+    assert len(datapoint_ids) == 1
+    assert isinstance(datapoint_ids[0], UUID)
+
+    # Clean up
+    sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+
+
+@pytest.mark.asyncio
+async def test_async_bulk_insert_datapoints_deprecated(
+    async_client: AsyncTensorZeroGateway,
+):
+    dataset_name = f"test_{uuid7()}"
+    datapoints = [
+        ChatDatapointInsert(
+            function_name="basic_test",
+            input={
+                "system": {"assistant_name": "foo"},
+                "messages": [{"role": "user", "content": [{"type": "text", "text": "bar"}]}],
+            },
+            output=[{"type": "text", "text": "foobar"}],
+        ),
+    ]
+
+    # Test that the deprecated function still works
+    with pytest.warns(DeprecationWarning, match="Please use `create_datapoints` instead"):
+        datapoint_ids = await async_client.bulk_insert_datapoints(dataset_name=dataset_name, datapoints=datapoints)
+
+    assert len(datapoint_ids) == 1
+    assert isinstance(datapoint_ids[0], UUID)
+
+    # Clean up
+    await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+
+
+def test_sync_datapoints_with_name(sync_client: TensorZeroGateway):
+    """Test that datapoints with name field are correctly stored and retrieved."""
+    dataset_name = f"test_name_{uuid7()}"
+
+    # Create datapoints with name field
+    datapoints = [
+        ChatDatapointInsert(
+            function_name="basic_test",
+            input={
+                "system": {"assistant_name": "TestBot"},
+                "messages": [
+                    {
+                        "role": "user",
+                        "content": [{"type": "text", "text": "Hello"}],
+                    }
+                ],
+            },
+            output=[{"type": "text", "text": "Hi there!"}],
+            name="greeting_example",
+        ),
+        JsonDatapointInsert(
+            function_name="json_success",
+            input={
+                "system": {"assistant_name": "JsonBot"},
+                "messages": [
+                    {
+                        "role": "user",
+                        "content": [{"type": "text", "arguments": {"country": "Japan"}}],
+                    }
+                ],
+            },
+            output={"answer": "Tokyo"},
+            name="tokyo_capital_query",
+        ),
+    ]
+
+    # Insert datapoints
+    datapoint_ids = sync_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
+    assert len(datapoint_ids) == 2
+
+    # Retrieve and verify chat datapoint with name
+    chat_datapoint = sync_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+    assert isinstance(chat_datapoint, ChatDatapoint)
+    assert chat_datapoint.name == "greeting_example"
+    assert chat_datapoint.function_name == "basic_test"
+
+    # Retrieve and verify json datapoint with name
+    json_datapoint = sync_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[1])
+    assert isinstance(json_datapoint, JsonDatapoint)
+    assert json_datapoint.name == "tokyo_capital_query"
+    assert json_datapoint.function_name == "json_success"
+
+    # Clean up
+    for datapoint_id in datapoint_ids:
+        sync_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_id)
+
+
+@pytest.mark.asyncio
+async def test_async_datapoints_with_name(async_client: AsyncTensorZeroGateway):
+    """Test that datapoints with name field are correctly stored and retrieved (async version)."""
+    dataset_name = f"test_name_async_{uuid7()}"
+
+    # Create datapoints with name field
+    datapoints = [
+        ChatDatapointInsert(
+            function_name="basic_test",
+            input={
+                "system": {"assistant_name": "AsyncBot"},
+                "messages": [
+                    {
+                        "role": "user",
+                        "content": [{"type": "text", "text": "Good morning"}],
+                    }
+                ],
+            },
+            output=[{"type": "text", "text": "Good morning to you!"}],
+            name="morning_greeting",
+        ),
+        JsonDatapointInsert(
+            function_name="json_success",
+            input={
+                "system": {"assistant_name": "AsyncJsonBot"},
+                "messages": [
+                    {
+                        "role": "user",
+                        "content": [
+                            {
+                                "type": "text",
+                                "arguments": {"country": "Germany"},
+                            }
+                        ],
+                    }
+                ],
+            },
+            output={"answer": "Berlin"},
+            name="berlin_capital_query",
+        ),
+    ]
+
+    # Insert datapoints
+    datapoint_ids = await async_client.create_datapoints(dataset_name=dataset_name, datapoints=datapoints)
+    assert len(datapoint_ids) == 2
+
+    # Retrieve and verify chat datapoint with name
+    chat_datapoint = await async_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[0])
+    assert isinstance(chat_datapoint, ChatDatapoint)
+    assert chat_datapoint.name == "morning_greeting"
+    assert chat_datapoint.function_name == "basic_test"
+
+    # Retrieve and verify json datapoint with name
+    json_datapoint = await async_client.get_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_ids[1])
+    assert isinstance(json_datapoint, JsonDatapoint)
+    assert json_datapoint.name == "berlin_capital_query"
+    assert json_datapoint.function_name == "json_success"
+
+    # List all datapoints and verify names are preserved
+    all_datapoints = await async_client.list_datapoints(dataset_name=dataset_name)
+    assert len(all_datapoints) == 2
+    names = {dp.name for dp in all_datapoints}
+    assert "morning_greeting" in names
+    assert "berlin_capital_query" in names
+
+    # Clean up
+    for datapoint_id in datapoint_ids:
+        await async_client.delete_datapoint(dataset_name=dataset_name, datapoint_id=datapoint_id)
