@@ -62,9 +62,9 @@ def test_simple_list_json_inferences(embedded_sync_client: TensorZeroGateway):
     # ORDER BY timestamp DESC is applied - verify timestamps are in descending order
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] >= timestamps[i + 1], (
-            f"Timestamps not in descending order: {timestamps[i]} < {timestamps[i + 1]}"
-        )
+        assert timestamps[i] >= timestamps[i + 1], f"Timestamps not in descending order: {timestamps[i]} < {
+            timestamps[i + 1]
+        }"
 
 
 def test_simple_query_with_float_filter(embedded_sync_client: TensorZeroGateway):
@@ -130,18 +130,19 @@ def test_simple_query_chat_function(embedded_sync_client: TensorZeroGateway):
         assert isinstance(inference_id, UUID)
         episode_id = inference.episode_id
         assert isinstance(episode_id, UUID)
-        tool_params = inference.tool_params
-        assert tool_params is not None
-        assert tool_params.tools_available == []
-        assert tool_params.parallel_tool_calls is None
+        # Test individual tool param fields
+        assert inference.allowed_tools is None
+        assert inference.additional_tools is None
+        assert inference.parallel_tool_calls is None
+        assert inference.provider_tools is None
         assert len(inference.dispreferred_outputs) == 0
 
     # ORDER BY timestamp ASC is applied - verify timestamps are in ascending order
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] <= timestamps[i + 1], (
-            f"Timestamps not in ascending order: {timestamps[i]} > {timestamps[i + 1]}"
-        )
+        assert timestamps[i] <= timestamps[i + 1], f"Timestamps not in ascending order: {timestamps[i]} > {
+            timestamps[i + 1]
+        }"
 
 
 def test_simple_query_chat_function_with_tools(embedded_sync_client: TensorZeroGateway):
@@ -211,20 +212,11 @@ def test_simple_query_chat_function_with_tools(embedded_sync_client: TensorZeroG
         assert isinstance(inference_id, UUID)
         episode_id = inference.episode_id
         assert isinstance(episode_id, UUID)
-        tool_params = inference.tool_params
-        assert tool_params is not None
-        assert len(tool_params.tools_available) == 4
-        for tool in tool_params.tools_available:
-            assert tool.name in [
-                "think",
-                "search_wikipedia",
-                "load_wikipedia_page",
-                "answer_question",
-            ]
-            assert tool.description is not None
-            assert tool.parameters is not None
-            assert tool.strict is True
-        assert tool_params.parallel_tool_calls
+        # Test individual tool param fields
+        assert inference.allowed_tools == ["think", "search_wikipedia", "load_wikipedia_page", "answer_question"]
+        assert inference.additional_tools is None
+        assert inference.parallel_tool_calls is True
+        assert inference.provider_tools is None
 
 
 def test_demonstration_output_source(embedded_sync_client: TensorZeroGateway):
@@ -347,9 +339,8 @@ def test_not_filter(embedded_sync_client: TensorZeroGateway):
 
 def test_simple_time_filter(embedded_sync_client: TensorZeroGateway):
     filters = TimeFilter(
-        time=datetime.fromtimestamp(
-            1672531200, tz=timezone.utc
-        ).isoformat(),  # 2023-01-01 00:00:00 UTC
+        # 2023-01-01 00:00:00 UTC
+        time=datetime.fromtimestamp(1672531200, tz=timezone.utc).isoformat(),
         comparison_operator=">",
     )
     order_by = [
@@ -379,9 +370,9 @@ def test_simple_time_filter(embedded_sync_client: TensorZeroGateway):
     # Verify timestamps are in ascending order (secondary sort)
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] <= timestamps[i + 1], (
-            f"Timestamps not in ascending order: {timestamps[i]} > {timestamps[i + 1]}"
-        )
+        assert timestamps[i] <= timestamps[i + 1], f"Timestamps not in ascending order: {timestamps[i]} > {
+            timestamps[i + 1]
+        }"
 
 
 def test_simple_tag_filter(embedded_sync_client: TensorZeroGateway):
@@ -505,16 +496,14 @@ async def test_simple_list_json_inferences_async(
         episode_id = inference.episode_id
         assert isinstance(episode_id, UUID)
         # StoredJsonInference has output_schema, StoredChatInference doesn't
-        assert (
-            hasattr(inference, "output_schema") and inference.output_schema is not None
-        )
+        assert hasattr(inference, "output_schema") and inference.output_schema is not None
 
     # ORDER BY timestamp DESC is applied - verify timestamps are in descending order
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] >= timestamps[i + 1], (
-            f"Timestamps not in descending order: {timestamps[i]} < {timestamps[i + 1]}"
-        )
+        assert timestamps[i] >= timestamps[i + 1], f"Timestamps not in descending order: {timestamps[i]} < {
+            timestamps[i + 1]
+        }"
 
 
 @pytest.mark.asyncio
@@ -584,17 +573,18 @@ async def test_simple_query_chat_function_async(
         assert output_0.text is not None
         assert isinstance(inference.inference_id, UUID)
         assert isinstance(inference.episode_id, UUID)
-        tp = inference.tool_params
-        assert tp is not None
-        assert tp.tools_available == []
-        assert tp.parallel_tool_calls is None
+        # Test individual tool param fields
+        assert inference.allowed_tools is None
+        assert inference.additional_tools is None
+        assert inference.parallel_tool_calls is None
+        assert inference.provider_tools is None
 
     # ORDER BY timestamp ASC is applied - verify timestamps are in ascending order
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] <= timestamps[i + 1], (
-            f"Timestamps not in ascending order: {timestamps[i]} > {timestamps[i + 1]}"
-        )
+        assert timestamps[i] <= timestamps[i + 1], f"Timestamps not in ascending order: {timestamps[i]} > {
+            timestamps[i + 1]
+        }"
 
 
 @pytest.mark.asyncio
@@ -733,9 +723,8 @@ async def test_simple_time_filter_async(
     embedded_async_client: AsyncTensorZeroGateway,
 ):
     filters = TimeFilter(
-        time=datetime.fromtimestamp(
-            1672531200, tz=timezone.utc
-        ).isoformat(),  # 2023-01-01 00:00:00 UTC
+        # 2023-01-01 00:00:00 UTC
+        time=datetime.fromtimestamp(1672531200, tz=timezone.utc).isoformat(),
         comparison_operator=">",
     )
     order_by = [
@@ -765,9 +754,9 @@ async def test_simple_time_filter_async(
     # Verify timestamps are in ascending order (secondary sort)
     timestamps = [inference.timestamp for inference in inferences]
     for i in range(len(timestamps) - 1):
-        assert timestamps[i] <= timestamps[i + 1], (
-            f"Timestamps not in ascending order: {timestamps[i]} > {timestamps[i + 1]}"
-        )
+        assert timestamps[i] <= timestamps[i + 1], f"Timestamps not in ascending order: {timestamps[i]} > {
+            timestamps[i + 1]
+        }"
 
 
 @pytest.mark.asyncio
