@@ -11,7 +11,6 @@ from tensorzero import (
     Thought,
     Tool,
     ToolCall,
-    ToolParams,
     ToolResult,
     UnknownContentBlock,
 )
@@ -77,18 +76,16 @@ def test_sync_render_samples_success(embedded_sync_client: TensorZeroGateway):
                 output=[Text(text="Hello world")],
                 episode_id=uuid7(),
                 inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[
-                        Tool(
-                            name="test",
-                            description="test",
-                            parameters={"foo": "bar"},
-                            strict=False,
-                        )
-                    ],
-                    tool_choice="auto",
-                    parallel_tool_calls=False,
-                ),
+                additional_tools=[
+                    Tool(
+                        name="test",
+                        description="test",
+                        parameters={"foo": "bar"},
+                        strict=False,
+                    )
+                ],
+                tool_choice="auto",
+                parallel_tool_calls=False,
                 output_schema=None,
                 dispreferred_outputs=[[Text(text="goodbye")]],
                 tags={},
@@ -103,27 +100,18 @@ def test_sync_render_samples_success(embedded_sync_client: TensorZeroGateway):
                     "messages": [
                         {
                             "role": "user",
-                            "content": [
-                                {"type": "text", "value": {"country": "Japan"}}
-                            ],
+                            "content": [{"type": "text", "value": {"country": "Japan"}}],
                         },
                     ],
                 },
-                output=JsonInferenceOutput(
-                    parsed={"answer": "Tokyo"}, raw='{"answer": "Tokyo"}'
-                ),
+                output=JsonInferenceOutput(parsed={"answer": "Tokyo"}, raw='{"answer": "Tokyo"}'),
                 episode_id=uuid7(),
                 inference_id=uuid7(),
                 output_schema={
                     "type": "object",
                     "properties": {"answer": {"type": "string"}},
                 },
-                tool_params=None,
-                dispreferred_outputs=[
-                    JsonInferenceOutput(
-                        parsed={"answer": "Kyoto"}, raw='{"answer": "Kyoto"}'
-                    )
-                ],
+                dispreferred_outputs=[JsonInferenceOutput(parsed={"answer": "Kyoto"}, raw='{"answer": "Kyoto"}')],
                 tags={},
                 timestamp=datetime.now(timezone.utc).isoformat(),
             ),
@@ -193,19 +181,17 @@ def test_sync_render_samples_success(embedded_sync_client: TensorZeroGateway):
     assert isinstance(output[0], Text)
     assert output[0].type == "text"
     assert output[0].text == "Hello world"
-    tool_params = rendered_samples[0].tool_params
-    assert tool_params is not None
-    tools_available = tool_params.tools_available
-    assert len(tools_available) == 1
-    tool = tools_available[0]
+    # Test individual tool param fields
+    assert rendered_samples[0].additional_tools is not None
+    assert len(rendered_samples[0].additional_tools) == 1
+    tool = rendered_samples[0].additional_tools[0]
     assert tool.name == "test"
     assert tool.description == "test"
     assert tool.parameters == {"foo": "bar"}
     assert not tool.strict
-    # Not implemented yet
-    # TODO: test this
-    # assert tool_params.tool_choice == "auto"
-    assert not tool_params.parallel_tool_calls
+    assert rendered_samples[0].allowed_tools is None
+    assert rendered_samples[0].parallel_tool_calls is False
+    assert rendered_samples[0].provider_tools is None
     json_inference = rendered_samples[1]
     assert json_inference.function_name == "json_success"
     assert json_inference.episode_id is not None
@@ -241,7 +227,11 @@ Example Response:
         "type": "object",
         "properties": {"answer": {"type": "string"}},
     }
-    assert json_inference.tool_params is None
+    # JSON inferences don't have tool params
+    assert json_inference.allowed_tools is None
+    assert json_inference.additional_tools is None
+    assert json_inference.parallel_tool_calls is None
+    assert json_inference.provider_tools is None
     assert json_inference.output_schema == {
         "type": "object",
         "properties": {"answer": {"type": "string"}},
@@ -252,79 +242,73 @@ Example Response:
 def test_sync_render_samples_nonexistent_function(
     embedded_sync_client: TensorZeroGateway,
 ):
-    """Test that render_samples drops if the function does not exist at all."""
-    rendered_samples = embedded_sync_client.experimental_render_samples(
-        stored_samples=[
-            StoredInference(
-                type="chat",
-                function_name="non_existent_function",
-                variant_name="default",
-                input={
-                    "system": {"assistant_name": "foo"},
-                    "messages": [
-                        {
-                            "role": "user",
-                            "content": [{"type": "text", "value": "bar"}],
-                        }
-                    ],
-                },
-                output=[Text(text="Hello world")],
-                episode_id=uuid7(),
-                inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
+    """Test that render_samples throws if the function does not exist at all."""
+    with pytest.raises(Exception) as excinfo:
+        embedded_sync_client.experimental_render_samples(
+            stored_samples=[
+                StoredInference(
+                    type="chat",
+                    function_name="non_existent_function",
+                    variant_name="default",
+                    input={
+                        "system": {"assistant_name": "foo"},
+                        "messages": [
+                            {
+                                "role": "user",
+                                "content": [{"type": "text", "value": "bar"}],
+                            }
+                        ],
+                    },
+                    output=[Text(text="Hello world")],
+                    episode_id=uuid7(),
+                    inference_id=uuid7(),
                     tool_choice="auto",
                     parallel_tool_calls=False,
-                ),
-                output_schema=None,
-                dispreferred_outputs=[],
-                tags={},
-                timestamp=datetime.now(timezone.utc).isoformat(),
-            )
-        ],
-        variants={},
-    )
-    # TODO: test that the warning message is logged (we do this in Rust)
-    assert len(rendered_samples) == 0
+                    output_schema=None,
+                    dispreferred_outputs=[],
+                    tags={},
+                    timestamp=datetime.now(timezone.utc).isoformat(),
+                )
+            ],
+            variants={},
+        )
+    assert "Unknown function: non_existent_function" in str(excinfo.value)
 
 
 def test_sync_render_samples_unspecified_function(
     embedded_sync_client: TensorZeroGateway,
 ):
-    """Test that render_samples drops if the function is not specified in the variants map."""
-    rendered_samples = embedded_sync_client.experimental_render_samples(
-        stored_samples=[
-            StoredInference(
-                type="chat",
-                function_name="non_existent_function",
-                variant_name="default",
-                input={
-                    "system": {"assistant_name": "foo"},
-                    "messages": [
-                        {
-                            "role": "user",
-                            "content": [{"type": "text", "value": "bar"}],
-                        }
-                    ],
-                },
-                output=[Text(text="Hello world")],
-                episode_id=uuid7(),
-                inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
+    """Test that render_samples throws if the function is not specified in the variants map."""
+    with pytest.raises(Exception) as excinfo:
+        embedded_sync_client.experimental_render_samples(
+            stored_samples=[
+                StoredInference(
+                    type="chat",
+                    function_name="non_existent_function",
+                    variant_name="default",
+                    input={
+                        "system": {"assistant_name": "foo"},
+                        "messages": [
+                            {
+                                "role": "user",
+                                "content": [{"type": "text", "value": "bar"}],
+                            }
+                        ],
+                    },
+                    output=[Text(text="Hello world")],
+                    episode_id=uuid7(),
+                    inference_id=uuid7(),
                     tool_choice="auto",
                     parallel_tool_calls=False,
-                ),
-                output_schema=None,
-                dispreferred_outputs=[],
-                tags={},
-                timestamp=datetime.now(timezone.utc).isoformat(),
-            )
-        ],
-        variants={},
-    )
-    assert len(rendered_samples) == 0
-    # TODO: test that the warning message is logged (we do this in Rust)
+                    output_schema=None,
+                    dispreferred_outputs=[],
+                    tags={},
+                    timestamp=datetime.now(timezone.utc).isoformat(),
+                )
+            ],
+            variants={},
+        )
+    assert "Unknown function: non_existent_function" in str(excinfo.value)
 
 
 def test_sync_render_samples_no_variant(embedded_sync_client: TensorZeroGateway):
@@ -348,11 +332,8 @@ def test_sync_render_samples_no_variant(embedded_sync_client: TensorZeroGateway)
                     output=[Text(text="Hello world")],
                     episode_id=uuid7(),
                     inference_id=uuid7(),
-                    tool_params=ToolParams(
-                        tools_available=[],
-                        tool_choice="auto",
-                        parallel_tool_calls=False,
-                    ),
+                    tool_choice="auto",
+                    parallel_tool_calls=False,
                     output_schema=None,
                     dispreferred_outputs=[],
                     tags={},
@@ -361,9 +342,7 @@ def test_sync_render_samples_no_variant(embedded_sync_client: TensorZeroGateway)
             ],
             variants={"basic_test": "non_existent_variant"},
         )
-    assert "Variant non_existent_variant for function basic_test not found" in str(
-        excinfo.value
-    )
+    assert "Variant non_existent_variant for function basic_test not found" in str(excinfo.value)
 
 
 def test_sync_render_samples_missing_variable(
@@ -377,7 +356,8 @@ def test_sync_render_samples_missing_variable(
                 function_name="basic_test",  # Uses assistant_name in system prompt
                 variant_name="default",
                 input={
-                    "system": {"some_other_variable": "foo"},  # Missing assistant_name
+                    # Missing assistant_name
+                    "system": {"some_other_variable": "foo"},
                     "messages": [
                         {
                             "role": "user",
@@ -388,11 +368,8 @@ def test_sync_render_samples_missing_variable(
                 output=[Text(text="Hello world")],
                 episode_id=uuid7(),
                 inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
-                    tool_choice="auto",
-                    parallel_tool_calls=False,
-                ),
+                tool_choice="auto",
+                parallel_tool_calls=False,
                 output_schema=None,
                 dispreferred_outputs=[],
                 tags={},
@@ -469,18 +446,16 @@ async def test_async_render_samples_success(
                 output=[Text(text="Hello world")],
                 episode_id=uuid7(),
                 inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[
-                        Tool(
-                            name="test",
-                            description="test",
-                            parameters={"foo": "bar"},
-                            strict=False,
-                        )
-                    ],
-                    tool_choice="auto",
-                    parallel_tool_calls=False,
-                ),
+                additional_tools=[
+                    Tool(
+                        name="test",
+                        description="test",
+                        parameters={"foo": "bar"},
+                        strict=False,
+                    )
+                ],
+                tool_choice="auto",
+                parallel_tool_calls=False,
                 output_schema=None,
                 dispreferred_outputs=[],
                 tags={},
@@ -495,9 +470,7 @@ async def test_async_render_samples_success(
                     "messages": [
                         {
                             "role": "user",
-                            "content": [
-                                {"type": "text", "value": {"country": "Japan"}}
-                            ],
+                            "content": [{"type": "text", "value": {"country": "Japan"}}],
                         }
                     ],
                 },
@@ -511,7 +484,6 @@ async def test_async_render_samples_success(
                     "type": "object",
                     "properties": {"answer": {"type": "string"}},
                 },
-                tool_params=None,
                 dispreferred_outputs=[],
                 tags={},
                 timestamp=datetime.now(timezone.utc).isoformat(),
@@ -578,19 +550,17 @@ async def test_async_render_samples_success(
     assert output[0].type == "text"
     assert isinstance(output[0], Text)
     assert output[0].text == "Hello world"
-    tool_params = rendered_samples[0].tool_params
-    assert tool_params is not None
-    tools_available = tool_params.tools_available
-    assert len(tools_available) == 1
-    tool = tools_available[0]
+    # Test individual tool param fields
+    assert rendered_samples[0].additional_tools is not None
+    assert len(rendered_samples[0].additional_tools) == 1
+    tool = rendered_samples[0].additional_tools[0]
     assert tool.name == "test"
     assert tool.description == "test"
     assert tool.parameters == {"foo": "bar"}
     assert not tool.strict
-    # Not implemented yet
-    # TODO: test this
-    # assert tool_params.tool_choice == "auto"
-    assert not tool_params.parallel_tool_calls
+    assert rendered_samples[0].allowed_tools is None
+    assert rendered_samples[0].parallel_tool_calls is False
+    assert rendered_samples[0].provider_tools is None
     assert rendered_samples[0].output_schema is None
 
     json_inference = rendered_samples[1]
@@ -628,7 +598,11 @@ Example Response:
         "type": "object",
         "properties": {"answer": {"type": "string"}},
     }
-    assert json_inference.tool_params is None
+    # JSON inferences don't have tool params
+    assert json_inference.allowed_tools is None
+    assert json_inference.additional_tools is None
+    assert json_inference.parallel_tool_calls is None
+    assert json_inference.provider_tools is None
     assert json_inference.output_schema == {
         "type": "object",
         "properties": {"answer": {"type": "string"}},
@@ -639,80 +613,74 @@ Example Response:
 async def test_async_render_samples_nonexistent_function(
     embedded_async_client: AsyncTensorZeroGateway,
 ):
-    """Test that render_samples drops if the function does not exist at all."""
-    rendered_samples = await embedded_async_client.experimental_render_samples(
-        stored_samples=[
-            StoredInference(
-                type="chat",
-                function_name="non_existent_function",
-                variant_name="default",
-                input={
-                    "system": {"assistant_name": "foo"},
-                    "messages": [
-                        {
-                            "role": "user",
-                            "content": [{"type": "text", "value": "bar"}],
-                        }
-                    ],
-                },
-                output=[Text(text="Hello world")],
-                episode_id=uuid7(),
-                inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
+    """Test that render_samples throws if the function does not exist at all."""
+    with pytest.raises(Exception) as excinfo:
+        await embedded_async_client.experimental_render_samples(
+            stored_samples=[
+                StoredInference(
+                    type="chat",
+                    function_name="non_existent_function",
+                    variant_name="default",
+                    input={
+                        "system": {"assistant_name": "foo"},
+                        "messages": [
+                            {
+                                "role": "user",
+                                "content": [{"type": "text", "value": "bar"}],
+                            }
+                        ],
+                    },
+                    output=[Text(text="Hello world")],
+                    episode_id=uuid7(),
+                    inference_id=uuid7(),
                     tool_choice="auto",
                     parallel_tool_calls=False,
-                ),
-                output_schema=None,
-                dispreferred_outputs=[],
-                tags={},
-                timestamp=datetime.now(timezone.utc).isoformat(),
-            )
-        ],
-        variants={},
-    )
-    assert len(rendered_samples) == 0
-    # TODO: test that the warning message is logged (we do this in Rust)
+                    output_schema=None,
+                    dispreferred_outputs=[],
+                    tags={},
+                    timestamp=datetime.now(timezone.utc).isoformat(),
+                )
+            ],
+            variants={},
+        )
+    assert "Unknown function: non_existent_function" in str(excinfo.value)
 
 
 @pytest.mark.asyncio
 async def test_async_render_samples_unspecified_function(
     embedded_async_client: AsyncTensorZeroGateway,
 ):
-    """Test that render_samples drops if the function is not specified in the variants map."""
-    rendered_samples = await embedded_async_client.experimental_render_samples(
-        stored_samples=[
-            StoredInference(
-                type="chat",
-                function_name="non_existent_function",
-                variant_name="default",
-                input={
-                    "system": {"assistant_name": "foo"},
-                    "messages": [
-                        {
-                            "role": "user",
-                            "content": [{"type": "text", "value": "bar"}],
-                        }
-                    ],
-                },
-                output=[Text(text="Hello world")],
-                episode_id=uuid7(),
-                inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
+    """Test that render_samples throws if the function is not specified in the variants map."""
+    with pytest.raises(Exception) as excinfo:
+        await embedded_async_client.experimental_render_samples(
+            stored_samples=[
+                StoredInference(
+                    type="chat",
+                    function_name="non_existent_function",
+                    variant_name="default",
+                    input={
+                        "system": {"assistant_name": "foo"},
+                        "messages": [
+                            {
+                                "role": "user",
+                                "content": [{"type": "text", "value": "bar"}],
+                            }
+                        ],
+                    },
+                    output=[Text(text="Hello world")],
+                    episode_id=uuid7(),
+                    inference_id=uuid7(),
                     tool_choice="auto",
                     parallel_tool_calls=False,
-                ),
-                output_schema=None,
-                dispreferred_outputs=[],
-                tags={},
-                timestamp=datetime.now(timezone.utc).isoformat(),
-            )
-        ],
-        variants={},
-    )
-    assert len(rendered_samples) == 0
-    # TODO: test that the warning message is logged (we do this in Rust)
+                    output_schema=None,
+                    dispreferred_outputs=[],
+                    tags={},
+                    timestamp=datetime.now(timezone.utc).isoformat(),
+                )
+            ],
+            variants={},
+        )
+    assert "Unknown function: non_existent_function" in str(excinfo.value)
 
 
 @pytest.mark.asyncio
@@ -739,11 +707,8 @@ async def test_async_render_samples_no_variant(
                     output=[Text(text="Hello world")],
                     episode_id=uuid7(),
                     inference_id=uuid7(),
-                    tool_params=ToolParams(
-                        tools_available=[],
-                        tool_choice="auto",
-                        parallel_tool_calls=False,
-                    ),
+                    tool_choice="auto",
+                    parallel_tool_calls=False,
                     output_schema=None,
                     dispreferred_outputs=[],
                     tags={},
@@ -752,9 +717,7 @@ async def test_async_render_samples_no_variant(
             ],
             variants={"basic_test": "non_existent_variant"},
         )
-    assert "Variant non_existent_variant for function basic_test not found" in str(
-        excinfo.value
-    )
+    assert "Variant non_existent_variant for function basic_test not found" in str(excinfo.value)
 
 
 @pytest.mark.asyncio
@@ -769,7 +732,8 @@ async def test_async_render_samples_missing_variable(
                 function_name="basic_test",  # Uses assistant_name in system prompt
                 variant_name="default",
                 input={
-                    "system": {"some_other_variable": "foo"},  # Missing assistant_name
+                    # Missing assistant_name
+                    "system": {"some_other_variable": "foo"},
                     "messages": [
                         {
                             "role": "user",
@@ -780,11 +744,8 @@ async def test_async_render_samples_missing_variable(
                 output=[Text(text="Hello world")],
                 episode_id=uuid7(),
                 inference_id=uuid7(),
-                tool_params=ToolParams(
-                    tools_available=[],
-                    tool_choice="auto",
-                    parallel_tool_calls=False,
-                ),
+                tool_choice="auto",
+                parallel_tool_calls=False,
                 output_schema=None,
                 dispreferred_outputs=[],
                 tags={},
