@@ -4,7 +4,7 @@ use std::sync::Arc;
 
 use crate::config::OtlpConfig;
 use crate::db::clickhouse::{ClickHouseConnectionInfo, TableName};
-use crate::embeddings::{EmbeddingModelResponse, EmbeddingRequest};
+use crate::embeddings::{Embedding, EmbeddingModelResponse, EmbeddingRequest};
 use crate::error::{warn_discarded_cache_write, Error, ErrorDetails};
 use crate::inference::types::file::serialize_with_file_data;
 use crate::inference::types::{
@@ -12,7 +12,7 @@ use crate::inference::types::{
     ModelInferenceResponse, ProviderInferenceResponseChunk, Usage,
 };
 use crate::model::StreamResponse;
-use crate::serde_util::deserialize_json_string;
+use crate::serde_util::{deserialize_json_string, serialize_json_string};
 use crate::tool::{ToolCallConfig, ToolCallOutput};
 use blake3::Hash;
 use clap::ValueEnum;
@@ -70,7 +70,7 @@ impl From<(CacheParamsOptions, bool)> for CacheOptions {
     }
 }
 
-#[derive(Debug, Default)]
+#[derive(Debug, Default, Clone)]
 pub struct CacheOptions {
     pub max_age_s: Option<u32>,
     pub enabled: CacheEnabledMode,
@@ -271,11 +271,21 @@ impl CacheOutput for EmbeddingCacheData {
     }
 }
 
+/// Cache data for embeddings.
+///
+/// Note: Unlike `NonStreamingCacheData` and `StreamingCacheData`, this requires both `serialize_with` and
+/// `deserialize_with` because `Embedding` is an untagged enum. Without `untagged`, OpenAI's API responses wouldn't
+/// deserialize correctly (they send bare arrays/strings). But with `untagged`, the enum serializes as bare JSON
+/// values ([1.0, 2.0] or "abc"), which breaks `deserialize_json_string` (which expects a JSON-encoded string). So we
+/// need `serialize_json_string` to ensure the data is stored in the format that `deserialize_json_string` expects.
 #[derive(Debug, Deserialize, Serialize)]
 #[serde(transparent)]
 pub struct EmbeddingCacheData {
-    #[serde(deserialize_with = "deserialize_json_string")]
-    pub embedding: Vec<f32>,
+    #[serde(
+        serialize_with = "serialize_json_string",
+        deserialize_with = "deserialize_json_string"
+    )]
+    pub embedding: Embedding,
 }
 
 #[derive(Debug, Deserialize, Serialize)]
@@ -297,6 +307,8 @@ fn spawn_maybe_cache_write<T: Serialize + CacheOutput + Send + Sync + 'static>(
     clickhouse_client: ClickHouseConnectionInfo,
     cache_validation_info: CacheValidationInfo,
 ) {
+    // TODO(https://github.com/tensorzero/tensorzero/issues/3983): Audit this callsite
+    #[expect(clippy::disallowed_methods)]
     tokio::spawn(async move {
         if row
             .data
@@ -555,8 +567,10 @@ mod tests {
             output_schema: None,
             extra_body: Default::default(),
             extra_headers: Default::default(),
+            fetch_and_encode_input_files_before_inference: false,
             extra_cache_key: None,
             stop_sequences: None,
+            ..Default::default()
         };
         let model_provider_request = ModelProviderRequest {
             request: &model_inference_request,
@@ -580,10 +594,12 @@ mod tests {
             json_mode: ModelInferenceRequestJsonMode::Off,
             function_type: FunctionType::Chat,
             output_schema: None,
+            fetch_and_encode_input_files_before_inference: false,
             extra_body: Default::default(),
             extra_headers: Default::default(),
             extra_cache_key: None,
             stop_sequences: None,
+            ..Default::default()
         };
         let model_provider_request = ModelProviderRequest {
             request: &model_inference_request,
@@ -609,10 +625,12 @@ mod tests {
             json_mode: ModelInferenceRequestJsonMode::Off,
             function_type: FunctionType::Chat,
             output_schema: None,
+            fetch_and_encode_input_files_before_inference: false,
             extra_body: Default::default(),
             extra_headers: Default::default(),
             extra_cache_key: None,
             stop_sequences: None,
+            ..Default::default()
         };
         let model_provider_request = ModelProviderRequest {
             request: &streaming_model_inference_request,
