@@ -1,26 +1,60 @@
 use serde::{Deserialize, Serialize};
 
 use crate::{
-    config::{
-        ExportConfig, ObservabilityConfig, TemplateFilesystemAccess,
-        UninitializedObservabilityConfig,
-    },
-    error::{Error, ErrorDetails},
+    config::{ExportConfig, ObservabilityConfig, TemplateFilesystemAccess},
+    error::Error,
+    inference::types::storage::StorageKind,
 };
 
+use super::ObjectStoreInfo;
+
+#[derive(Debug, Deserialize, Serialize)]
+#[cfg_attr(test, derive(ts_rs::TS))]
+#[cfg_attr(test, ts(export))]
+#[serde(deny_unknown_fields)]
+pub struct GatewayAuthCacheConfig {
+    #[serde(default = "default_gateway_auth_cache_enabled")]
+    pub enabled: bool,
+    #[serde(default = "default_gateway_auth_cache_ttl_ms")]
+    pub ttl_ms: u64,
+}
+
+impl Default for GatewayAuthCacheConfig {
+    fn default() -> Self {
+        Self {
+            enabled: default_gateway_auth_cache_enabled(),
+            ttl_ms: default_gateway_auth_cache_ttl_ms(),
+        }
+    }
+}
+
+fn default_gateway_auth_cache_enabled() -> bool {
+    true
+}
+
+fn default_gateway_auth_cache_ttl_ms() -> u64 {
+    1000
+}
+
+#[derive(Debug, Default, Deserialize, Serialize)]
+#[cfg_attr(test, derive(ts_rs::TS))]
+#[cfg_attr(test, ts(export))]
+#[serde(deny_unknown_fields)]
+pub struct AuthConfig {
+    pub enabled: bool,
+    #[serde(default)]
+    pub cache: Option<GatewayAuthCacheConfig>,
+}
+
 #[derive(Debug, Default, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
 pub struct UninitializedGatewayConfig {
     #[serde(serialize_with = "serialize_optional_socket_addr")]
     pub bind_address: Option<std::net::SocketAddr>,
     #[serde(default)]
-    pub observability: UninitializedObservabilityConfig,
+    pub observability: ObservabilityConfig,
     #[serde(default)]
     pub debug: bool,
-    /// If `true`, allow minijinja to read from the filesystem (within the tree of the config file) for '{% include %}'
-    /// Defaults to `false`
-    #[serde(default)]
-    pub enable_template_filesystem_access: Option<bool>,
     #[serde(default)]
     pub template_filesystem_access: Option<TemplateFilesystemAccess>,
     #[serde(default)]
@@ -40,45 +74,43 @@ pub struct UninitializedGatewayConfig {
     pub unstable_error_json: bool,
     #[serde(default)]
     pub disable_pseudonymous_usage_analytics: bool,
+    pub fetch_and_encode_input_files_before_inference: Option<bool>,
+    #[serde(default)]
+    pub auth: AuthConfig,
 }
 
 impl UninitializedGatewayConfig {
-    pub fn load(self) -> Result<GatewayConfig, Error> {
-        if self.enable_template_filesystem_access.is_some() {
-            tracing::warn!("Deprecation Warning: `gateway.enable_template_filesystem_access` is deprecated. Please use `[gateway.template_filesystem_access.enabled]` instead.");
-        }
-        let template_filesystem_access = match (
-            self.enable_template_filesystem_access,
-            self.template_filesystem_access,
-        ) {
-            (Some(enabled), None) => TemplateFilesystemAccess {
-                enabled,
-                base_path: None,
-            },
-            (None, Some(template_filesystem_access)) => template_filesystem_access,
-            (None, None) => Default::default(),
-            (Some(_), Some(_)) => {
-                return Err(Error::new(ErrorDetails::Config {
-                    message: "`gateway.enable_template_filesystem_access` and `gateway.template_filesystem_access` cannot both be set".to_string(),
-                }));
+    pub fn load(self, object_store_info: Option<&ObjectStoreInfo>) -> Result<GatewayConfig, Error> {
+        let fetch_and_encode_input_files_before_inference = if let Some(value) =
+            self.fetch_and_encode_input_files_before_inference
+        {
+            value
+        } else {
+            if let Some(info) = object_store_info {
+                if !matches!(info.kind, StorageKind::Disabled) {
+                    tracing::info!("Object store is enabled but `gateway.fetch_and_encode_input_files_before_inference` is unset (defaults to `false`). In rare cases, the files we fetch for object storage may differ from the image the inference provider fetched if this setting is disabled.");
+                }
             }
+            default_fetch_and_encode_input_files_before_inference()
         };
         Ok(GatewayConfig {
             bind_address: self.bind_address,
-            observability: self.observability.load(),
+            observability: self.observability,
             debug: self.debug,
-            template_filesystem_access,
+            template_filesystem_access: self.template_filesystem_access.unwrap_or_default(),
             export: self.export,
             base_path: self.base_path,
             unstable_error_json: self.unstable_error_json,
             unstable_disable_feedback_target_validation: self
                 .unstable_disable_feedback_target_validation,
             disable_pseudonymous_usage_analytics: self.disable_pseudonymous_usage_analytics,
+            fetch_and_encode_input_files_before_inference,
+            auth: self.auth,
         })
     }
 }
 
-#[derive(Debug, Default, Serialize)]
+#[derive(Debug, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
 pub struct GatewayConfig {
@@ -94,6 +126,32 @@ pub struct GatewayConfig {
     pub unstable_disable_feedback_target_validation: bool,
     #[serde(default)]
     pub disable_pseudonymous_usage_analytics: bool,
+    #[serde(default = "default_fetch_and_encode_input_files_before_inference")]
+    pub fetch_and_encode_input_files_before_inference: bool,
+    pub auth: AuthConfig,
+}
+
+impl Default for GatewayConfig {
+    fn default() -> Self {
+        Self {
+            bind_address: Default::default(),
+            observability: Default::default(),
+            debug: Default::default(),
+            template_filesystem_access: Default::default(),
+            export: Default::default(),
+            base_path: Default::default(),
+            unstable_error_json: Default::default(),
+            unstable_disable_feedback_target_validation: Default::default(),
+            disable_pseudonymous_usage_analytics: Default::default(),
+            fetch_and_encode_input_files_before_inference:
+                default_fetch_and_encode_input_files_before_inference(),
+            auth: Default::default(),
+        }
+    }
+}
+
+fn default_fetch_and_encode_input_files_before_inference() -> bool {
+    false
 }
 
 fn serialize_optional_socket_addr<S>(
