@@ -6,19 +6,19 @@ use crate::cache::{
     embedding_cache_lookup, start_cache_write, CacheData, CacheValidationInfo, EmbeddingCacheData,
     EmbeddingModelProviderRequest,
 };
-use crate::config::{ProviderTypesConfig, TimeoutsConfig};
+use crate::config::{provider_types::ProviderTypesConfig, TimeoutsConfig};
 use crate::endpoints::inference::InferenceClients;
 use crate::http::TensorzeroHttpClient;
 use crate::inference::types::extra_body::ExtraBodyConfig;
 use crate::inference::types::RequestMessagesOrBatch;
 use crate::inference::types::{ContentBlock, Text};
 use crate::model::{ModelProviderRequestInfo, UninitializedProviderConfig};
-use crate::model_table::BaseModelTable;
-use crate::model_table::ShorthandModelConfig;
+use crate::model_table::{BaseModelTable, ProviderKind, ProviderTypeDefaultCredentials};
+use crate::model_table::{OpenAIKind, ShorthandModelConfig};
 use crate::providers::azure::AzureProvider;
 use crate::rate_limiting::{
-    get_estimated_tokens, RateLimitResourceUsage, RateLimitedInputContent, RateLimitedRequest,
-    RateLimitedResponse, ScopeInfo,
+    get_estimated_tokens, EstimatedRateLimitResourceUsage, RateLimitResource,
+    RateLimitResourceUsage, RateLimitedInputContent, RateLimitedRequest, RateLimitedResponse,
 };
 use crate::{
     endpoints::inference::InferenceCredentials,
@@ -27,7 +27,7 @@ use crate::{
         current_timestamp, Latency, ModelInferenceResponseWithMetadata, RequestMessage, Role, Usage,
     },
     model::ProviderConfig,
-    providers::openai::OpenAIProvider,
+    providers::openai::{OpenAIAPIType, OpenAIProvider},
 };
 use futures::future::try_join_all;
 use serde::{Deserialize, Serialize};
@@ -44,12 +44,24 @@ pub type EmbeddingModelTable = BaseModelTable<EmbeddingModelConfig>;
 impl ShorthandModelConfig for EmbeddingModelConfig {
     const SHORTHAND_MODEL_PREFIXES: &[&str] = &["openai::"];
     const MODEL_TYPE: &str = "Embedding model";
-    async fn from_shorthand(provider_type: &str, model_name: &str) -> Result<Self, Error> {
+    async fn from_shorthand(
+        provider_type: &str,
+        model_name: &str,
+        default_credentials: &ProviderTypeDefaultCredentials,
+    ) -> Result<Self, Error> {
         let model_name = model_name.to_string();
         let provider_config = match provider_type {
-            "openai" => {
-                EmbeddingProviderConfig::OpenAI(OpenAIProvider::new(model_name, None, None)?)
-            }
+            "openai" => EmbeddingProviderConfig::OpenAI(OpenAIProvider::new(
+                model_name,
+                None,
+                OpenAIKind
+                    .get_defaulted_credential(None, default_credentials)
+                    .await?,
+                // TODO: handle the fact that there are also embeddings
+                OpenAIAPIType::ChatCompletions,
+                false,
+                Vec::new(),
+            )?),
             #[cfg(any(test, feature = "e2e_tests"))]
             "dummy" => EmbeddingProviderConfig::Dummy(DummyProvider::new(model_name, None)?),
             _ => {
@@ -60,14 +72,14 @@ impl ShorthandModelConfig for EmbeddingModelConfig {
         };
         let provider_info = EmbeddingProviderInfo {
             inner: provider_config,
-            timeouts: TimeoutsConfig::default(),
+            timeout_ms: None,
             provider_name: Arc::from(provider_type.to_string()),
             extra_body: Default::default(),
         };
         Ok(EmbeddingModelConfig {
             routing: vec![provider_type.to_string().into()],
             providers: HashMap::from([(provider_type.to_string().into(), provider_info)]),
-            timeouts: TimeoutsConfig::default(),
+            timeout_ms: None,
         })
     }
 
@@ -84,6 +96,8 @@ pub struct UninitializedEmbeddingModelConfig {
     pub routing: Vec<Arc<str>>,
     pub providers: HashMap<Arc<str>, UninitializedEmbeddingProviderConfig>,
     #[serde(default)]
+    pub timeout_ms: Option<u64>,
+    #[serde(default)]
     pub timeouts: TimeoutsConfig,
 }
 
@@ -91,9 +105,31 @@ impl UninitializedEmbeddingModelConfig {
     pub async fn load(
         self,
         provider_types: &ProviderTypesConfig,
+        default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<EmbeddingModelConfig, Error> {
+        // Handle timeout deprecation
+        let timeout_ms = match (self.timeout_ms, self.timeouts.non_streaming.total_ms) {
+            (Some(timeout_ms), None) => Some(timeout_ms),
+            (None, Some(old_timeout)) => {
+                tracing::warn!(
+                    "Deprecation Warning: `timeouts` is deprecated for embedding models. \
+                    Please use `timeout_ms` instead."
+                );
+                Some(old_timeout)
+            }
+            (None, None) => None,
+            (Some(_), Some(_)) => {
+                return Err(Error::new(ErrorDetails::Config {
+                    message: "`timeout_ms` and `timeouts` cannot both be set for embedding models"
+                        .to_string(),
+                }));
+            }
+        };
+
         let providers = try_join_all(self.providers.into_iter().map(|(name, config)| async {
-            let provider_config = config.load(provider_types, name.clone()).await?;
+            let provider_config = config
+                .load(provider_types, name.clone(), default_credentials)
+                .await?;
             Ok::<_, Error>((name, provider_config))
         }))
         .await?
@@ -102,7 +138,7 @@ impl UninitializedEmbeddingModelConfig {
         Ok(EmbeddingModelConfig {
             routing: self.routing,
             providers,
-            timeouts: self.timeouts,
+            timeout_ms,
         })
     }
 }
@@ -113,7 +149,7 @@ impl UninitializedEmbeddingModelConfig {
 pub struct EmbeddingModelConfig {
     pub routing: Vec<Arc<str>>,
     pub providers: HashMap<Arc<str>, EmbeddingProviderInfo>,
-    pub timeouts: TimeoutsConfig,
+    pub timeout_ms: Option<u64>,
 }
 
 impl EmbeddingModelConfig {
@@ -122,7 +158,7 @@ impl EmbeddingModelConfig {
         &self,
         request: &EmbeddingRequest,
         model_name: &str,
-        clients: &InferenceClients<'_>,
+        clients: &InferenceClients,
     ) -> Result<EmbeddingModelResponse, Error> {
         let mut provider_errors: HashMap<String, Error> = HashMap::new();
         let run_all_embedding_models = async {
@@ -136,12 +172,12 @@ impl EmbeddingModelConfig {
                     request,
                     model_name,
                     provider_name,
-                    otlp_config: clients.otlp_config,
+                    otlp_config: &clients.otlp_config,
                 };
                 // TODO: think about how to best handle errors here
                 if clients.cache_options.enabled.read() {
                     let cache_lookup = embedding_cache_lookup(
-                        clients.clickhouse_connection_info,
+                        &clients.clickhouse_connection_info,
                         &provider_request,
                         clients.cache_options.max_age_s,
                     )
@@ -152,11 +188,8 @@ impl EmbeddingModelConfig {
                         return Ok(cache_lookup);
                     }
                 }
-                let scope_info = ScopeInfo {
-                    tags: &HashMap::default(),
-                };
                 let response = provider_config
-                    .embed(request, clients, &scope_info, &provider_config.into())
+                    .embed(request, clients, &provider_config.into())
                     .await;
 
                 match response {
@@ -168,23 +201,21 @@ impl EmbeddingModelConfig {
                             }
                             .into());
                             };
-                            if let Some(float_data) = first_embedding.as_float() {
-                                let _ = start_cache_write(
-                                    clients.clickhouse_connection_info,
-                                    provider_request.get_cache_key()?,
-                                    CacheData {
-                                        output: EmbeddingCacheData {
-                                            embedding: float_data.clone(),
-                                        },
-                                        raw_request: response.raw_request.clone(),
-                                        raw_response: response.raw_response.clone(),
-                                        input_tokens: response.usage.input_tokens,
-                                        output_tokens: response.usage.output_tokens,
-                                        finish_reason: None,
+                            let _ = start_cache_write(
+                                &clients.clickhouse_connection_info,
+                                provider_request.get_cache_key()?,
+                                CacheData {
+                                    output: EmbeddingCacheData {
+                                        embedding: first_embedding.clone(),
                                     },
-                                    CacheValidationInfo { tool_config: None },
-                                );
-                            }
+                                    raw_request: response.raw_request.clone(),
+                                    raw_response: response.raw_response.clone(),
+                                    input_tokens: response.usage.input_tokens,
+                                    output_tokens: response.usage.output_tokens,
+                                    finish_reason: None,
+                                },
+                                CacheValidationInfo { tool_config: None },
+                            );
                         };
                         let embedding_response =
                             EmbeddingModelResponse::new(response, provider_name.clone());
@@ -201,8 +232,8 @@ impl EmbeddingModelConfig {
         // Some of the providers may themselves have timeouts, which is fine. Provider timeouts
         // are treated as just another kind of provider error - a timeout of N ms is equivalent
         // to a provider taking N ms, and then producing a normal HTTP error.
-        if let Some(timeout) = self.timeouts.non_streaming.total_ms {
-            let timeout = Duration::from_millis(timeout);
+        if let Some(timeout_ms) = self.timeout_ms {
+            let timeout = Duration::from_millis(timeout_ms);
             tokio::time::timeout(timeout, run_all_embedding_models)
                 .await
                 // Convert the outer `Elapsed` error into a TensorZero error,
@@ -225,6 +256,8 @@ impl EmbeddingModelConfig {
 pub enum EmbeddingInput {
     Single(String),
     Batch(Vec<String>),
+    SingleTokens(Vec<u32>),
+    BatchTokens(Vec<Vec<u32>>),
 }
 
 impl EmbeddingInput {
@@ -232,6 +265,8 @@ impl EmbeddingInput {
         match self {
             EmbeddingInput::Single(_) => 1,
             EmbeddingInput::Batch(texts) => texts.len(),
+            EmbeddingInput::SingleTokens(_) => 1,
+            EmbeddingInput::BatchTokens(tokens) => tokens.len(),
         }
     }
 
@@ -239,6 +274,8 @@ impl EmbeddingInput {
         match self {
             EmbeddingInput::Single(text) => Some(text),
             EmbeddingInput::Batch(texts) => texts.first(),
+            EmbeddingInput::SingleTokens(_) => None,
+            EmbeddingInput::BatchTokens(_) => None,
         }
     }
 }
@@ -251,6 +288,12 @@ impl RateLimitedInputContent for EmbeddingInput {
                 .iter()
                 .map(|text| get_estimated_tokens(text))
                 .sum::<u64>(),
+            // For token arrays, we have exact counts, not estimates
+            EmbeddingInput::SingleTokens(tokens) => tokens.len() as u64,
+            EmbeddingInput::BatchTokens(token_arrays) => token_arrays
+                .iter()
+                .map(|tokens| tokens.len() as u64)
+                .sum::<u64>(),
         }
     }
 }
@@ -269,15 +312,31 @@ pub struct EmbeddingRequest {
 }
 
 impl RateLimitedRequest for EmbeddingRequest {
-    fn estimated_resource_usage(&self) -> Result<RateLimitResourceUsage, Error> {
+    fn estimated_resource_usage(
+        &self,
+        resources: &[RateLimitResource],
+    ) -> Result<EstimatedRateLimitResourceUsage, Error> {
         let EmbeddingRequest {
             input,
             dimensions: _,
             encoding_format: _,
         } = self;
-        Ok(RateLimitResourceUsage {
-            model_inferences: 1,
-            tokens: input.estimated_input_token_usage(),
+
+        let tokens = if resources.contains(&RateLimitResource::Token) {
+            Some(input.estimated_input_token_usage())
+        } else {
+            None
+        };
+
+        let model_inferences = if resources.contains(&RateLimitResource::ModelInference) {
+            Some(1)
+        } else {
+            None
+        };
+
+        Ok(EstimatedRateLimitResourceUsage {
+            model_inferences,
+            tokens,
         })
     }
 }
@@ -311,7 +370,7 @@ pub struct EmbeddingProviderResponse {
 
 impl RateLimitedResponse for EmbeddingProviderResponse {
     fn resource_usage(&self) -> RateLimitResourceUsage {
-        RateLimitResourceUsage {
+        RateLimitResourceUsage::Exact {
             model_inferences: 1,
             tokens: self.usage.total_tokens() as u64,
         }
@@ -341,7 +400,7 @@ impl EmbeddingModelResponse {
             id: Uuid::now_v7(),
             created: current_timestamp(),
             input: request.request.input.clone(),
-            embeddings: vec![Embedding::Float(cache_lookup.output.embedding)],
+            embeddings: vec![cache_lookup.output.embedding],
             raw_request: cache_lookup.raw_request,
             raw_response: cache_lookup.raw_response,
             usage: Usage {
@@ -479,7 +538,7 @@ pub enum EmbeddingProviderConfig {
 #[cfg_attr(test, ts(export))]
 pub struct EmbeddingProviderInfo {
     pub inner: EmbeddingProviderConfig,
-    pub timeouts: TimeoutsConfig,
+    pub timeout_ms: Option<u64>,
     pub provider_name: Arc<str>,
     #[cfg_attr(test, ts(skip))]
     pub extra_body: Option<ExtraBodyConfig>,
@@ -514,21 +573,24 @@ impl EmbeddingProviderInfo {
     pub async fn embed(
         &self,
         request: &EmbeddingRequest,
-        clients: &InferenceClients<'_>,
-        scope_info: &ScopeInfo<'_>,
+        clients: &InferenceClients,
         model_provider_data: &EmbeddingProviderRequestInfo,
     ) -> Result<EmbeddingProviderResponse, Error> {
         let ticket_borrow = clients
             .rate_limiting_config
-            .consume_tickets(clients.postgres_connection_info, scope_info, request)
+            .consume_tickets(
+                &clients.postgres_connection_info,
+                &clients.scope_info,
+                request,
+            )
             .await?;
         let response_fut = self.inner.embed(
             request,
-            clients.http_client,
-            clients.credentials,
+            &clients.http_client,
+            &clients.credentials,
             model_provider_data,
         );
-        let response = if let Some(timeout_ms) = self.timeouts.non_streaming.total_ms {
+        let response = if let Some(timeout_ms) = self.timeout_ms {
             let timeout = Duration::from_millis(timeout_ms);
             tokio::time::timeout(timeout, response_fut)
                 .await
@@ -544,7 +606,8 @@ impl EmbeddingProviderInfo {
         };
         let postgres_connection_info = clients.postgres_connection_info.clone();
         let resource_usage = response.resource_usage();
-        tokio::spawn(
+        // Make sure that we finish updating rate-limiting tickets if the gateway shuts down
+        clients.deferred_tasks.spawn(
             async move {
                 if let Err(e) = ticket_borrow
                     .return_tickets(&postgres_connection_info, resource_usage)
@@ -564,6 +627,8 @@ pub struct UninitializedEmbeddingProviderConfig {
     #[serde(flatten)]
     config: UninitializedProviderConfig,
     #[serde(default)]
+    pub timeout_ms: Option<u64>,
+    #[serde(default)]
     timeouts: TimeoutsConfig,
     #[serde(default)]
     pub extra_body: Option<ExtraBodyConfig>,
@@ -574,27 +639,50 @@ impl UninitializedEmbeddingProviderConfig {
         self,
         provider_types: &ProviderTypesConfig,
         provider_name: Arc<str>,
+        default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<EmbeddingProviderInfo, Error> {
-        let provider_config = self.config.load(provider_types).await?;
-        let timeouts = self.timeouts;
+        let provider_config = self
+            .config
+            .load(provider_types, default_credentials)
+            .await?;
+        // Handle timeout deprecation
+        let timeout_ms = match (self.timeout_ms, self.timeouts.non_streaming.total_ms) {
+            (Some(timeout_ms), None) => Some(timeout_ms),
+            (None, Some(old_timeout)) => {
+                tracing::warn!(
+                    "Deprecation Warning: `timeouts` is deprecated for embedding providers. \
+                    Please use `timeout_ms` instead."
+                );
+                Some(old_timeout)
+            }
+            (None, None) => None,
+            (Some(_), Some(_)) => {
+                return Err(Error::new(ErrorDetails::Config {
+                    message:
+                        "`timeout_ms` and `timeouts` cannot both be set for embedding providers"
+                            .to_string(),
+                }));
+            }
+        };
+
         let extra_body = self.extra_body;
         Ok(match provider_config {
             ProviderConfig::OpenAI(provider) => EmbeddingProviderInfo {
                 inner: EmbeddingProviderConfig::OpenAI(provider),
-                timeouts,
+                timeout_ms,
                 provider_name,
                 extra_body,
             },
             ProviderConfig::Azure(provider) => EmbeddingProviderInfo {
                 inner: EmbeddingProviderConfig::Azure(provider),
-                timeouts,
+                timeout_ms,
                 provider_name,
                 extra_body,
             },
             #[cfg(any(test, feature = "e2e_tests"))]
             ProviderConfig::Dummy(provider) => EmbeddingProviderInfo {
                 inner: EmbeddingProviderConfig::Dummy(provider),
-                timeouts,
+                timeout_ms,
                 provider_name,
                 extra_body,
             },
@@ -690,6 +778,7 @@ mod tests {
     use crate::{
         cache::{CacheEnabledMode, CacheOptions},
         db::{clickhouse::ClickHouseConnectionInfo, postgres::PostgresConnectionInfo},
+        model_table::ProviderTypeDefaultCredentials,
     };
 
     use super::*;
@@ -703,7 +792,7 @@ mod tests {
         });
         let bad_provider_info = EmbeddingProviderInfo {
             inner: bad_provider,
-            timeouts: Default::default(),
+            timeout_ms: None,
             provider_name: Arc::from("error".to_string()),
             extra_body: None,
         };
@@ -713,7 +802,7 @@ mod tests {
         });
         let good_provider_info = EmbeddingProviderInfo {
             inner: good_provider,
-            timeouts: Default::default(),
+            timeout_ms: None,
             provider_name: Arc::from("good".to_string()),
             extra_body: None,
         };
@@ -723,7 +812,7 @@ mod tests {
                 ("error".to_string().into(), bad_provider_info),
                 ("good".to_string().into(), good_provider_info),
             ]),
-            timeouts: TimeoutsConfig::default(),
+            timeout_ms: None,
         };
         let request = EmbeddingRequest {
             input: "Hello, world!".to_string().into(),
@@ -735,17 +824,21 @@ mod tests {
                 &request,
                 "fallback",
                 &InferenceClients {
-                    http_client: &TensorzeroHttpClient::new().unwrap(),
-                    clickhouse_connection_info: &ClickHouseConnectionInfo::new_disabled(),
-                    postgres_connection_info: &PostgresConnectionInfo::Disabled,
-                    credentials: &InferenceCredentials::default(),
-                    cache_options: &CacheOptions {
+                    http_client: TensorzeroHttpClient::new().unwrap(),
+                    clickhouse_connection_info: ClickHouseConnectionInfo::new_disabled(),
+                    postgres_connection_info: PostgresConnectionInfo::Disabled,
+                    credentials: Arc::new(InferenceCredentials::default()),
+                    cache_options: CacheOptions {
                         max_age_s: None,
                         enabled: CacheEnabledMode::Off,
                     },
-                    tags: &Default::default(),
-                    rate_limiting_config: &Default::default(),
-                    otlp_config: &Default::default(),
+                    tags: Arc::new(Default::default()),
+                    rate_limiting_config: Arc::new(Default::default()),
+                    otlp_config: Default::default(),
+                    deferred_tasks: tokio_util::task::TaskTracker::new(),
+                    scope_info: crate::rate_limiting::ScopeInfo {
+                        tags: Arc::new(HashMap::new()),
+                    },
                 },
             )
             .await;
@@ -771,17 +864,27 @@ mod tests {
         };
 
         let uninitialized_config = UninitializedEmbeddingProviderConfig {
-            config: crate::model::UninitializedProviderConfig::OpenAI {
+            config: UninitializedProviderConfig::OpenAI {
                 model_name: "text-embedding-ada-002".to_string(),
                 api_base: None,
-                api_key_location: Some(crate::model::CredentialLocation::None),
+                api_key_location: Some(crate::model::CredentialLocationWithFallback::Single(
+                    crate::model::CredentialLocation::None,
+                )),
+                api_type: Default::default(),
+                include_encrypted_reasoning: false,
+                provider_tools: Vec::new(),
             },
+            timeout_ms: None,
             timeouts: TimeoutsConfig::default(),
             extra_body: Some(extra_body_config.clone()),
         };
 
         let provider_info = uninitialized_config
-            .load(&ProviderTypesConfig::default(), Arc::from("test_provider"))
+            .load(
+                &ProviderTypesConfig::default(),
+                Arc::from("test_provider"),
+                &ProviderTypeDefaultCredentials::default(),
+            )
             .await
             .unwrap();
 
