@@ -1,6 +1,7 @@
 use std::{collections::HashMap, sync::Arc};
 
 use serde::Deserialize;
+use tokio_util::task::TaskTracker;
 use tracing::instrument;
 
 use crate::{
@@ -12,6 +13,7 @@ use crate::{
     error::{Error, ErrorDetails},
     http::TensorzeroHttpClient,
     inference::types::Usage,
+    rate_limiting::ScopeInfo,
 };
 
 use super::inference::InferenceCredentials;
@@ -36,6 +38,7 @@ pub async fn embeddings(
     http_client: &TensorzeroHttpClient,
     clickhouse_connection_info: ClickHouseConnectionInfo,
     postgres_connection_info: PostgresConnectionInfo,
+    deferred_tasks: TaskTracker,
     params: Params,
 ) -> Result<EmbeddingResponse, Error> {
     let span = tracing::Span::current();
@@ -63,18 +66,22 @@ pub async fn embeddings(
         dimensions: params.dimensions,
         encoding_format: params.encoding_format,
     };
+
+    // NOTE: we do not support tags for embeddings yet
+    // we should fix this once the tags are implemented
+    let tags = Arc::new(HashMap::default());
     let dryrun = params.dryrun.unwrap_or(false);
     let clients = InferenceClients {
-        http_client,
-        credentials: &params.credentials,
-        cache_options: &(params.cache_options, dryrun).into(),
-        clickhouse_connection_info: &clickhouse_connection_info,
-        postgres_connection_info: &postgres_connection_info,
-        // NOTE: we do not support tags for embeddings yet
-        // we should fix this once the tags are implemented
-        tags: &HashMap::default(),
-        rate_limiting_config: &config.rate_limiting,
-        otlp_config: &config.gateway.export.otlp,
+        http_client: http_client.clone(),
+        credentials: Arc::new(params.credentials.clone()),
+        cache_options: (params.cache_options, dryrun).into(),
+        clickhouse_connection_info: clickhouse_connection_info.clone(),
+        postgres_connection_info: postgres_connection_info.clone(),
+        tags: tags.clone(),
+        rate_limiting_config: Arc::new(config.rate_limiting.clone()),
+        otlp_config: config.gateway.export.otlp.clone(),
+        deferred_tasks,
+        scope_info: ScopeInfo { tags: tags.clone() },
     };
     let response = embedding_model
         .embed(&request, &params.model_name, &clients)
@@ -96,9 +103,10 @@ pub struct EmbeddingResponse {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::config::provider_types::ProviderTypesConfig;
     use crate::config::Config;
-    use crate::config::TimeoutsConfig;
     use crate::embeddings::{EmbeddingModelConfig, EmbeddingProviderConfig, EmbeddingProviderInfo};
+    use crate::model_table::ProviderTypeDefaultCredentials;
     use crate::providers::dummy::DummyProvider;
     use std::collections::HashMap;
     use tracing_test::traced_test;
@@ -113,22 +121,29 @@ mod tests {
         });
         let provider_info = EmbeddingProviderInfo {
             inner: dummy_provider,
-            timeouts: TimeoutsConfig::default(),
+            timeout_ms: None,
             provider_name: Arc::from("dummy"),
             extra_body: None,
         };
         let embedding_model = EmbeddingModelConfig {
             routing: vec!["dummy".to_string().into()],
             providers: HashMap::from([("dummy".to_string().into(), provider_info)]),
-            timeouts: TimeoutsConfig::default(),
+            timeout_ms: None,
         };
 
         // Create a minimal config with just the embedding model
         let mut embedding_models = HashMap::new();
         embedding_models.insert("test-model".to_string().into(), embedding_model);
 
+        let provider_types = ProviderTypesConfig::default();
         let config = Config {
-            embedding_models: embedding_models.try_into().unwrap(),
+            embedding_models: Arc::new(
+                crate::embeddings::EmbeddingModelTable::new(
+                    embedding_models,
+                    Arc::new(ProviderTypeDefaultCredentials::new(&provider_types)),
+                )
+                .unwrap(),
+            ),
             ..Default::default()
         };
 
@@ -145,13 +160,14 @@ mod tests {
             cache_options: CacheParamsOptions::default(),
         };
 
-        let clickhouse_connection_info = ClickHouseConnectionInfo::Disabled;
+        let clickhouse_connection_info = ClickHouseConnectionInfo::new_disabled();
 
         let result = embeddings(
             config,
             &http_client,
             clickhouse_connection_info,
             PostgresConnectionInfo::Disabled,
+            tokio_util::task::TaskTracker::new(),
             params,
         )
         .await;
@@ -180,13 +196,14 @@ mod tests {
             cache_options: CacheParamsOptions::default(),
         };
 
-        let clickhouse_connection_info = ClickHouseConnectionInfo::Disabled;
+        let clickhouse_connection_info = ClickHouseConnectionInfo::new_disabled();
 
         let result = embeddings(
             config,
             &http_client,
             clickhouse_connection_info,
             PostgresConnectionInfo::Disabled,
+            tokio_util::task::TaskTracker::new(),
             params,
         )
         .await;
