@@ -12,15 +12,17 @@ use serde::{Deserialize, Serialize};
 
 use crate::{
     config::Config,
-    db::clickhouse::{
-        query_builder::{
-            InferenceFilterTreeNode, InferenceOutputSource, ListInferencesParams, OrderBy,
+    db::{
+        clickhouse::{
+            query_builder::{InferenceFilter, OrderBy},
+            ClickHouseConnectionInfo,
         },
-        ClickHouseConnectionInfo, ClickhouseFormat,
+        inferences::{InferenceOutputSource, InferenceQueries, ListInferencesParams},
     },
     endpoints::{inference::InferenceCredentials, stored_inference::render_samples},
     error::{Error, ErrorDetails},
     http::TensorzeroHttpClient,
+    model_table::ProviderTypeDefaultCredentials,
     optimization::{
         JobHandle, OptimizationJobHandle, OptimizationJobInfo, Optimizer,
         UninitializedOptimizerInfo,
@@ -37,7 +39,7 @@ pub struct LaunchOptimizationWorkflowParams {
     pub function_name: String,
     pub template_variant_name: String,
     pub query_variant_name: Option<String>,
-    pub filters: Option<InferenceFilterTreeNode>,
+    pub filters: Option<InferenceFilter>,
     pub output_source: InferenceOutputSource,
     pub order_by: Option<Vec<OrderBy>>,
     #[serde(deserialize_with = "deserialize_option_u64")]
@@ -45,8 +47,6 @@ pub struct LaunchOptimizationWorkflowParams {
     #[serde(deserialize_with = "deserialize_option_u64")]
     pub offset: Option<u64>,
     pub val_fraction: Option<f64>,
-    #[serde(default)]
-    pub format: ClickhouseFormat,
     pub optimizer_config: UninitializedOptimizerInfo,
 }
 
@@ -87,7 +87,6 @@ pub async fn launch_optimization_workflow(
         limit,
         offset,
         val_fraction,
-        format,
         optimizer_config,
     } = params;
     // Query the database for the stored inferences
@@ -95,13 +94,14 @@ pub async fn launch_optimization_workflow(
         .list_inferences(
             &config,
             &ListInferencesParams {
-                function_name: &function_name,
+                function_name: Some(&function_name),
+                ids: None,
                 variant_name: query_variant_name.as_deref(),
+                episode_id: None,
                 filters: filters.as_ref(),
                 output_source,
                 limit,
                 offset,
-                format,
                 order_by: order_by.as_deref(),
             },
         )
@@ -118,10 +118,11 @@ pub async fn launch_optimization_workflow(
 
     // Split the inferences into train and val sets
     let (train_examples, val_examples) = split_examples(rendered_inferences, val_fraction)?;
+    let default_credentials = &config.models.default_credentials;
 
     // Launch the optimization job
     optimizer_config
-        .load()
+        .load(default_credentials)
         .await?
         .launch(
             http_client,
@@ -160,7 +161,9 @@ pub async fn launch_optimization(
         val_samples: val_examples,
         optimization_config: optimizer_config,
     } = params;
-    let optimizer = optimizer_config.load().await?;
+    let optimizer = optimizer_config
+        .load(&config.models.default_credentials)
+        .await?;
     optimizer
         .launch(
             http_client,
@@ -174,11 +177,16 @@ pub async fn launch_optimization(
 }
 
 pub async fn poll_optimization_handler(
-    State(AppStateData { http_client, .. }): AppState,
+    State(AppStateData {
+        http_client,
+        config,
+        ..
+    }): AppState,
     Path(job_handle): Path<String>,
 ) -> Result<Response<Body>, Error> {
     let job_handle = OptimizationJobHandle::from_base64_urlencoded(&job_handle)?;
-    let info = poll_optimization(&http_client, &job_handle).await?;
+    let default_credentials = &config.models.default_credentials;
+    let info = poll_optimization(&http_client, &job_handle, default_credentials).await?;
     Ok(Json(info).into_response())
 }
 
@@ -187,9 +195,14 @@ pub async fn poll_optimization_handler(
 pub async fn poll_optimization(
     http_client: &TensorzeroHttpClient,
     job_handle: &OptimizationJobHandle,
+    default_credentials: &ProviderTypeDefaultCredentials,
 ) -> Result<OptimizationJobInfo, Error> {
     job_handle
-        .poll(http_client, &InferenceCredentials::default())
+        .poll(
+            http_client,
+            &InferenceCredentials::default(),
+            default_credentials,
+        )
         .await
 }
 
