@@ -1,10 +1,12 @@
 use std::collections::HashSet;
 use std::{collections::HashMap, sync::Arc};
 
+use serde::de::{self, Deserializer, MapAccess, Visitor};
 use serde::{Deserialize, Serialize};
 use tensorzero_derive::TensorZeroDeserialize;
 
 use crate::config::{ErrorContext, LoadableConfig, UninitializedSchemas};
+use crate::experimentation::ExperimentationConfig;
 use crate::utils::retries::RetryConfig;
 use crate::variant::chat_completion::UninitializedChatCompletionConfig;
 use crate::variant::Variant;
@@ -39,17 +41,21 @@ pub const LLM_JUDGE_BOOLEAN_OUTPUT_SCHEMA_TEXT: &str =
 #[derive(Debug, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
-pub struct StaticEvaluationConfig {
+pub struct InferenceEvaluationConfig {
     pub evaluators: HashMap<String, EvaluatorConfig>,
     pub function_name: String,
 }
 
+/// Deprecated: Use `InferenceEvaluationConfig` instead
+pub type StaticEvaluationConfig = InferenceEvaluationConfig;
+
 #[derive(Debug, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
 #[serde(tag = "type", rename_all = "snake_case")]
 pub enum EvaluationConfig {
-    Static(StaticEvaluationConfig),
+    #[serde(alias = "static")]
+    Inference(InferenceEvaluationConfig),
 }
 
 #[derive(Debug, Serialize)]
@@ -81,6 +87,7 @@ impl EvaluatorConfig {
 #[derive(Debug, Deserialize, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
+#[serde(deny_unknown_fields)]
 pub struct ExactMatchConfig {
     #[serde(default)]
     pub cutoff: Option<f32>,
@@ -89,6 +96,7 @@ pub struct ExactMatchConfig {
 #[derive(Debug, Deserialize, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
+#[serde(deny_unknown_fields)]
 pub struct LLMJudgeConfig {
     pub input_format: LLMJudgeInputFormat,
     pub output_type: LLMJudgeOutputType,
@@ -100,6 +108,7 @@ pub struct LLMJudgeConfig {
 #[derive(Debug, Default, Deserialize, Serialize)]
 #[cfg_attr(test, derive(ts_rs::TS))]
 #[cfg_attr(test, ts(export))]
+#[serde(deny_unknown_fields)]
 pub struct LLMJudgeIncludeConfig {
     #[serde(default)]
     pub reference_output: bool,
@@ -159,12 +168,9 @@ pub fn get_evaluator_metric_name(evaluation_name: &str, evaluator_name: &str) ->
     format!("tensorzero::evaluation_name::{evaluation_name}::evaluator_name::{evaluator_name}")
 }
 
-#[derive(Debug, TensorZeroDeserialize)]
-#[serde(tag = "type")]
-#[serde(rename_all = "snake_case")]
-#[serde(deny_unknown_fields)]
+#[derive(Debug)]
 pub enum UninitializedEvaluationConfig {
-    Static(UninitializedStaticEvaluationConfig),
+    Inference(UninitializedInferenceEvaluationConfig),
 }
 
 impl UninitializedEvaluationConfig {
@@ -175,29 +181,95 @@ impl UninitializedEvaluationConfig {
         evaluation_name: &str,
     ) -> EvaluationLoadResult {
         match self {
-            UninitializedEvaluationConfig::Static(config) => {
+            UninitializedEvaluationConfig::Inference(config) => {
                 config.load(functions, evaluation_name)
             }
         }
     }
 }
 
+// Custom deserializer to log deprecation warning when "static" is used
+impl<'de> Deserialize<'de> for UninitializedEvaluationConfig {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        struct EvaluationConfigVisitor;
+
+        impl<'de> Visitor<'de> for EvaluationConfigVisitor {
+            type Value = UninitializedEvaluationConfig;
+
+            fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+                formatter.write_str("struct UninitializedEvaluationConfig")
+            }
+
+            fn visit_map<V>(self, mut map: V) -> Result<UninitializedEvaluationConfig, V::Error>
+            where
+                V: MapAccess<'de>,
+            {
+                let mut type_value: Option<String> = None;
+                let mut other_fields = serde_json::Map::new();
+
+                while let Some(key) = map.next_key::<String>()? {
+                    if key == "type" {
+                        type_value = Some(map.next_value()?);
+                    } else {
+                        let value: serde_json::Value = map.next_value()?;
+                        other_fields.insert(key, value);
+                    }
+                }
+
+                let type_str = type_value.ok_or_else(|| de::Error::missing_field("type"))?;
+
+                // Log deprecation warning if "static" is used
+                if type_str == "static" {
+                    tracing::warn!(
+                        "DEPRECATED: The evaluation type 'static' is deprecated. Please use 'inference' instead. Support for 'static' will be removed in a future version."
+                    );
+                }
+
+                // Validate type
+                if type_str != "inference" && type_str != "static" {
+                    return Err(de::Error::unknown_variant(&type_str, &["inference"]));
+                }
+
+                // Deserialize the config
+                let config: UninitializedInferenceEvaluationConfig =
+                    serde_json::from_value(serde_json::Value::Object(other_fields))
+                        .map_err(de::Error::custom)?;
+
+                Ok(UninitializedEvaluationConfig::Inference(config))
+            }
+        }
+
+        deserializer.deserialize_struct(
+            "UninitializedEvaluationConfig",
+            &["type"],
+            EvaluationConfigVisitor,
+        )
+    }
+}
+
 #[derive(Debug, Deserialize)]
-pub struct UninitializedStaticEvaluationConfig {
+#[serde(deny_unknown_fields)]
+pub struct UninitializedInferenceEvaluationConfig {
     evaluators: HashMap<String, UninitializedEvaluatorConfig>,
     function_name: String,
 }
 
+/// Deprecated: Use `UninitializedInferenceEvaluationConfig` instead
+pub type UninitializedStaticEvaluationConfig = UninitializedInferenceEvaluationConfig;
+
 type EvaluationLoadResult = Result<
     (
-        StaticEvaluationConfig,               // The evaluation itself
+        InferenceEvaluationConfig,            // The evaluation itself
         HashMap<String, Arc<FunctionConfig>>, // All functions which the evaluation needs {function_name -> function_config}
         HashMap<String, MetricConfig>, // All metrics which the evaluation needs {metric_name -> metric_config}
     ),
     Error,
 >;
 
-impl UninitializedStaticEvaluationConfig {
+impl UninitializedInferenceEvaluationConfig {
     pub fn load(
         self,
         functions: &HashMap<String, Arc<FunctionConfig>>,
@@ -258,7 +330,7 @@ impl UninitializedStaticEvaluationConfig {
             );
         }
         Ok((
-            StaticEvaluationConfig {
+            InferenceEvaluationConfig {
                 evaluators,
                 function_name: self.function_name,
             },
@@ -278,6 +350,7 @@ enum UninitializedEvaluatorConfig {
 }
 
 #[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
 struct UninitializedLLMJudgeConfig {
     #[serde(default)]
     input_format: LLMJudgeInputFormat,
@@ -410,6 +483,7 @@ impl UninitializedEvaluatorConfig {
                     .values()
                     .flat_map(|v| v.get_all_explicit_template_names())
                     .collect();
+                let experimentation = ExperimentationConfig::legacy_from_variants_map(&variants);
                 let function_config = FunctionConfig::Json(FunctionConfigJson {
                     variants,
                     schemas: SchemaData::load(
@@ -423,6 +497,7 @@ impl UninitializedEvaluatorConfig {
                     implicit_tool_call_config,
                     description: None,
                     all_explicit_template_names: all_template_names,
+                    experimentation,
                 });
                 Ok((
                     EvaluatorConfig::LLMJudge(LLMJudgeConfig {
@@ -481,6 +556,12 @@ struct UninitializedLLMJudgeChatCompletionVariantConfig {
     seed: Option<u32>,
     json_mode: JsonMode, // This is a JSON function
     stop_sequences: Option<Vec<String>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    reasoning_effort: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    thinking_budget_tokens: Option<i32>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    verbosity: Option<String>,
     #[serde(default)]
     retries: RetryConfig,
     #[serde(default)]
@@ -524,24 +605,27 @@ fn convert_chat_completion_judge_to_variant(
         LLMJudgeInputFormat::Messages => None,
     };
     UninitializedChatCompletionConfig {
-        weight: get_weight(params.active),
-        model: params.model,
-        templates: Default::default(),
-        system_template: Some(system_template.path),
-        user_template: user_template.map(|t| t.path),
         assistant_template: None,
-        input_wrappers: None,
-        temperature: params.temperature,
-        top_p: params.top_p,
-        max_tokens: params.max_tokens,
-        presence_penalty: params.presence_penalty,
-        frequency_penalty: params.frequency_penalty,
-        seed: params.seed,
-        stop_sequences: params.stop_sequences,
-        json_mode: Some(params.json_mode),
-        retries: params.retries,
         extra_body: params.extra_body,
         extra_headers: params.extra_headers,
+        frequency_penalty: params.frequency_penalty,
+        input_wrappers: None,
+        json_mode: Some(params.json_mode),
+        max_tokens: params.max_tokens,
+        model: params.model,
+        presence_penalty: params.presence_penalty,
+        reasoning_effort: params.reasoning_effort,
+        retries: params.retries,
+        seed: params.seed,
+        stop_sequences: params.stop_sequences,
+        system_template: Some(system_template.path),
+        temperature: params.temperature,
+        templates: Default::default(),
+        thinking_budget_tokens: params.thinking_budget_tokens,
+        top_p: params.top_p,
+        user_template: user_template.map(|t| t.path),
+        verbosity: params.verbosity,
+        weight: get_weight(params.active),
     }
     .load(
         &SchemaData::load(
@@ -563,6 +647,7 @@ fn default_timeout() -> f64 {
 }
 
 #[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
 struct UninitializedLLMJudgeBestOfNVariantConfig {
     #[serde(default)]
     active: Option<bool>,
@@ -574,6 +659,7 @@ struct UninitializedLLMJudgeBestOfNVariantConfig {
 }
 
 #[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
 struct UninitializedLLMJudgeMixtureOfNVariantConfig {
     #[serde(default)]
     active: Option<bool>,
@@ -585,6 +671,7 @@ struct UninitializedLLMJudgeMixtureOfNVariantConfig {
 }
 
 #[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
 struct UninitializedLLMJudgeDiclVariantConfig {
     #[serde(default)]
     active: Option<bool>,
@@ -609,6 +696,7 @@ struct UninitializedLLMJudgeDiclVariantConfig {
 }
 
 #[derive(Debug, Deserialize)]
+#[serde(deny_unknown_fields)]
 struct UninitializedLLMJudgeChainOfThoughtVariantConfig {
     #[serde(flatten)]
     inner: UninitializedLLMJudgeChatCompletionVariantConfig,
@@ -691,24 +779,27 @@ impl UninitializedLLMJudgeVariantInfo {
                         candidates: params.candidates,
                         evaluator: UninitializedBestOfNEvaluatorConfig {
                             inner: UninitializedChatCompletionConfig {
-                                weight: None,
-                                model: params.evaluator.model,
-                                user_template: evaluator_user_template.map(|t| t.path),
-                                system_template: Some(evaluator_system_template.path),
-                                templates: Default::default(),
-                                input_wrappers: None,
                                 assistant_template: None,
-                                temperature: params.evaluator.temperature,
-                                top_p: params.evaluator.top_p,
-                                max_tokens: params.evaluator.max_tokens,
-                                presence_penalty: params.evaluator.presence_penalty,
-                                frequency_penalty: params.evaluator.frequency_penalty,
-                                seed: params.evaluator.seed,
-                                json_mode: Some(params.evaluator.json_mode),
-                                stop_sequences: params.evaluator.stop_sequences,
-                                retries: params.evaluator.retries,
                                 extra_body: params.evaluator.extra_body,
                                 extra_headers: params.evaluator.extra_headers,
+                                frequency_penalty: params.evaluator.frequency_penalty,
+                                input_wrappers: None,
+                                json_mode: Some(params.evaluator.json_mode),
+                                max_tokens: params.evaluator.max_tokens,
+                                model: params.evaluator.model,
+                                presence_penalty: params.evaluator.presence_penalty,
+                                reasoning_effort: params.evaluator.reasoning_effort,
+                                retries: params.evaluator.retries,
+                                seed: params.evaluator.seed,
+                                stop_sequences: params.evaluator.stop_sequences,
+                                system_template: Some(evaluator_system_template.path),
+                                temperature: params.evaluator.temperature,
+                                templates: Default::default(),
+                                thinking_budget_tokens: params.evaluator.thinking_budget_tokens,
+                                top_p: params.evaluator.top_p,
+                                user_template: evaluator_user_template.map(|t| t.path),
+                                verbosity: params.evaluator.verbosity,
+                                weight: None,
                             },
                         },
                     }
@@ -759,24 +850,27 @@ impl UninitializedLLMJudgeVariantInfo {
                         candidates: params.candidates,
                         fuser: UninitializedFuserConfig {
                             inner: UninitializedChatCompletionConfig {
-                                weight: None,
-                                model: params.fuser.model,
-                                user_template: fuser_user_template.map(|t| t.path),
-                                system_template: Some(fuser_system_template.path),
-                                templates: Default::default(),
                                 assistant_template: None,
-                                input_wrappers: None,
-                                temperature: params.fuser.temperature,
-                                top_p: params.fuser.top_p,
-                                max_tokens: params.fuser.max_tokens,
-                                presence_penalty: params.fuser.presence_penalty,
-                                frequency_penalty: params.fuser.frequency_penalty,
-                                seed: params.fuser.seed,
-                                json_mode: Some(params.fuser.json_mode),
-                                retries: params.fuser.retries,
-                                stop_sequences: params.fuser.stop_sequences,
                                 extra_body: params.fuser.extra_body,
                                 extra_headers: params.fuser.extra_headers,
+                                frequency_penalty: params.fuser.frequency_penalty,
+                                input_wrappers: None,
+                                json_mode: Some(params.fuser.json_mode),
+                                max_tokens: params.fuser.max_tokens,
+                                model: params.fuser.model,
+                                presence_penalty: params.fuser.presence_penalty,
+                                reasoning_effort: params.fuser.reasoning_effort,
+                                retries: params.fuser.retries,
+                                seed: params.fuser.seed,
+                                stop_sequences: params.fuser.stop_sequences,
+                                system_template: Some(fuser_system_template.path),
+                                temperature: params.fuser.temperature,
+                                templates: Default::default(),
+                                thinking_budget_tokens: params.fuser.thinking_budget_tokens,
+                                top_p: params.fuser.top_p,
+                                user_template: fuser_user_template.map(|t| t.path),
+                                verbosity: params.fuser.verbosity,
+                                weight: None,
                             },
                         },
                     }
@@ -826,6 +920,8 @@ impl UninitializedLLMJudgeVariantInfo {
                     extra_headers: params.extra_headers,
                     retries: params.retries,
                     stop_sequences: params.stop_sequences,
+                    max_distance: None,
+                    ..Default::default()
                 };
                 VariantConfig::Dicl(uninitialized_config.load()?)
             }
@@ -879,6 +975,9 @@ fn check_convert_variant_to_llm_judge_variant(
                     stop_sequences: variant.stop_sequences().cloned(),
                     extra_body: variant.extra_body().cloned(),
                     extra_headers: variant.extra_headers().cloned(),
+                    reasoning_effort: variant.reasoning_effort().cloned(),
+                    thinking_budget_tokens: variant.thinking_budget_tokens(),
+                    verbosity: variant.verbosity().cloned(),
                 },
             ))
         }
@@ -906,6 +1005,9 @@ fn check_convert_variant_to_llm_judge_variant(
                         stop_sequences: variant.evaluator().inner.stop_sequences().cloned(),
                         extra_body: variant.evaluator().inner.extra_body().cloned(),
                         extra_headers: variant.evaluator().inner.extra_headers().cloned(),
+                        reasoning_effort: variant.evaluator().inner.reasoning_effort().cloned(),
+                        thinking_budget_tokens: variant.evaluator().inner.thinking_budget_tokens(),
+                        verbosity: variant.evaluator().inner.verbosity().cloned(),
                     },
                 },
             ))
@@ -934,6 +1036,18 @@ fn check_convert_variant_to_llm_judge_variant(
                         stop_sequences: variant.fuser().inner.stop_sequences().cloned(),
                         extra_body: variant.fuser().inner.extra_body().cloned(),
                         extra_headers: variant.fuser().inner.extra_headers().cloned(),
+                        reasoning_effort: variant
+                            .fuser()
+                            .inner
+                            .inference_params_v2
+                            .reasoning_effort
+                            .clone(),
+                        thinking_budget_tokens: variant
+                            .fuser()
+                            .inner
+                            .inference_params_v2
+                            .thinking_budget_tokens,
+                        verbosity: variant.fuser().inner.inference_params_v2.verbosity.clone(),
                     },
                 },
             ))
@@ -979,6 +1093,16 @@ fn check_convert_variant_to_llm_judge_variant(
                         stop_sequences: variant.inner.stop_sequences().cloned(),
                         extra_body: variant.inner.extra_body().cloned(),
                         extra_headers: variant.inner.extra_headers().cloned(),
+                        reasoning_effort: variant
+                            .inner
+                            .inference_params_v2
+                            .reasoning_effort
+                            .clone(),
+                        thinking_budget_tokens: variant
+                            .inner
+                            .inference_params_v2
+                            .thinking_budget_tokens,
+                        verbosity: variant.inner.inference_params_v2.verbosity.clone(),
                     },
                 },
             ))
@@ -1008,6 +1132,7 @@ mod tests {
             implicit_tool_call_config: create_implicit_tool_call_config(create_test_schema()),
             description: None,
             all_explicit_template_names: HashSet::new(),
+            experimentation: ExperimentationConfig::legacy_from_variants_map(&HashMap::new()),
         });
         functions.insert(function_name.to_string(), Arc::new(function_config));
 
@@ -1019,7 +1144,7 @@ mod tests {
                 UninitializedEvaluatorConfig::ExactMatch(ExactMatchConfig { cutoff: Some(0.4) }),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1064,7 +1189,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: Some(true),
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions:
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt"
                                     .into(),
@@ -1079,6 +1204,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1102,7 +1230,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1188,7 +1316,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: Some(true),
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions:
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt"
                                     .into(),
@@ -1203,6 +1331,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1226,7 +1357,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1296,7 +1427,7 @@ mod tests {
                 UninitializedEvaluatorConfig::ExactMatch(ExactMatchConfig { cutoff: None }),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: "nonexistent_function".to_string(),
             };
@@ -1317,7 +1448,7 @@ mod tests {
                 UninitializedEvaluatorConfig::ExactMatch(ExactMatchConfig { cutoff: None }),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1339,7 +1470,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: Some(true),
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions:
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt"
                                     .into(),
@@ -1354,6 +1485,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1382,6 +1516,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1414,7 +1551,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1446,6 +1583,9 @@ mod tests {
                     ),
                     description: None,
                     all_explicit_template_names: HashSet::new(),
+                    experimentation: ExperimentationConfig::legacy_from_variants_map(
+                        &HashMap::new(),
+                    ),
                 })),
             );
 
@@ -1455,7 +1595,7 @@ mod tests {
                 UninitializedEvaluatorConfig::ExactMatch(ExactMatchConfig { cutoff: None }),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1481,7 +1621,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: Some(true),
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions: ResolvedTomlPath::new_for_tests(PathBuf::from(
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt",
                             ), None),
@@ -1496,6 +1636,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1519,7 +1662,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1552,7 +1695,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: None, // No 'active' field specified
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions: ResolvedTomlPath::new_for_tests(PathBuf::from(
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt",
                             ), None),
@@ -1567,6 +1710,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1588,7 +1734,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1625,7 +1771,7 @@ mod tests {
                     inner: UninitializedLLMJudgeVariantConfig::ChatCompletion(
                         UninitializedLLMJudgeChatCompletionVariantConfig {
                             active: Some(false), // Explicitly inactive
-                            model: Arc::from("gpt-3.5-turbo"),
+                            model: Arc::from("gpt-4.1-mini"),
                             system_instructions: ResolvedTomlPath::new_for_tests(PathBuf::from(
                                 "fixtures/config/evaluations/evaluation1/llm_judge_bool/system_instructions.txt",
                             ), None),
@@ -1640,6 +1786,9 @@ mod tests {
                             extra_body: Default::default(),
                             extra_headers: Default::default(),
                             stop_sequences: None,
+                            reasoning_effort: None,
+                            thinking_budget_tokens: None,
+                            verbosity: None,
                         },
                     ),
                     timeouts: None,
@@ -1661,7 +1810,7 @@ mod tests {
                 UninitializedEvaluatorConfig::LLMJudge(llm_judge_config),
             );
 
-            let uninitialized_config = UninitializedStaticEvaluationConfig {
+            let uninitialized_config = UninitializedInferenceEvaluationConfig {
                 evaluators,
                 function_name: function_name.to_string(),
             };
@@ -1677,6 +1826,64 @@ mod tests {
         }
     }
 
+    /// Test backward compatibility: verify deprecated "static" type still works
+    #[test]
+    fn test_backward_compatibility_static_type() {
+        // Setup: Create a function that the evaluation will reference
+        let mut functions = HashMap::new();
+        let function_name = "test_function";
+        let function_config = FunctionConfig::Json(FunctionConfigJson {
+            variants: HashMap::new(),
+            schemas: SchemaData::default(),
+            output_schema: create_test_schema(),
+            implicit_tool_call_config: create_implicit_tool_call_config(create_test_schema()),
+            description: None,
+            all_explicit_template_names: HashSet::new(),
+            experimentation: ExperimentationConfig::legacy_from_variants_map(&HashMap::new()),
+        });
+        functions.insert(function_name.to_string(), Arc::new(function_config));
+
+        // Test TOML config with deprecated "static" type
+        let toml = format!(
+            r#"
+            type = "static"
+            function_name = "{function_name}"
+
+            [evaluators.test_evaluator]
+            type = "exact_match"
+        "#
+        );
+
+        // Deserialize the TOML
+        let uninitialized: UninitializedEvaluationConfig = toml::from_str(&toml).unwrap();
+
+        // Verify it loads correctly despite using deprecated "static" type
+        let result = uninitialized.load(&functions, "test_backward_compat");
+        assert!(
+            result.is_ok(),
+            "Expected successful load with 'static' type"
+        );
+
+        let (config, additional_functions, metric_configs) = result.unwrap();
+
+        // Verify the config was loaded correctly
+        assert_eq!(config.function_name, function_name);
+        assert_eq!(config.evaluators.len(), 1);
+        assert!(config.evaluators.contains_key("test_evaluator"));
+
+        // Verify exact match evaluator was loaded
+        match config.evaluators.get("test_evaluator").unwrap() {
+            EvaluatorConfig::ExactMatch(_) => {} // Expected
+            EvaluatorConfig::LLMJudge(_) => panic!("Expected ExactMatch evaluator"),
+        }
+
+        // Verify no additional functions (exact match doesn't need them)
+        assert_eq!(additional_functions.len(), 0);
+
+        // Verify metric was created
+        assert_eq!(metric_configs.len(), 1);
+    }
+
     // Helper functions for tests
     fn create_test_schema() -> StaticJSONSchema {
         let schema_value = serde_json::json!({
