@@ -1,13 +1,17 @@
-#[cfg(feature = "pyo3")]
-use pyo3::prelude::*;
+use futures::future::try_join_all;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
+use std::{collections::HashMap, sync::Arc};
+use tokio::sync::Semaphore;
+use uuid::Uuid;
 
 use crate::{
     cache::CacheOptions,
     config::{Config, UninitializedVariantConfig},
     db::{
-        clickhouse::{ClickHouseConnectionInfo, ExternalDataInfo},
+        clickhouse::{
+            clickhouse_client::ClickHouseClientType, ClickHouseConnectionInfo, ExternalDataInfo,
+        },
         postgres::PostgresConnectionInfo,
     },
     embeddings::{Embedding, EmbeddingEncodingFormat, EmbeddingInput, EmbeddingRequest},
@@ -15,19 +19,20 @@ use crate::{
     error::{Error, ErrorDetails, IMPOSSIBLE_ERROR_MESSAGE},
     function::FunctionConfig,
     http::TensorzeroHttpClient,
-    model::{build_creds_caching_default, CredentialLocation},
+    inference::types::StoredInputMessageContent,
+    model::CredentialLocationWithFallback,
+    model_table::{OpenAIKind, ProviderKind, ProviderTypeDefaultCredentials},
     optimization::{JobHandle, OptimizationJobInfo, Optimizer, OptimizerOutput},
-    providers::openai::{
-        default_api_key_location, OpenAICredentials, DEFAULT_CREDENTIALS, PROVIDER_TYPE,
-    },
+    providers::openai::OpenAICredentials,
+    rate_limiting::ScopeInfo,
     stored_inference::RenderedSample,
-    utils::retries::RetryConfig,
     variant::dicl::UninitializedDiclConfig,
 };
-use futures::future::try_join_all;
-use std::{collections::HashMap, sync::Arc};
-use tokio::sync::Semaphore;
-use uuid::Uuid;
+
+#[cfg(feature = "pyo3")]
+use crate::model::CredentialLocation;
+#[cfg(feature = "pyo3")]
+use pyo3::prelude::*;
 
 fn default_batch_size() -> usize {
     128
@@ -65,7 +70,7 @@ pub struct DiclOptimizationConfig {
     #[serde(skip)]
     pub credentials: OpenAICredentials,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
 }
 
 #[cfg_attr(test, derive(ts_rs::TS))]
@@ -88,7 +93,7 @@ pub struct UninitializedDiclOptimizationConfig {
     #[serde(default = "default_append_to_existing_variants")]
     pub append_to_existing_variants: bool,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credentials: Option<CredentialLocation>,
+    pub credentials: Option<CredentialLocationWithFallback>,
 }
 
 impl Default for UninitializedDiclOptimizationConfig {
@@ -138,9 +143,12 @@ impl UninitializedDiclOptimizationConfig {
         append_to_existing_variants: Option<bool>,
         credentials: Option<String>,
     ) -> PyResult<Self> {
-        // Use Deserialize to convert the string to a CredentialLocation
-        let credentials =
-            credentials.map(|s| serde_json::from_str(&s).unwrap_or(CredentialLocation::Env(s)));
+        // Use Deserialize to convert the string to a CredentialLocationWithFallback
+        let credentials = credentials.map(|s| {
+            serde_json::from_str(&s).unwrap_or(CredentialLocationWithFallback::Single(
+                CredentialLocation::Env(s),
+            ))
+        });
         Ok(Self {
             embedding_model,
             variant_name,
@@ -188,7 +196,10 @@ impl UninitializedDiclOptimizationConfig {
 }
 
 impl UninitializedDiclOptimizationConfig {
-    pub fn load(self) -> Result<DiclOptimizationConfig, Error> {
+    pub async fn load(
+        self,
+        default_credentials: &ProviderTypeDefaultCredentials,
+    ) -> Result<DiclOptimizationConfig, Error> {
         Ok(DiclOptimizationConfig {
             embedding_model: Arc::from(self.embedding_model),
             variant_name: self.variant_name,
@@ -199,12 +210,9 @@ impl UninitializedDiclOptimizationConfig {
             k: self.k,
             model: Arc::from(self.model),
             append_to_existing_variants: self.append_to_existing_variants,
-            credentials: build_creds_caching_default(
-                self.credentials.clone(),
-                default_api_key_location(),
-                PROVIDER_TYPE,
-                &DEFAULT_CREDENTIALS,
-            )?,
+            credentials: OpenAIKind
+                .get_defaulted_credential(self.credentials.as_ref(), default_credentials)
+                .await?,
             credential_location: self.credentials,
         })
     }
@@ -248,10 +256,7 @@ impl Optimizer for DiclOptimizationConfig {
         }
 
         // Check if ClickHouse is available (required for DICL)
-        if matches!(
-            clickhouse_connection_info,
-            ClickHouseConnectionInfo::Disabled
-        ) {
+        if clickhouse_connection_info.client_type() == ClickHouseClientType::Disabled {
             return Err(Error::new(ErrorDetails::AppState {
                 message: "DICL optimization requires ClickHouse to be enabled to store examples"
                     .to_string(),
@@ -376,6 +381,7 @@ impl JobHandle for DiclOptimizationJobHandle {
         &self,
         client: &TensorzeroHttpClient,
         credentials: &InferenceCredentials,
+        _default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<OptimizationJobInfo, Error> {
         // DICL optimization is synchronous, so it's always complete once launched
         let _ = (client, credentials);
@@ -385,22 +391,10 @@ impl JobHandle for DiclOptimizationJobHandle {
         Ok(OptimizationJobInfo::Completed {
             output: OptimizerOutput::Variant(Box::new(UninitializedVariantConfig::Dicl(
                 UninitializedDiclConfig {
-                    weight: None,
                     embedding_model: self.embedding_model.to_string(),
                     k: self.k,
                     model: self.model.to_string(),
-                    system_instructions: None,
-                    temperature: None,
-                    top_p: None,
-                    stop_sequences: None,
-                    presence_penalty: None,
-                    frequency_penalty: None,
-                    max_tokens: None,
-                    seed: None,
-                    json_mode: None,
-                    extra_body: None,
-                    extra_headers: None,
-                    retries: RetryConfig::default(),
+                    ..Default::default()
                 },
             ))),
         })
@@ -427,12 +421,17 @@ fn validate_function_config(
         }
         FunctionConfig::Json(json_config) => {
             // JSON functions should have exactly one implicit tool for schema validation
-            if json_config.implicit_tool_call_config.tools_available.len() != 1 {
+            if json_config
+                .implicit_tool_call_config
+                .tools_available()
+                .count()
+                != 1
+            {
                 return Err(Error::new(ErrorDetails::InvalidRequest {
                     message: format!(
                         "DICL optimization expected JSON function '{}' to have exactly 1 implicit tool, but found {}. This indicates a configuration issue.",
                         function_name,
-                        json_config.implicit_tool_call_config.tools_available.len()
+                        json_config.implicit_tool_call_config.tools_available().count()
                     ),
                 }));
             }
@@ -462,24 +461,41 @@ fn validate_train_examples(train_examples: &[RenderedSample]) -> Result<(), Erro
                 ),
             }));
         }
-        // Check if tools_available contains actual tools
-        if let Some(tool_params) = &example.tool_params {
-            if !tool_params.tools_available.is_empty() {
-                return Err(Error::new(ErrorDetails::InvalidRequest {
+        // Check if tools are available
+        let has_additional_tools = example
+            .tool_params
+            .additional_tools
+            .as_ref()
+            .map(|tools| !tools.is_empty())
+            .unwrap_or(false);
+        let has_allowed_tools = example
+            .tool_params
+            .allowed_tools
+            .as_ref()
+            .map(|tools| !tools.is_empty())
+            .unwrap_or(false);
+
+        if has_additional_tools || has_allowed_tools {
+            let num_tools = example
+                .tool_params
+                .additional_tools
+                .as_ref()
+                .map(std::vec::Vec::len)
+                .unwrap_or(0);
+            return Err(Error::new(ErrorDetails::InvalidRequest {
                     message: format!(
                         "DICL optimization does not support tool calls. Training example {} contains {} available tools.",
                         i + 1,
-                        tool_params.tools_available.len()
+                        num_tools
                     ),
                 }));
-            }
         }
 
         // Check stored_input messages for ToolCall or ToolResult content
         for message in &example.stored_input.messages {
             for content in &message.content {
                 match content {
-                    crate::inference::types::StoredInputMessageContent::ToolCall(_) => {
+                    StoredInputMessageContent::ToolCall(_) => {
                         return Err(Error::new(ErrorDetails::InvalidRequest {
                             message: format!(
                                 "DICL optimization does not support tool calls. Training example {} contains a tool call in message content.",
@@ -487,7 +503,7 @@ fn validate_train_examples(train_examples: &[RenderedSample]) -> Result<(), Erro
                             ),
                         }));
                     }
-                    crate::inference::types::StoredInputMessageContent::ToolResult(_) => {
+                    StoredInputMessageContent::ToolResult(_) => {
                         return Err(Error::new(ErrorDetails::InvalidRequest {
                             message: format!(
                                 "DICL optimization does not support tool calls. Training example {} contains a tool result in message content.",
@@ -531,24 +547,33 @@ async fn process_embedding_batch(
                 })
             })?;
 
+    let tags = Arc::new(HashMap::default());
+
     // Create InferenceClients context for the embedding model
-    let cache_options = CacheOptions::default();
+    let deferred_tasks = tokio_util::task::TaskTracker::new();
     let clients = InferenceClients {
-        http_client: client,
-        credentials,
-        clickhouse_connection_info: &ClickHouseConnectionInfo::Disabled,
-        postgres_connection_info: &PostgresConnectionInfo::Disabled,
-        cache_options: &cache_options,
-        tags: &HashMap::default(),
-        rate_limiting_config: &config.rate_limiting,
+        http_client: client.clone(),
+        credentials: Arc::new(credentials.clone()),
+        clickhouse_connection_info: ClickHouseConnectionInfo::new_disabled(),
+        postgres_connection_info: PostgresConnectionInfo::Disabled,
+        cache_options: CacheOptions::default(),
+        tags: tags.clone(),
+        rate_limiting_config: Arc::new(config.rate_limiting.clone()),
         // We don't currently perform any OTLP export in optimization workflows
-        otlp_config: &Default::default(),
+        otlp_config: Default::default(),
+        deferred_tasks: deferred_tasks.clone(),
+        scope_info: ScopeInfo { tags: tags.clone() },
     };
 
     let response = embedding_model_config
         .embed(&embedding_request, model_name, &clients)
         .await?;
 
+    // We're running an optimization, so we don't really have a gateway to shutdown
+    // Let's just wait for the deferred tasks to finish immediately
+    deferred_tasks.close();
+    deferred_tasks.wait().await;
+
     tracing::debug!("Successfully processed embedding batch {}", batch_index);
 
     // Convert embeddings from Vec<Embedding> to Vec<Vec<f64>>
@@ -791,14 +816,35 @@ pub async fn dicl_examples_exist(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{
-        config::TimeoutsConfig,
-        embeddings::{EmbeddingModelConfig, EmbeddingProviderConfig, EmbeddingProviderInfo},
-        endpoints::inference::InferenceCredentials,
-    };
     use std::collections::{HashMap, HashSet};
     use std::sync::Arc;
 
+    use crate::{
+        config::{provider_types::ProviderTypesConfig, SchemaData},
+        embeddings::{
+            EmbeddingModelConfig, EmbeddingModelTable, EmbeddingProviderConfig,
+            EmbeddingProviderInfo,
+        },
+        endpoints::inference::InferenceCredentials,
+        experimentation::ExperimentationConfig,
+        function::{FunctionConfigChat, FunctionConfigJson},
+    };
+    use crate::{
+        inference::types::{
+            ContentBlockChatOutput, ModelInput, ResolvedContentBlock, ResolvedRequestMessage, Role,
+            StoredInput, StoredInputMessage, StoredInputMessageContent, System, Text,
+        },
+        jsonschema_util::StaticJSONSchema,
+        stored_inference::StoredOutput,
+        tool::{
+            create_implicit_tool_call_config, DynamicToolParams, Tool, ToolCall, ToolCallConfig,
+            ToolChoice, ToolResult,
+        },
+    };
+
+    #[cfg(any(test, feature = "e2e_tests"))]
+    use crate::providers::dummy::DummyProvider;
+
     // Helper functions to create test embedding models using the Dummy provider
 
     fn create_test_embedding_model_config() -> Config {
@@ -812,7 +858,6 @@ mod tests {
     fn create_test_embedding_model_with_name(model_name: &str) -> Config {
         #[cfg(any(test, feature = "e2e_tests"))]
         {
-            use crate::providers::dummy::DummyProvider;
             let mut providers = HashMap::new();
             providers.insert(
                 Arc::from("dummy"),
@@ -821,7 +866,7 @@ mod tests {
                         model_name: model_name.to_string(),
                         ..Default::default()
                     }),
-                    timeouts: TimeoutsConfig::default(),
+                    timeout_ms: None,
                     provider_name: Arc::from("dummy"),
                     extra_body: None,
                 },
@@ -829,12 +874,17 @@ mod tests {
             let embedding_model_config = EmbeddingModelConfig {
                 routing: vec![Arc::from("dummy")],
                 providers,
-                timeouts: TimeoutsConfig::default(),
+                timeout_ms: None,
             };
+            let provider_types = ProviderTypesConfig::default();
             Config {
-                embedding_models: HashMap::from([(Arc::from(model_name), embedding_model_config)])
-                    .try_into()
+                embedding_models: Arc::new(
+                    EmbeddingModelTable::new(
+                        HashMap::from([(Arc::from(model_name), embedding_model_config)]),
+                        ProviderTypeDefaultCredentials::new(&provider_types).into(),
+                    )
                     .unwrap(),
+                ),
                 ..Default::default()
             }
         }
@@ -979,17 +1029,6 @@ mod tests {
 
     // Helper function to create a basic RenderedSample for testing
     fn create_test_rendered_sample() -> RenderedSample {
-        use crate::{
-            inference::types::{
-                ContentBlockChatOutput, ModelInput, ResolvedContentBlock, ResolvedRequestMessage,
-                Role, StoredInput, StoredInputMessage, StoredInputMessageContent, Text,
-            },
-            stored_inference::StoredOutput,
-        };
-        use serde_json::json;
-        use std::collections::HashMap;
-        use uuid::Uuid;
-
         RenderedSample {
             function_name: "test_function".to_string(),
             input: ModelInput {
@@ -1002,12 +1041,12 @@ mod tests {
                 }],
             },
             stored_input: StoredInput {
-                system: Some(json!("Test system")),
+                system: Some(System::Text("Test system".to_string())),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("Test message"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "Test message".to_string(),
+                    })],
                 }],
             },
             output: Some(vec![ContentBlockChatOutput::Text(Text {
@@ -1020,32 +1059,26 @@ mod tests {
             )])),
             episode_id: Some(Uuid::now_v7()),
             inference_id: Some(Uuid::now_v7()),
-            tool_params: None,
+            tool_params: DynamicToolParams::default(),
             output_schema: None,
             dispreferred_outputs: vec![],
             tags: HashMap::new(),
         }
     }
 
-    fn create_test_rendered_sample_with_tools(tools: Vec<crate::tool::Tool>) -> RenderedSample {
-        use crate::tool::{ToolCallConfigDatabaseInsert, ToolChoice};
-
+    fn create_test_rendered_sample_with_tools(tools: Vec<Tool>) -> RenderedSample {
         let mut sample = create_test_rendered_sample();
-        sample.tool_params = Some(ToolCallConfigDatabaseInsert {
-            tools_available: tools,
-            tool_choice: ToolChoice::Auto,
+        sample.tool_params = DynamicToolParams {
+            allowed_tools: None,
+            additional_tools: Some(tools),
+            tool_choice: Some(ToolChoice::Auto),
             parallel_tool_calls: Some(true),
-        });
+            provider_tools: None,
+        };
         sample
     }
 
     fn create_test_rendered_sample_with_tool_content() -> RenderedSample {
-        use crate::{
-            inference::types::{Role, StoredInputMessage, StoredInputMessageContent},
-            tool::{ToolCall, ToolCallConfigDatabaseInsert, ToolChoice},
-        };
-        use serde_json::json;
-
         let mut sample = create_test_rendered_sample();
 
         // Add a message with tool call content
@@ -1054,26 +1087,23 @@ mod tests {
             content: vec![StoredInputMessageContent::ToolCall(ToolCall {
                 id: "test_call".to_string(),
                 name: "test_tool".to_string(),
-                arguments: json!({"arg": "value"}).to_string(),
+                arguments: serde_json::json!({"arg": "value"}).to_string(),
             })],
         });
 
         // Also add empty tool params to make it realistic
-        sample.tool_params = Some(ToolCallConfigDatabaseInsert {
-            tools_available: vec![],
-            tool_choice: ToolChoice::None,
+        sample.tool_params = DynamicToolParams {
+            allowed_tools: None,
+            additional_tools: None,
+            tool_choice: Some(ToolChoice::None),
             parallel_tool_calls: Some(false),
-        });
+            provider_tools: None,
+        };
 
         sample
     }
 
     fn create_test_rendered_sample_with_tool_result() -> RenderedSample {
-        use crate::{
-            inference::types::{Role, StoredInputMessage, StoredInputMessageContent},
-            tool::{ToolCallConfigDatabaseInsert, ToolChoice, ToolResult},
-        };
-
         let mut sample = create_test_rendered_sample();
 
         // Add a message with tool result content
@@ -1087,11 +1117,13 @@ mod tests {
         });
 
         // Also add empty tool params to make it realistic
-        sample.tool_params = Some(ToolCallConfigDatabaseInsert {
-            tools_available: vec![],
-            tool_choice: ToolChoice::None,
+        sample.tool_params = DynamicToolParams {
+            allowed_tools: None,
+            additional_tools: None,
+            tool_choice: Some(ToolChoice::None),
             parallel_tool_calls: Some(false),
-        });
+            provider_tools: None,
+        };
 
         sample
     }
@@ -1123,13 +1155,10 @@ mod tests {
 
     #[test]
     fn test_validate_train_examples_rejects_tools_available() {
-        use crate::tool::Tool;
-        use serde_json::json;
-
         let tool = Tool {
             name: "test_tool".to_string(),
             description: "A test tool".to_string(),
-            parameters: json!({"type": "object", "properties": {}}),
+            parameters: serde_json::json!({"type": "object", "properties": {}}),
             strict: false,
         };
 
@@ -1170,15 +1199,12 @@ mod tests {
 
     #[test]
     fn test_validate_train_examples_multiple_samples() {
-        use crate::tool::Tool;
-        use serde_json::json;
-
         let valid_sample = create_test_rendered_sample();
 
         let tool = Tool {
             name: "test_tool".to_string(),
             description: "A test tool".to_string(),
-            parameters: json!({"type": "object", "properties": {}}),
+            parameters: serde_json::json!({"type": "object", "properties": {}}),
             strict: false,
         };
         let invalid_sample = create_test_rendered_sample_with_tools(vec![tool]);
@@ -1217,9 +1243,6 @@ mod tests {
     }
 
     fn create_test_chat_function_config_no_tools() -> FunctionConfig {
-        use crate::{config::SchemaData, function::FunctionConfigChat, tool::ToolChoice};
-        use std::collections::HashMap;
-
         FunctionConfig::Chat(FunctionConfigChat {
             variants: HashMap::new(),
             schemas: SchemaData::default(),
@@ -1229,13 +1252,11 @@ mod tests {
             description: None,
 
             all_explicit_templates_names: HashSet::new(),
+            experimentation: ExperimentationConfig::default(),
         })
     }
 
     fn create_test_chat_function_config_with_tools() -> FunctionConfig {
-        use crate::{config::SchemaData, function::FunctionConfigChat, tool::ToolChoice};
-        use std::collections::HashMap;
-
         FunctionConfig::Chat(FunctionConfigChat {
             variants: HashMap::new(),
             schemas: SchemaData::default(),
@@ -1244,18 +1265,12 @@ mod tests {
             parallel_tool_calls: None,
             description: None,
             all_explicit_templates_names: HashSet::new(),
+            experimentation: ExperimentationConfig::default(),
         })
     }
 
     fn create_test_json_function_config() -> FunctionConfig {
-        use crate::{
-            config::SchemaData, function::FunctionConfigJson, jsonschema_util::StaticJSONSchema,
-            tool::create_implicit_tool_call_config,
-        };
-        use serde_json::json;
-        use std::collections::HashMap;
-
-        let output_schema = StaticJSONSchema::from_value(json!({
+        let output_schema = StaticJSONSchema::from_value(serde_json::json!({
             "type": "object",
             "properties": {
                 "answer": {"type": "string"}
@@ -1273,20 +1288,12 @@ mod tests {
             implicit_tool_call_config,
             description: None,
             all_explicit_template_names: HashSet::new(),
+            experimentation: ExperimentationConfig::default(),
         })
     }
 
     fn create_test_json_function_config_invalid_tools() -> FunctionConfig {
-        use crate::{
-            config::SchemaData,
-            function::FunctionConfigJson,
-            jsonschema_util::StaticJSONSchema,
-            tool::{ToolCallConfig, ToolChoice},
-        };
-        use serde_json::json;
-        use std::collections::HashMap;
-
-        let output_schema = StaticJSONSchema::from_value(json!({
+        let output_schema = StaticJSONSchema::from_value(serde_json::json!({
             "type": "object",
             "properties": {
                 "answer": {"type": "string"}
@@ -1296,11 +1303,7 @@ mod tests {
         .unwrap();
 
         // Create an invalid config with no tools (should have exactly 1)
-        let invalid_tool_call_config = ToolCallConfig {
-            tools_available: vec![], // Invalid: should have exactly 1 implicit tool
-            tool_choice: ToolChoice::None,
-            parallel_tool_calls: None,
-        };
+        let invalid_tool_call_config = ToolCallConfig::default();
 
         FunctionConfig::Json(FunctionConfigJson {
             variants: HashMap::new(),
@@ -1309,6 +1312,7 @@ mod tests {
             implicit_tool_call_config: invalid_tool_call_config,
             description: None,
             all_explicit_template_names: HashSet::new(),
+            experimentation: ExperimentationConfig::default(),
         })
     }
 
