@@ -31,6 +31,9 @@ use crate::http::TensorzeroHttpClient;
 use crate::model::UninitializedModelConfig;
 use crate::model::UninitializedModelProvider;
 use crate::model::UninitializedProviderConfig;
+use crate::model_table::FireworksKind;
+use crate::model_table::ProviderKind;
+use crate::model_table::ProviderTypeDefaultCredentials;
 use crate::optimization::JobHandle;
 use crate::optimization::OptimizationJobInfo;
 use crate::optimization::Optimizer;
@@ -39,6 +42,7 @@ use crate::providers::fireworks::prepare_fireworks_messages;
 use crate::providers::fireworks::FIREWORKS_API_BASE;
 use crate::providers::helpers::UrlParseErrExt;
 use crate::providers::openai::tensorzero_to_openai_assistant_message;
+use crate::providers::openai::OpenAIMessagesConfig;
 use crate::stored_inference::LazyRenderedSample;
 use crate::stored_inference::RenderedSample;
 use crate::{
@@ -46,12 +50,9 @@ use crate::{
     endpoints::inference::InferenceCredentials,
     error::{DisplayOrDebugGateway, Error, ErrorDetails},
     inference::types::ContentBlock,
-    model::{build_creds_caching_default, CredentialLocation},
+    model::CredentialLocationWithFallback,
     providers::{
-        fireworks::{
-            default_api_key_location, FireworksCredentials, FireworksTool, DEFAULT_CREDENTIALS,
-            PROVIDER_TYPE,
-        },
+        fireworks::{FireworksCredentials, FireworksTool, PROVIDER_TYPE},
         openai::OpenAIRequestMessage,
     },
 };
@@ -89,20 +90,32 @@ pub struct FireworksSupervisedRow<'a> {
 
 impl<'a> FireworksSupervisedRow<'a> {
     pub async fn from_rendered_sample(inference: &'a LazyRenderedSample) -> Result<Self, Error> {
-        let tools = match &inference.tool_params {
-            Some(tool_params) => {
-                if tool_params.parallel_tool_calls.unwrap_or_default() {
-                    return Err(Error::new(ErrorDetails::InvalidRenderedStoredInference {
-                        message: "Parallel tool calls are not supported for Fireworks".to_string(),
-                    }));
-                }
-                tool_params.tools_available.iter().map(Into::into).collect()
-            }
-            None => vec![],
-        };
-        let mut messages =
-            prepare_fireworks_messages(inference.system_input.as_deref(), &inference.messages)
-                .await?;
+        if inference
+            .tool_params
+            .parallel_tool_calls
+            .unwrap_or_default()
+        {
+            return Err(Error::new(ErrorDetails::InvalidRenderedStoredInference {
+                message: "Parallel tool calls are not supported for Fireworks".to_string(),
+            }));
+        }
+        let tools = inference
+            .tool_params
+            .additional_tools
+            .as_ref()
+            .map(|tools| tools.iter().map(Into::into).collect())
+            .unwrap_or_default();
+        let mut messages = prepare_fireworks_messages(
+            inference.system_input.as_deref(),
+            &inference.messages,
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                // For now, this isn't configurable in SFT (we should never need to resolve a file URL here)
+                fetch_and_encode_input_files_before_inference: true,
+            },
+        )
+        .await?;
 
         let Some(output) = &inference.output else {
             return Err(Error::new(ErrorDetails::InvalidRenderedStoredInference {
@@ -118,7 +131,12 @@ impl<'a> FireworksSupervisedRow<'a> {
             output.iter().map(|c| c.clone().into()).collect::<Vec<_>>();
         let final_assistant_message = tensorzero_to_openai_assistant_message(
             Cow::Owned(output_content_blocks),
-            PROVIDER_TYPE,
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                // For now, this isn't configurable in SFT (we should never need to resolve a file URL here)
+                fetch_and_encode_input_files_before_inference: true,
+            },
         )
         .await?;
         messages.push(final_assistant_message);
@@ -149,7 +167,7 @@ pub struct FireworksSFTConfig {
     #[serde(skip)]
     pub credentials: FireworksCredentials,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
     pub account_id: String,
     pub api_base: Url,
 }
@@ -176,7 +194,7 @@ pub struct UninitializedFireworksSFTConfig {
     pub mtp_num_draft_tokens: Option<usize>,
     pub mtp_freeze_base_model: Option<bool>,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credentials: Option<CredentialLocation>,
+    pub credentials: Option<CredentialLocationWithFallback>,
     pub account_id: String,
     pub api_base: Option<Url>,
 }
@@ -218,8 +236,7 @@ impl UninitializedFireworksSFTConfig {
         let credentials = credentials
             .map(|s| serde_json::from_str(&s))
             .transpose()
-            .map_err(|e| PyErr::new::<PyValueError, _>(format!("Invalid credentials JSON: {e}")))?
-            .or_else(|| Some(default_api_key_location()));
+            .map_err(|e| PyErr::new::<PyValueError, _>(format!("Invalid credentials JSON: {e}")))?;
         let api_base = api_base
             .map(|s| {
                 Url::parse(&s)
@@ -299,7 +316,10 @@ impl UninitializedFireworksSFTConfig {
 }
 
 impl UninitializedFireworksSFTConfig {
-    pub fn load(self) -> Result<FireworksSFTConfig, Error> {
+    pub async fn load(
+        self,
+        default_credentials: &ProviderTypeDefaultCredentials,
+    ) -> Result<FireworksSFTConfig, Error> {
         Ok(FireworksSFTConfig {
             model: self.model,
             early_stop: self.early_stop,
@@ -319,12 +339,9 @@ impl UninitializedFireworksSFTConfig {
             mtp_freeze_base_model: self.mtp_freeze_base_model,
             api_base: self.api_base.unwrap_or_else(|| FIREWORKS_API_BASE.clone()),
             account_id: self.account_id,
-            credentials: build_creds_caching_default(
-                self.credentials.clone(),
-                default_api_key_location(),
-                PROVIDER_TYPE,
-                &DEFAULT_CREDENTIALS,
-            )?,
+            credentials: FireworksKind
+                .get_defaulted_credential(self.credentials.as_ref(), default_credentials)
+                .await?,
             credential_location: self.credentials,
         })
     }
@@ -681,7 +698,7 @@ pub struct FireworksSFTJobHandle {
     pub job_url: Url,
     pub job_path: String,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
 }
 
 impl std::fmt::Display for FireworksSFTJobHandle {
@@ -953,13 +970,11 @@ impl JobHandle for FireworksSFTJobHandle {
         &self,
         client: &TensorzeroHttpClient,
         credentials: &InferenceCredentials,
+        default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<OptimizationJobInfo, Error> {
-        let fireworks_credentials = build_creds_caching_default(
-            self.credential_location.clone(),
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
+        let fireworks_credentials: FireworksCredentials = crate::model_table::FireworksKind
+            .get_defaulted_credential(self.credential_location.as_ref(), default_credentials)
+            .await?;
         let api_key = fireworks_credentials.get_api_key(credentials)?;
         let job_status = self.poll_job(client, api_key).await?;
         if let FireworksFineTuningJobState::JobStateCompleted = job_status.state {
