@@ -1,6 +1,10 @@
 #[cfg(feature = "pyo3")]
 use crate::inference::types::pyo3_helpers::deserialize_from_pyobj;
-use crate::{error::IMPOSSIBLE_ERROR_MESSAGE, http::TensorzeroHttpClient};
+use crate::{
+    error::IMPOSSIBLE_ERROR_MESSAGE,
+    http::TensorzeroHttpClient,
+    model_table::{OpenAIKind, ProviderKind, ProviderTypeDefaultCredentials},
+};
 use futures::future::try_join_all;
 #[cfg(feature = "pyo3")]
 use pyo3::exceptions::PyValueError;
@@ -16,21 +20,22 @@ use crate::{
     db::clickhouse::ClickHouseConnectionInfo,
     endpoints::inference::InferenceCredentials,
     error::{DisplayOrDebugGateway, Error, ErrorDetails},
-    model::{build_creds_caching_default, CredentialLocation},
+    model::CredentialLocationWithFallback,
     optimization::{JobHandle, OptimizationJobInfo, Optimizer},
     providers::openai::{
-        default_api_key_location,
         optimization::{
             convert_to_optimizer_status, OpenAIFineTuningJob, OpenAIFineTuningMethod,
             OpenAIFineTuningRequest, OpenAIGrader, OpenAIRFTResponseFormat, OpenAIReinforcementRow,
             Reinforcement, ReinforcementHyperparameters,
         },
-        upload_openai_file, OpenAICredentials, DEFAULT_CREDENTIALS, OPENAI_DEFAULT_BASE_URL,
-        PROVIDER_TYPE,
+        upload_openai_file, OpenAICredentials, OPENAI_DEFAULT_BASE_URL, PROVIDER_TYPE,
     },
     stored_inference::RenderedSample,
 };
 
+#[cfg(feature = "pyo3")]
+use crate::model::CredentialLocation;
+
 const OPENAI_FINE_TUNE_PURPOSE: &str = "fine-tune";
 
 #[derive(Debug, Clone, Serialize)]
@@ -50,7 +55,7 @@ pub struct OpenAIRFTConfig {
     #[serde(skip)]
     pub credentials: OpenAICredentials,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
     pub api_base: Option<Url>,
     pub seed: Option<u64>,
     pub suffix: Option<String>,
@@ -72,7 +77,7 @@ pub struct UninitializedOpenAIRFTConfig {
     pub n_epochs: Option<usize>,
     pub reasoning_effort: Option<String>,
     #[serde(skip)]
-    pub credentials: Option<CredentialLocation>,
+    pub credentials: Option<CredentialLocationWithFallback>,
     pub api_base: Option<Url>,
     pub seed: Option<u64>,
     pub suffix: Option<String>,
@@ -135,9 +140,12 @@ impl UninitializedOpenAIRFTConfig {
             None
         };
 
-        // Use Deserialize to convert the string to a CredentialLocation
-        let credentials =
-            credentials.map(|s| serde_json::from_str(&s).unwrap_or(CredentialLocation::Env(s)));
+        // Use Deserialize to convert the string to a CredentialLocationWithFallback
+        let credentials = credentials.map(|s| {
+            serde_json::from_str(&s).unwrap_or(CredentialLocationWithFallback::Single(
+                CredentialLocation::Env(s),
+            ))
+        });
         let api_base = api_base
             .map(|s| {
                 Url::parse(&s)
@@ -202,7 +210,10 @@ impl UninitializedOpenAIRFTConfig {
 }
 
 impl UninitializedOpenAIRFTConfig {
-    pub fn load(self) -> Result<OpenAIRFTConfig, Error> {
+    pub async fn load(
+        self,
+        default_credentials: &ProviderTypeDefaultCredentials,
+    ) -> Result<OpenAIRFTConfig, Error> {
         Ok(OpenAIRFTConfig {
             model: self.model,
             grader: self.grader,
@@ -214,12 +225,9 @@ impl UninitializedOpenAIRFTConfig {
             learning_rate_multiplier: self.learning_rate_multiplier,
             n_epochs: self.n_epochs,
             reasoning_effort: self.reasoning_effort,
-            credentials: build_creds_caching_default(
-                self.credentials.clone(),
-                default_api_key_location(),
-                PROVIDER_TYPE,
-                &DEFAULT_CREDENTIALS,
-            )?,
+            credentials: OpenAIKind
+                .get_defaulted_credential(self.credentials.as_ref(), default_credentials)
+                .await?,
             credential_location: self.credentials,
             api_base: self.api_base,
             suffix: self.suffix,
@@ -238,7 +246,7 @@ pub struct OpenAIRFTJobHandle {
     pub job_url: Url,
     pub job_api_url: Url,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
 }
 
 impl std::fmt::Display for OpenAIRFTJobHandle {
@@ -416,13 +424,11 @@ impl JobHandle for OpenAIRFTJobHandle {
         &self,
         client: &TensorzeroHttpClient,
         credentials: &InferenceCredentials,
+        default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<OptimizationJobInfo, Error> {
-        let openai_credentials = build_creds_caching_default(
-            self.credential_location.clone(),
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
+        let openai_credentials: OpenAICredentials = OpenAIKind
+            .get_defaulted_credential(None, default_credentials)
+            .await?;
         let mut request = client.get(self.job_api_url.clone());
         let api_key = openai_credentials.get_api_key(credentials)?;
         if let Some(api_key) = api_key {
