@@ -1,4 +1,8 @@
-use crate::{error::IMPOSSIBLE_ERROR_MESSAGE, http::TensorzeroHttpClient};
+use crate::{
+    error::IMPOSSIBLE_ERROR_MESSAGE,
+    http::TensorzeroHttpClient,
+    model_table::{OpenAIKind, ProviderKind, ProviderTypeDefaultCredentials},
+};
 use futures::future::try_join_all;
 #[cfg(feature = "pyo3")]
 use pyo3::exceptions::PyValueError;
@@ -14,16 +18,14 @@ use crate::{
     db::clickhouse::ClickHouseConnectionInfo,
     endpoints::inference::InferenceCredentials,
     error::{DisplayOrDebugGateway, Error, ErrorDetails},
-    model::{build_creds_caching_default, CredentialLocation},
+    model::CredentialLocationWithFallback,
     optimization::{JobHandle, OptimizationJobInfo, Optimizer},
     providers::openai::{
-        default_api_key_location,
         optimization::{
             convert_to_optimizer_status, OpenAIFineTuningJob, OpenAIFineTuningMethod,
             OpenAIFineTuningRequest, OpenAISupervisedRow, Supervised, SupervisedHyperparameters,
         },
-        upload_openai_file, OpenAICredentials, DEFAULT_CREDENTIALS, OPENAI_DEFAULT_BASE_URL,
-        PROVIDER_TYPE,
+        upload_openai_file, OpenAICredentials, OPENAI_DEFAULT_BASE_URL, PROVIDER_TYPE,
     },
     stored_inference::RenderedSample,
 };
@@ -41,7 +43,7 @@ pub struct OpenAISFTConfig {
     #[serde(skip)]
     pub credentials: OpenAICredentials,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
     pub seed: Option<u64>,
     pub suffix: Option<String>,
     pub api_base: Option<Url>,
@@ -57,7 +59,7 @@ pub struct UninitializedOpenAISFTConfig {
     pub learning_rate_multiplier: Option<f64>,
     pub n_epochs: Option<usize>,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credentials: Option<CredentialLocation>,
+    pub credentials: Option<CredentialLocationWithFallback>,
     pub api_base: Option<Url>,
     pub seed: Option<u64>,
     pub suffix: Option<String>,
@@ -96,8 +98,8 @@ impl UninitializedOpenAISFTConfig {
         let credentials = credentials
             .map(|s| serde_json::from_str(&s))
             .transpose()
-            .map_err(|e| PyErr::new::<PyValueError, _>(format!("Invalid credentials JSON: {e}")))?
-            .or_else(|| Some(default_api_key_location()));
+            .map_err(|e| PyErr::new::<PyValueError, _>(format!("Invalid credentials JSON: {e}")))?;
+
         let api_base = api_base
             .map(|s| {
                 Url::parse(&s)
@@ -144,19 +146,19 @@ impl UninitializedOpenAISFTConfig {
 }
 
 impl UninitializedOpenAISFTConfig {
-    pub fn load(self) -> Result<OpenAISFTConfig, Error> {
+    pub async fn load(
+        self,
+        default_credentials: &ProviderTypeDefaultCredentials,
+    ) -> Result<OpenAISFTConfig, Error> {
         Ok(OpenAISFTConfig {
             model: self.model,
             api_base: self.api_base,
             batch_size: self.batch_size,
             learning_rate_multiplier: self.learning_rate_multiplier,
             n_epochs: self.n_epochs,
-            credentials: build_creds_caching_default(
-                self.credentials.clone(),
-                default_api_key_location(),
-                PROVIDER_TYPE,
-                &DEFAULT_CREDENTIALS,
-            )?,
+            credentials: OpenAIKind
+                .get_defaulted_credential(self.credentials.as_ref(), default_credentials)
+                .await?,
             credential_location: self.credentials,
             seed: self.seed,
             suffix: self.suffix,
@@ -174,7 +176,7 @@ pub struct OpenAISFTJobHandle {
     pub job_url: Url,
     pub job_api_url: Url,
     #[cfg_attr(test, ts(type = "string | null"))]
-    pub credential_location: Option<CredentialLocation>,
+    pub credential_location: Option<CredentialLocationWithFallback>,
 }
 
 impl std::fmt::Display for OpenAISFTJobHandle {
@@ -346,13 +348,11 @@ impl JobHandle for OpenAISFTJobHandle {
         &self,
         client: &TensorzeroHttpClient,
         credentials: &InferenceCredentials,
+        default_credentials: &ProviderTypeDefaultCredentials,
     ) -> Result<OptimizationJobInfo, Error> {
-        let openai_credentials = build_creds_caching_default(
-            self.credential_location.clone(),
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
+        let openai_credentials: OpenAICredentials = OpenAIKind
+            .get_defaulted_credential(self.credential_location.as_ref(), default_credentials)
+            .await?;
         let mut request = client.get(self.job_api_url.clone());
         let api_key = openai_credentials.get_api_key(credentials)?;
         if let Some(api_key) = api_key {
