@@ -16,6 +16,9 @@ use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{DisplayOrDebugGateway, Error, ErrorDetails};
 use crate::http::{TensorZeroEventSource, TensorzeroHttpClient};
 use crate::inference::types::batch::{BatchRequestRow, PollBatchInferenceResponse};
+use crate::inference::types::chat_completion_inference_params::{
+    warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+};
 use crate::inference::types::{
     batch::StartBatchProviderInferenceResponse, Latency, ModelInferenceRequest,
     ModelInferenceRequestJsonMode, ProviderInferenceResponse,
@@ -28,15 +31,17 @@ use crate::inference::types::{
     PeekableProviderInferenceResponseStream, ProviderInferenceResponseChunk,
 };
 use crate::inference::InferenceProvider;
-use crate::model::{Credential, CredentialLocation, ModelProvider};
+use crate::model::{Credential, ModelProvider};
+use crate::providers::openai::OpenAIMessagesConfig;
 use crate::tool::ToolCallChunk;
 
+use super::helpers::convert_stream_error;
 use super::openai::{
-    convert_stream_error, get_chat_url, handle_openai_error, prepare_openai_tools,
-    prepare_system_or_developer_message, tensorzero_to_openai_messages,
-    OpenAIAssistantRequestMessage, OpenAIContentBlock, OpenAIFinishReason, OpenAIRequestMessage,
-    OpenAIResponseToolCall, OpenAISystemRequestMessage, OpenAITool, OpenAIToolChoice, OpenAIUsage,
-    OpenAIUserRequestMessage, StreamOptions, SystemOrDeveloper,
+    get_chat_url, handle_openai_error, prepare_openai_tools, prepare_system_or_developer_message,
+    tensorzero_to_openai_messages, OpenAIAssistantRequestMessage, OpenAIContentBlock,
+    OpenAIFinishReason, OpenAIRequestMessage, OpenAIResponseToolCall, OpenAISystemRequestMessage,
+    OpenAITool, OpenAIToolChoice, OpenAIUsage, OpenAIUserRequestMessage, StreamOptions,
+    SystemOrDeveloper,
 };
 
 lazy_static! {
@@ -47,18 +52,18 @@ lazy_static! {
     };
 }
 
-fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("DEEPSEEK_API_KEY".to_string())
-}
-
 const PROVIDER_NAME: &str = "DeepSeek";
 pub const PROVIDER_TYPE: &str = "deepseek";
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub enum DeepSeekCredentials {
     Static(SecretString),
     Dynamic(String),
     None,
+    WithFallback {
+        default: Box<DeepSeekCredentials>,
+        fallback: Box<DeepSeekCredentials>,
+    },
 }
 
 impl TryFrom<Credential> for DeepSeekCredentials {
@@ -69,6 +74,12 @@ impl TryFrom<Credential> for DeepSeekCredentials {
             Credential::Static(key) => Ok(DeepSeekCredentials::Static(key)),
             Credential::Dynamic(key_name) => Ok(DeepSeekCredentials::Dynamic(key_name)),
             Credential::Missing => Ok(DeepSeekCredentials::None),
+            Credential::WithFallback { default, fallback } => {
+                Ok(DeepSeekCredentials::WithFallback {
+                    default: Box::new((*default).try_into()?),
+                    fallback: Box::new((*fallback).try_into()?),
+                })
+            }
             _ => Err(Error::new(ErrorDetails::Config {
                 message: "Invalid api_key_location for DeepSeek provider".to_string(),
             })),
@@ -92,6 +103,16 @@ impl DeepSeekCredentials {
                     .into()
                 })
             }
+            DeepSeekCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             DeepSeekCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
                 message: "No credentials are set".to_string(),
@@ -111,18 +132,11 @@ pub struct DeepSeekProvider {
 }
 
 impl DeepSeekProvider {
-    pub fn new(
-        model_name: String,
-        api_key_location: Option<CredentialLocation>,
-    ) -> Result<Self, Error> {
-        let credential_location = api_key_location.unwrap_or_else(default_api_key_location);
-        let generic_credentials = Credential::try_from((credential_location, PROVIDER_TYPE))?;
-        let provider_credentials = DeepSeekCredentials::try_from(generic_credentials)?;
-
-        Ok(DeepSeekProvider {
+    pub fn new(model_name: String, credentials: DeepSeekCredentials) -> Self {
+        DeepSeekProvider {
             model_name,
-            credentials: provider_credentials,
-        })
+            credentials,
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -322,7 +336,7 @@ impl DeepSeekResponseFormat {
     }
 }
 
-#[derive(Debug, Serialize)]
+#[derive(Debug, Default, Serialize)]
 struct DeepSeekRequest<'a> {
     messages: Vec<OpenAIRequestMessage<'a>>,
     model: &'a str,
@@ -352,6 +366,29 @@ struct DeepSeekRequest<'a> {
     response_format: Option<DeepSeekResponseFormat>,
 }
 
+fn apply_inference_params(
+    _request: &mut DeepSeekRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "reasoning_effort", None);
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "thinking_budget_tokens", None);
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
+}
+
 impl<'a> DeepSeekRequest<'a> {
     pub async fn new(
         model: &'a str,
@@ -385,11 +422,21 @@ impl<'a> DeepSeekRequest<'a> {
         // NOTE: as mentioned by the DeepSeek team here: https://github.com/deepseek-ai/DeepSeek-R1?tab=readme-ov-file#usage-recommendations
         // the R1 series of models does not perform well with the system prompt. As we move towards first-class support for reasoning models we should check
         // if a model is an R1 model and if so, remove the system prompt from the request and instead put it in the first user message.
-        let messages = prepare_deepseek_messages(request, model).await?;
+        let messages = prepare_deepseek_messages(
+            request,
+            model,
+            OpenAIMessagesConfig {
+                json_mode: Some(&request.json_mode),
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: request
+                    .fetch_and_encode_input_files_before_inference,
+            },
+        )
+        .await?;
 
         let (tools, tool_choice, _) = prepare_openai_tools(request);
 
-        Ok(DeepSeekRequest {
+        let mut deepseek_request = DeepSeekRequest {
             messages,
             model,
             temperature,
@@ -404,7 +451,11 @@ impl<'a> DeepSeekRequest<'a> {
             response_format,
             tools,
             tool_choice,
-        })
+        };
+
+        apply_inference_params(&mut deepseek_request, &request.inference_params_v2);
+
+        Ok(deepseek_request)
     }
 }
 
@@ -526,6 +577,8 @@ fn deepseek_to_tensorzero_chunk(
                 text: Some(reasoning),
                 signature: None,
                 id: "0".to_string(),
+                summary_id: None,
+                summary_text: None,
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             }));
         }
@@ -596,10 +649,11 @@ struct DeepSeekResponse {
 pub(super) async fn prepare_deepseek_messages<'a>(
     request: &'a ModelInferenceRequest<'_>,
     model_name: &'a str,
+    config: OpenAIMessagesConfig<'a>,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut messages = Vec::with_capacity(request.messages.len());
     for message in &request.messages {
-        messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE).await?);
+        messages.extend(tensorzero_to_openai_messages(message, config).await?);
     }
     // If this is an R1 model, prepend the system message as the first user message instead of using it as a system message
     if model_name.to_lowercase().contains("reasoner") {
@@ -614,7 +668,10 @@ pub(super) async fn prepare_deepseek_messages<'a>(
             );
         }
     } else if let Some(system_msg) = prepare_system_or_developer_message(
-        request.system.as_deref().map(SystemOrDeveloper::System),
+        request
+            .system
+            .as_deref()
+            .map(|m| SystemOrDeveloper::System(Cow::Borrowed(m))),
         Some(&request.json_mode),
         &messages,
     ) {
@@ -675,6 +732,7 @@ impl<'a> TryFrom<DeepSeekResponseWithMetadata<'a>> for ProviderInferenceResponse
             content.push(ContentBlockOutput::Thought(Thought {
                 text: Some(reasoning),
                 signature: None,
+                summary: None,
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             }));
         }
@@ -765,6 +823,7 @@ mod tests {
     use super::*;
     use std::borrow::Cow;
     use std::time::Duration;
+    use tracing_test::traced_test;
     use uuid::Uuid;
 
     use crate::inference::types::{
@@ -939,7 +998,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -986,6 +1044,7 @@ mod tests {
             ContentBlockOutput::Thought(Thought {
                 text: Some("I'm thinking about the weather".to_string()),
                 signature: None,
+                summary: None,
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             })
         );
@@ -1031,17 +1090,33 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request, "deepseek-chat")
-            .await
-            .unwrap();
+        let messages = prepare_deepseek_messages(
+            &request,
+            "deepseek-chat",
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: false,
+            },
+        )
+        .await
+        .unwrap();
         assert_eq!(messages.len(), 2);
         assert!(matches!(messages[0], OpenAIRequestMessage::System(_)));
         assert!(matches!(messages[1], OpenAIRequestMessage::User(_)));
 
         // Test case 2: Reasoner model with system message
-        let messages = prepare_deepseek_messages(&request, "deepseek-reasoner")
-            .await
-            .unwrap();
+        let messages = prepare_deepseek_messages(
+            &request,
+            "deepseek-reasoner",
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: false,
+            },
+        )
+        .await
+        .unwrap();
         assert_eq!(messages.len(), 1);
         match &messages[0] {
             OpenAIRequestMessage::User(user_msg) => {
@@ -1083,9 +1158,17 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request_no_system, "deepseek-chat")
-            .await
-            .unwrap();
+        let messages = prepare_deepseek_messages(
+            &request_no_system,
+            "deepseek-chat",
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: false,
+            },
+        )
+        .await
+        .unwrap();
         assert_eq!(messages.len(), 1);
         assert!(matches!(messages[0], OpenAIRequestMessage::User(_)));
 
@@ -1122,9 +1205,17 @@ mod tests {
             ..Default::default()
         };
 
-        let messages = prepare_deepseek_messages(&request_multiple, "deepseek-chat")
-            .await
-            .unwrap();
+        let messages = prepare_deepseek_messages(
+            &request_multiple,
+            "deepseek-chat",
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: false,
+            },
+        )
+        .await
+        .unwrap();
         assert_eq!(messages.len(), 4);
         assert!(matches!(messages[0], OpenAIRequestMessage::System(_)));
         assert!(matches!(messages[1], OpenAIRequestMessage::User(_)));
@@ -1161,6 +1252,34 @@ mod tests {
         })
     }
 
+    #[test]
+    #[traced_test]
+    fn test_deepseek_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = DeepSeekRequest::default();
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort warns
+        assert!(logs_contain(
+            "DeepSeek does not support the inference parameter `reasoning_effort`"
+        ));
+
+        // Test that thinking_budget_tokens warns
+        assert!(logs_contain(
+            "DeepSeek does not support the inference parameter `thinking_budget_tokens`"
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "DeepSeek does not support the inference parameter `verbosity`"
+        ));
+    }
+
     #[tokio::test]
     async fn test_coalesce_consecutive_messages() {
         // Create dummy tool calls to test assistant message merging.
