@@ -1,6 +1,10 @@
-use std::{borrow::Cow, sync::OnceLock};
+use std::borrow::Cow;
 
 use crate::http::TensorzeroHttpClient;
+use crate::inference::types::chat_completion_inference_params::{
+    warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+};
+use crate::providers::openai::OpenAIMessagesConfig;
 use crate::{
     http::TensorZeroEventSource, providers::helpers_thinking_block::THINK_CHUNK_ID, tool::Tool,
 };
@@ -34,7 +38,7 @@ use crate::{
         },
         InferenceProvider,
     },
-    model::{build_creds_caching_default, Credential, CredentialLocation, ModelProvider},
+    model::{Credential, ModelProvider},
     tool::{ToolCall, ToolCallChunk},
 };
 
@@ -72,25 +76,17 @@ pub struct FireworksProvider {
     parse_think_blocks: bool,
 }
 
-pub static DEFAULT_CREDENTIALS: OnceLock<FireworksCredentials> = OnceLock::new();
-
 impl FireworksProvider {
     pub fn new(
         model_name: String,
-        api_key_location: Option<CredentialLocation>,
+        credentials: FireworksCredentials,
         parse_think_blocks: bool,
-    ) -> Result<Self, Error> {
-        let credentials = build_creds_caching_default(
-            api_key_location,
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
-        Ok(FireworksProvider {
+    ) -> Self {
+        FireworksProvider {
             model_name,
             credentials,
             parse_think_blocks,
-        })
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -107,6 +103,10 @@ pub enum FireworksCredentials {
     Static(SecretString),
     Dynamic(String),
     None,
+    WithFallback {
+        default: Box<FireworksCredentials>,
+        fallback: Box<FireworksCredentials>,
+    },
 }
 
 impl TryFrom<Credential> for FireworksCredentials {
@@ -117,6 +117,12 @@ impl TryFrom<Credential> for FireworksCredentials {
             Credential::Static(key) => Ok(FireworksCredentials::Static(key)),
             Credential::Dynamic(key_name) => Ok(FireworksCredentials::Dynamic(key_name)),
             Credential::Missing => Ok(FireworksCredentials::None),
+            Credential::WithFallback { default, fallback } => {
+                Ok(FireworksCredentials::WithFallback {
+                    default: Box::new((*default).try_into()?),
+                    fallback: Box::new((*fallback).try_into()?),
+                })
+            }
             _ => Err(Error::new(ErrorDetails::Config {
                 message: "Invalid api_key_location for Fireworks provider".to_string(),
             })),
@@ -140,6 +146,16 @@ impl FireworksCredentials {
                     .into()
                 })
             }
+            FireworksCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             &FireworksCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
                 message: "No credentials are set".to_string(),
@@ -149,10 +165,6 @@ impl FireworksCredentials {
     }
 }
 
-pub fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("FIREWORKS_API_KEY".to_string())
-}
-
 /// Key differences between Fireworks and OpenAI inference:
 /// - Fireworks allows you to specify output format in JSON mode
 /// - Fireworks automatically returns usage in streaming inference, we don't have to ask for it
@@ -340,7 +352,7 @@ enum FireworksResponseFormat<'a> {
 /// presence_penalty, frequency_penalty, service_tier, stop, user,
 /// or context_length_exceeded_behavior.
 /// NOTE: Fireworks does not support seed.
-#[derive(Debug, Serialize)]
+#[derive(Debug, Default, Serialize)]
 struct FireworksRequest<'a> {
     messages: Vec<OpenAIRequestMessage<'a>>,
     model: &'a str,
@@ -363,6 +375,35 @@ struct FireworksRequest<'a> {
     tools: Option<Vec<FireworksTool<'a>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
     tool_choice: Option<OpenAIToolChoice<'a>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    reasoning_effort: Option<String>,
+}
+
+fn apply_inference_params(
+    request: &mut FireworksRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        request.reasoning_effort = reasoning_effort.clone();
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(
+            PROVIDER_NAME,
+            "thinking_budget_tokens",
+            Some("Tip: You might want to use `reasoning_effort` for this provider."),
+        );
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
 }
 
 impl<'a> FireworksRequest<'a> {
@@ -380,12 +421,21 @@ impl<'a> FireworksRequest<'a> {
             }
             ModelInferenceRequestJsonMode::Off => None,
         };
-        let messages =
-            prepare_fireworks_messages(request.system.as_deref(), &request.messages).await?;
+        let messages = prepare_fireworks_messages(
+            request.system.as_deref(),
+            &request.messages,
+            OpenAIMessagesConfig {
+                json_mode: Some(&request.json_mode),
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: request
+                    .fetch_and_encode_input_files_before_inference,
+            },
+        )
+        .await?;
         let (tools, tool_choice, _) = prepare_openai_tools(request);
         let tools = tools.map(|t| t.into_iter().map(OpenAITool::into).collect());
 
-        Ok(FireworksRequest {
+        let mut fireworks_request = FireworksRequest {
             messages,
             model,
             temperature: request.temperature,
@@ -398,17 +448,23 @@ impl<'a> FireworksRequest<'a> {
             response_format,
             tools,
             tool_choice,
-        })
+            reasoning_effort: None,
+        };
+
+        apply_inference_params(&mut fireworks_request, &request.inference_params_v2);
+
+        Ok(fireworks_request)
     }
 }
 
 pub async fn prepare_fireworks_messages<'a>(
     system: Option<&'a str>,
     messages: &'a [RequestMessage],
+    config: OpenAIMessagesConfig<'a>,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut output_messages = Vec::with_capacity(messages.len());
     for message in messages {
-        output_messages.extend(tensorzero_to_openai_messages(message, PROVIDER_TYPE).await?);
+        output_messages.extend(tensorzero_to_openai_messages(message, config).await?);
     }
     if let Some(system_msg) = tensorzero_to_fireworks_system_message(system) {
         output_messages.insert(0, system_msg);
@@ -650,6 +706,8 @@ fn fireworks_to_tensorzero_chunk(
             content.push(ContentBlockChunk::Thought(ThoughtChunk {
                 text: Some(reasoning),
                 signature: None,
+                summary_id: None,
+                summary_text: None,
                 id: THINK_CHUNK_ID.to_string(),
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             }));
@@ -668,6 +726,8 @@ fn fireworks_to_tensorzero_chunk(
                             content.push(ContentBlockChunk::Thought(ThoughtChunk {
                                 text: Some(text.to_string()),
                                 signature: None,
+                                summary_id: None,
+                                summary_text: None,
                                 id: thinking_state.get_id(),
                                 provider_type: Some(PROVIDER_TYPE.to_string()),
                             }));
@@ -773,6 +833,7 @@ impl<'a> TryFrom<FireworksResponseWithMetadata<'a>> for ProviderInferenceRespons
             content.push(ContentBlockOutput::Thought(Thought {
                 text: Some(reasoning),
                 signature: None,
+                summary: None,
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             }));
         }
@@ -783,6 +844,7 @@ impl<'a> TryFrom<FireworksResponseWithMetadata<'a>> for ProviderInferenceRespons
                 content.push(ContentBlockOutput::Thought(Thought {
                     text: Some(reasoning),
                     signature: None,
+                    summary: None,
                     provider_type: Some(PROVIDER_TYPE.to_string()),
                 }));
             }
@@ -816,6 +878,7 @@ impl<'a> TryFrom<FireworksResponseWithMetadata<'a>> for ProviderInferenceRespons
 mod tests {
     use std::borrow::Cow;
     use std::time::Duration;
+    use tracing_test::traced_test;
 
     use uuid::Uuid;
 
@@ -865,7 +928,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
 
@@ -1053,7 +1115,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -1190,7 +1251,6 @@ mod tests {
             usage: Some(OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             }),
         };
         let message = fireworks_to_tensorzero_chunk(
@@ -1464,4 +1524,30 @@ mod tests {
             panic!("Expected a tool call chunk");
         }
     }
+
+    #[test]
+    #[traced_test]
+    fn test_fireworks_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = FireworksRequest::default();
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort is applied correctly
+        assert_eq!(request.reasoning_effort, Some("high".to_string()));
+
+        // Test that thinking_budget_tokens warns with tip about reasoning_effort
+        assert!(logs_contain(
+            "Fireworks does not support the inference parameter `thinking_budget_tokens`, so it will be ignored. Tip: You might want to use `reasoning_effort` for this provider."
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "Fireworks does not support the inference parameter `verbosity`"
+        ));
+    }
 }
