@@ -5,9 +5,8 @@ use reqwest_eventsource::Event;
 use secrecy::{ExposeSecret, SecretString};
 use serde::de::IntoDeserializer;
 use serde::{Deserialize, Deserializer, Serialize};
-use serde_json::Value;
+use serde_json::{json, Value};
 use std::borrow::Cow;
-use std::sync::OnceLock;
 use std::time::Duration;
 use tokio::time::Instant;
 
@@ -16,28 +15,27 @@ use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{warn_discarded_thought_block, DisplayOrDebugGateway, Error, ErrorDetails};
 use crate::http::TensorzeroHttpClient;
 use crate::inference::types::batch::{BatchRequestRow, PollBatchInferenceResponse};
-use crate::inference::types::resolved_input::FileWithPath;
+use crate::inference::types::chat_completion_inference_params::{
+    warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+};
 use crate::inference::types::{
     batch::StartBatchProviderInferenceResponse, ContentBlock, ContentBlockChunk,
     ContentBlockOutput, Latency, ModelInferenceRequest, ModelInferenceRequestJsonMode,
-    PeekableProviderInferenceResponseStream, ProviderInferenceResponse,
+    ObjectStorageFile, PeekableProviderInferenceResponseStream, ProviderInferenceResponse,
     ProviderInferenceResponseChunk, RequestMessage, Role, Text, TextChunk, Usage,
 };
 use crate::inference::types::{
     FinishReason, ProviderInferenceResponseArgs, ProviderInferenceResponseStreamInner,
 };
 use crate::inference::{InferenceProvider, TensorZeroEventError};
-use crate::model::{build_creds_caching_default, Credential, CredentialLocation, ModelProvider};
+use crate::model::{Credential, ModelProvider};
 use crate::tool::{ToolCall, ToolCallChunk, ToolChoice, ToolConfig};
 
 use crate::providers::helpers::{
-    inject_extra_request_data_and_send, inject_extra_request_data_and_send_eventsource,
+    convert_stream_error, inject_extra_request_data_and_send,
+    inject_extra_request_data_and_send_eventsource,
 };
 
-fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("GROQ_API_KEY".to_string())
-}
-
 const PROVIDER_NAME: &str = "Groq";
 pub const PROVIDER_TYPE: &str = "groq";
 
@@ -50,23 +48,12 @@ pub struct GroqProvider {
     credentials: GroqCredentials,
 }
 
-static DEFAULT_CREDENTIALS: OnceLock<GroqCredentials> = OnceLock::new();
-
 impl GroqProvider {
-    pub fn new(
-        model_name: String,
-        api_key_location: Option<CredentialLocation>,
-    ) -> Result<Self, Error> {
-        let credentials = build_creds_caching_default(
-            api_key_location,
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
-        Ok(GroqProvider {
+    pub fn new(model_name: String, credentials: GroqCredentials) -> Self {
+        GroqProvider {
             model_name,
             credentials,
-        })
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -79,6 +66,10 @@ pub enum GroqCredentials {
     Static(SecretString),
     Dynamic(String),
     None,
+    WithFallback {
+        default: Box<GroqCredentials>,
+        fallback: Box<GroqCredentials>,
+    },
 }
 
 impl TryFrom<Credential> for GroqCredentials {
@@ -90,6 +81,10 @@ impl TryFrom<Credential> for GroqCredentials {
             Credential::Dynamic(key_name) => Ok(GroqCredentials::Dynamic(key_name)),
             Credential::None => Ok(GroqCredentials::None),
             Credential::Missing => Ok(GroqCredentials::None),
+            Credential::WithFallback { default, fallback } => Ok(GroqCredentials::WithFallback {
+                default: Box::new((*default).try_into()?),
+                fallback: Box::new((*fallback).try_into()?),
+            }),
             _ => Err(Error::new(ErrorDetails::Config {
                 message: "Invalid api_key_location for Groq provider".to_string(),
             })),
@@ -114,6 +109,16 @@ impl GroqCredentials {
                 }))
                 .transpose()
             }
+            GroqCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             GroqCredentials::None => Ok(None),
         }
     }
@@ -286,20 +291,6 @@ impl InferenceProvider for GroqProvider {
         .into())
     }
 }
-pub async fn convert_stream_error(provider_type: String, e: reqwest_eventsource::Error) -> Error {
-    let message = e.to_string();
-    let mut raw_response = None;
-    if let reqwest_eventsource::Error::InvalidStatusCode(_, resp) = e {
-        raw_response = resp.text().await.ok();
-    }
-    ErrorDetails::InferenceServer {
-        message,
-        raw_request: None,
-        raw_response,
-        provider_type,
-    }
-    .into()
-}
 
 pub fn stream_groq(
     provider_type: String,
@@ -560,10 +551,10 @@ pub(super) fn prepare_groq_tools<'a>(
     match &request.tool_config {
         None => (None, None, None),
         Some(tool_config) => {
-            if tool_config.tools_available.is_empty() {
+            if !tool_config.any_tools_available() {
                 return (None, None, None);
             }
-            let tools = Some(tool_config.tools_available.iter().map(Into::into).collect());
+            let tools = Some(tool_config.tools_available().map(Into::into).collect());
             let tool_choice = Some((&tool_config.tool_choice).into());
             let parallel_tool_calls = tool_config.parallel_tool_calls;
             (tools, tool_choice, parallel_tool_calls)
@@ -653,16 +644,11 @@ async fn tensorzero_to_groq_user_messages(
                 }));
             }
             ContentBlock::File(file) => {
-                let file = file.resolve().await?;
-                let FileWithPath {
-                    file,
-                    storage_path: _,
-                } = &*file;
+                let resolved_file = file.resolve().await?;
+                let ObjectStorageFile { file, data } = &*resolved_file;
                 user_content_blocks.push(GroqContentBlock::ImageUrl {
                     image_url: GroqImageUrl {
-                        // This will only produce an error if we pass in a bad
-                        // `Base64Image` (with missing image data)
-                        url: format!("data:{};base64,{}", file.mime_type, file.data()?),
+                        url: format!("data:{};base64,{}", file.mime_type, data),
                     },
                 });
             }
@@ -723,15 +709,10 @@ async fn tensorzero_to_groq_assistant_messages(
             }
             ContentBlock::File(file) => {
                 let resolved_file = file.resolve().await?;
-                let FileWithPath {
-                    file,
-                    storage_path: _,
-                } = &*resolved_file;
+                let ObjectStorageFile { file, data } = &*resolved_file;
                 assistant_content_blocks.push(GroqContentBlock::ImageUrl {
                     image_url: GroqImageUrl {
-                        // This will only produce an error if we pass in a bad
-                        // `Base64Image` (with missing image data)
-                        url: format!("data:{};base64,{}", file.mime_type, file.data()?),
+                        url: format!("data:{};base64,{}", file.mime_type, data),
                     },
                 });
             }
@@ -770,24 +751,27 @@ async fn tensorzero_to_groq_assistant_messages(
 #[derive(Clone, Debug, Default, PartialEq, Serialize)]
 #[serde(rename_all = "snake_case")]
 #[serde(tag = "type")]
-enum GroqResponseFormat<'a> {
+enum GroqResponseFormat {
     #[default]
     Text,
-    JsonObject {
-        #[serde(skip_serializing_if = "Option::is_none")]
-        schema: Option<&'a Value>, // the desired JSON schema
+    JsonObject,
+    JsonSchema {
+        json_schema: Value,
     },
 }
 
-impl<'a> GroqResponseFormat<'a> {
-    fn new(json_mode: ModelInferenceRequestJsonMode, output_schema: Option<&'a Value>) -> Self {
+impl GroqResponseFormat {
+    fn new(json_mode: ModelInferenceRequestJsonMode, output_schema: Option<&Value>) -> Self {
         match json_mode {
-            ModelInferenceRequestJsonMode::On | ModelInferenceRequestJsonMode::Strict => {
-                GroqResponseFormat::JsonObject {
-                    schema: output_schema,
-                }
-            }
+            ModelInferenceRequestJsonMode::On => GroqResponseFormat::JsonObject,
             ModelInferenceRequestJsonMode::Off => GroqResponseFormat::Text,
+            ModelInferenceRequestJsonMode::Strict => match output_schema {
+                Some(schema) => {
+                    let json_schema = json!({"name": "response", "strict": true, "schema": schema});
+                    GroqResponseFormat::JsonSchema { json_schema }
+                }
+                None => GroqResponseFormat::JsonObject,
+            },
         }
     }
 }
@@ -884,7 +868,7 @@ pub(super) struct StreamOptions {
 /// We are not handling logprobs, top_logprobs, n,
 /// presence_penalty, seed, service_tier, stop, user,
 /// or the deprecated function_call and functions arguments.
-#[derive(Debug, Serialize)]
+#[derive(Debug, Default, Serialize)]
 struct GroqRequest<'a> {
     messages: Vec<GroqRequestMessage<'a>>,
     model: &'a str,
@@ -904,7 +888,7 @@ struct GroqRequest<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     stream_options: Option<StreamOptions>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    response_format: Option<GroqResponseFormat<'a>>,
+    response_format: Option<GroqResponseFormat>,
     #[serde(skip_serializing_if = "Option::is_none")]
     tools: Option<Vec<GroqTool<'a>>>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -913,6 +897,35 @@ struct GroqRequest<'a> {
     parallel_tool_calls: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
     stop: Option<Cow<'a, [String]>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    reasoning_effort: Option<String>,
+}
+
+fn apply_inference_params(
+    request: &mut GroqRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        request.reasoning_effort = reasoning_effort.clone();
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(
+            PROVIDER_NAME,
+            "thinking_budget_tokens",
+            Some("Tip: You might want to use `reasoning_effort` for this provider."),
+        );
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
 }
 
 impl<'a> GroqRequest<'a> {
@@ -951,7 +964,7 @@ impl<'a> GroqRequest<'a> {
             }
         }
 
-        Ok(GroqRequest {
+        let mut groq_request = GroqRequest {
             messages,
             model,
             temperature: request.temperature,
@@ -967,7 +980,12 @@ impl<'a> GroqRequest<'a> {
             tool_choice,
             parallel_tool_calls,
             stop: request.borrow_stop_sequences(),
-        })
+            reasoning_effort: None,
+        };
+
+        apply_inference_params(&mut groq_request, &request.inference_params_v2);
+
+        Ok(groq_request)
     }
 }
 
@@ -976,7 +994,6 @@ pub(super) struct GroqUsage {
     pub prompt_tokens: u32,
     #[serde(default)]
     pub completion_tokens: u32,
-    pub total_tokens: u32,
 }
 
 impl From<GroqUsage> for Usage {
@@ -1272,10 +1289,10 @@ fn groq_to_tensorzero_chunk(
 }
 #[cfg(test)]
 mod tests {
-
-    use std::borrow::Cow;
-
+    use super::*;
     use serde_json::json;
+    use std::borrow::Cow;
+    use tracing_test::traced_test;
 
     use crate::{
         inference::types::{FunctionType, RequestMessage},
@@ -1285,8 +1302,6 @@ mod tests {
         tool::ToolCallConfig,
     };
 
-    use super::*;
-
     #[test]
     fn test_handle_groq_error() {
         use reqwest::StatusCode;
@@ -1475,7 +1490,7 @@ mod tests {
         assert!(!groq_request.stream);
         assert_eq!(
             groq_request.response_format,
-            Some(GroqResponseFormat::JsonObject { schema: None })
+            Some(GroqResponseFormat::JsonObject)
         );
         assert!(groq_request.tools.is_some());
         let tools = groq_request.tools.as_ref().unwrap();
@@ -1536,7 +1551,7 @@ mod tests {
         // Resolves to normal JSON mode since no schema is provided (this shouldn't really happen in practice)
         assert_eq!(
             groq_request.response_format,
-            Some(GroqResponseFormat::JsonObject { schema: None })
+            Some(GroqResponseFormat::JsonObject)
         );
 
         // Test request with strict JSON mode with an output schema
@@ -1583,12 +1598,14 @@ mod tests {
         assert_eq!(groq_request.presence_penalty, None);
         assert_eq!(groq_request.frequency_penalty, None);
         let expected_schema = serde_json::json!({});
-        assert_eq!(
-            groq_request.response_format,
-            Some(GroqResponseFormat::JsonObject {
-                schema: Some(&expected_schema),
-            })
-        );
+        match groq_request.response_format {
+            Some(GroqResponseFormat::JsonSchema { json_schema }) => {
+                assert_eq!(json_schema["schema"], expected_schema);
+                assert_eq!(json_schema["name"], "response");
+                assert_eq!(json_schema["strict"], true);
+            }
+            _ => panic!("Expected JsonSchema variant"),
+        }
     }
 
     #[test]
@@ -1606,7 +1623,6 @@ mod tests {
             usage: GroqUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -1647,6 +1663,7 @@ mod tests {
             tool_choice: None,
             parallel_tool_calls: None,
             stop: None,
+            reasoning_effort: None,
         };
         let raw_request = serde_json::to_string(&request_body).unwrap();
         let raw_response = "test_response".to_string();
@@ -1704,7 +1721,6 @@ mod tests {
             usage: GroqUsage {
                 prompt_tokens: 15,
                 completion_tokens: 25,
-                total_tokens: 40,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -1745,6 +1761,7 @@ mod tests {
             tool_choice: None,
             parallel_tool_calls: None,
             stop: None,
+            reasoning_effort: None,
         };
         let raw_request = serde_json::to_string(&request_body).unwrap();
         let result = ProviderInferenceResponse::try_from(GroqResponseWithMetadata {
@@ -1794,7 +1811,6 @@ mod tests {
             usage: GroqUsage {
                 prompt_tokens: 5,
                 completion_tokens: 0,
-                total_tokens: 5,
             },
         };
         let request_body = GroqRequest {
@@ -1813,6 +1829,7 @@ mod tests {
             tool_choice: None,
             parallel_tool_calls: None,
             stop: None,
+            reasoning_effort: None,
         };
         let result = ProviderInferenceResponse::try_from(GroqResponseWithMetadata {
             response: invalid_response_no_choices,
@@ -1851,7 +1868,6 @@ mod tests {
             usage: GroqUsage {
                 prompt_tokens: 10,
                 completion_tokens: 10,
-                total_tokens: 20,
             },
         };
 
@@ -1871,6 +1887,7 @@ mod tests {
             tool_choice: None,
             parallel_tool_calls: None,
             stop: None,
+            reasoning_effort: None,
         };
         let result = ProviderInferenceResponse::try_from(GroqResponseWithMetadata {
             response: invalid_response_multiple_choices,
@@ -1925,9 +1942,9 @@ mod tests {
         let parallel_tool_calls = parallel_tool_calls.unwrap();
         assert!(parallel_tool_calls);
         let tool_config = ToolCallConfig {
-            tools_available: vec![],
             tool_choice: ToolChoice::Required,
             parallel_tool_calls: Some(true),
+            ..Default::default()
         };
 
         // Test no tools but a tool choice and make sure tool choice output is None
@@ -2159,7 +2176,6 @@ mod tests {
             usage: Some(GroqUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             }),
         };
         let message =
@@ -2181,7 +2197,7 @@ mod tests {
         let json_mode = ModelInferenceRequestJsonMode::On;
         let output_schema = None;
         let format = GroqResponseFormat::new(json_mode, output_schema);
-        assert_eq!(format, GroqResponseFormat::JsonObject { schema: None });
+        assert_eq!(format, GroqResponseFormat::JsonObject);
 
         // Test JSON mode Off
         let json_mode = ModelInferenceRequestJsonMode::Off;
@@ -2191,24 +2207,41 @@ mod tests {
         // Test JSON mode Strict with no schema
         let json_mode = ModelInferenceRequestJsonMode::Strict;
         let format = GroqResponseFormat::new(json_mode, output_schema);
-        assert_eq!(format, GroqResponseFormat::JsonObject { schema: None });
+        assert_eq!(format, GroqResponseFormat::JsonObject);
 
         // Test JSON mode Strict with schema
         let json_mode = ModelInferenceRequestJsonMode::Strict;
-        let json_schema = serde_json::json!({
+        let schema = serde_json::json!({
             "type": "object",
             "properties": {
                 "foo": {"type": "string"}
             }
         });
-        let output_schema = Some(&json_schema);
+        let output_schema = Some(&schema);
         let format = GroqResponseFormat::new(json_mode, output_schema);
-        assert_eq!(
-            format,
-            GroqResponseFormat::JsonObject {
-                schema: output_schema
+        match format {
+            GroqResponseFormat::JsonSchema { json_schema } => {
+                assert_eq!(json_schema["schema"], schema);
+                assert_eq!(json_schema["name"], "response");
+                assert_eq!(json_schema["strict"], true);
             }
-        );
+            _ => panic!("Expected JsonSchema variant"),
+        }
+
+        // Test serialization of JsonSchema format
+        let json_mode = ModelInferenceRequestJsonMode::Strict;
+        let schema = serde_json::json!({
+            "type": "object",
+            "properties": {
+                "foo": {"type": "string"}
+            }
+        });
+        let format = GroqResponseFormat::new(json_mode, Some(&schema));
+        let serialized = serde_json::to_value(&format).unwrap();
+        assert_eq!(serialized["type"], "json_schema");
+        assert_eq!(serialized["json_schema"]["name"], "response");
+        assert_eq!(serialized["json_schema"]["strict"], true);
+        assert_eq!(serialized["json_schema"]["schema"], schema);
     }
 
     #[test]
@@ -2448,4 +2481,33 @@ mod tests {
             r#"{"content":[{"type":"text","text":"My first message"},{"type":"text","text":"My second message"}]}"#
         );
     }
+
+    #[test]
+    #[traced_test]
+    fn test_groq_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = GroqRequest {
+            model: "test-model",
+            ..Default::default()
+        };
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort is applied correctly
+        assert_eq!(request.reasoning_effort, Some("high".to_string()));
+
+        // Test that thinking_budget_tokens warns with tip about reasoning_effort
+        assert!(logs_contain(
+            "Groq does not support the inference parameter `thinking_budget_tokens`, so it will be ignored. Tip: You might want to use `reasoning_effort` for this provider."
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "Groq does not support the inference parameter `verbosity`"
+        ));
+    }
 }
