@@ -3,6 +3,7 @@ use std::{collections::HashMap, pin::Pin};
 use crate::{
     error::IMPOSSIBLE_ERROR_MESSAGE,
     http::{TensorZeroEventSource, TensorzeroRequestBuilder},
+    inference::types::resolved_input::{FileUrl, LazyFile},
 };
 use axum::http;
 use bytes::Bytes;
@@ -31,6 +32,65 @@ pub struct JsonlBatchFileInfo {
     pub file_id: String,
 }
 
+pub async fn convert_stream_error(provider_type: String, e: reqwest_eventsource::Error) -> Error {
+    let message = e.to_string();
+    // If we get an invalid status code, content type, or generic transport error,
+    // then we assume that we're never going to be able to read more chunks from the stream,
+    // The `wrap_provider_stream` function will bail out when it sees this error,
+    // to avoid holding open a broken stream (which will delay gateway shutdown when we
+    // wait on the parent `Span` to finish)
+    match e {
+        reqwest_eventsource::Error::InvalidStatusCode(_, resp)
+        | reqwest_eventsource::Error::InvalidContentType(_, resp) => {
+            ErrorDetails::FatalStreamError {
+                message,
+                provider_type,
+                raw_request: None,
+                raw_response: resp.text().await.ok(),
+            }
+            .into()
+        }
+        reqwest_eventsource::Error::Transport(_) => ErrorDetails::FatalStreamError {
+            message,
+            provider_type,
+            raw_request: None,
+            raw_response: None,
+        }
+        .into(),
+        _ => ErrorDetails::InferenceServer {
+            message,
+            raw_request: None,
+            raw_response: None,
+            provider_type,
+        }
+        .into(),
+    }
+}
+
+// If we could have forwarded an file/image (except for the fact that we're missing the mime_type), log a warning.
+pub fn warn_cannot_forward_url_if_missing_mime_type(
+    file: &LazyFile,
+    fetch_and_encode_input_files_before_inference: bool,
+    provider_type: &str,
+) {
+    // We're not forwarding any urls, so it doesn't matter whether or not we have a mime type
+    if fetch_and_encode_input_files_before_inference {
+        return;
+    }
+    if matches!(
+        file,
+        LazyFile::Url {
+            file_url: FileUrl {
+                url: _,
+                mime_type: None,
+            },
+            future: _
+        }
+    ) {
+        tracing::warn!("Cannot forward image_url to {provider_type} because no mime_type was provided. Specify `mime_type` (or `tensorzero::mime_type` for openai-compatible requests) when sending files to allow URL forwarding.");
+    }
+}
+
 /// A helper function to parse lines from a JSONL file into a batch response.
 /// The provided type `T` is used to parse each line.
 pub async fn parse_jsonl_batch_file<T: DeserializeOwned, E: std::error::Error>(
@@ -802,7 +862,7 @@ mod tests {
                 extra_headers: None,
             },
             "dummy_model",
-            &mut serde_json::Value::String("test".to_string()),
+            &mut "test".into(),
         )
         .unwrap_err()
         .to_string();
