@@ -1,6 +1,9 @@
-use std::{borrow::Cow, sync::OnceLock, time::Duration};
+use std::{borrow::Cow, time::Duration};
 
-use crate::http::{TensorZeroEventSource, TensorzeroHttpClient};
+use crate::{
+    http::{TensorZeroEventSource, TensorzeroHttpClient},
+    providers::openai::OpenAIMessagesConfig,
+};
 use futures::{future::try_join_all, StreamExt};
 use lazy_static::lazy_static;
 use reqwest::StatusCode;
@@ -19,6 +22,9 @@ use crate::{
             batch::{
                 BatchRequestRow, PollBatchInferenceResponse, StartBatchProviderInferenceResponse,
             },
+            chat_completion_inference_params::{
+                warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+            },
             ContentBlockChunk, ContentBlockOutput, FinishReason, Latency, ModelInferenceRequest,
             ModelInferenceRequestJsonMode, PeekableProviderInferenceResponseStream,
             ProviderInferenceResponse, ProviderInferenceResponseArgs,
@@ -26,17 +32,17 @@ use crate::{
         },
         InferenceProvider,
     },
-    model::{build_creds_caching_default, Credential, CredentialLocation, ModelProvider},
+    model::{Credential, ModelProvider},
     providers::helpers::{
-        check_new_tool_call_name, inject_extra_request_data_and_send,
+        check_new_tool_call_name, convert_stream_error, inject_extra_request_data_and_send,
         inject_extra_request_data_and_send_eventsource,
     },
     tool::{ToolCall, ToolCallChunk, ToolChoice},
 };
 
 use super::openai::{
-    convert_stream_error, get_chat_url, tensorzero_to_openai_messages, OpenAIFunction,
-    OpenAIRequestMessage, OpenAISystemRequestMessage, OpenAITool, OpenAIToolType,
+    get_chat_url, tensorzero_to_openai_messages, OpenAIFunction, OpenAIRequestMessage,
+    OpenAISystemRequestMessage, OpenAITool, OpenAIToolType,
 };
 
 lazy_static! {
@@ -46,10 +52,6 @@ lazy_static! {
     };
 }
 
-fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("MISTRAL_API_KEY".to_string())
-}
-
 const PROVIDER_NAME: &str = "Mistral";
 pub const PROVIDER_TYPE: &str = "mistral";
 
@@ -62,23 +64,12 @@ pub struct MistralProvider {
     credentials: MistralCredentials,
 }
 
-static DEFAULT_CREDENTIALS: OnceLock<MistralCredentials> = OnceLock::new();
-
 impl MistralProvider {
-    pub fn new(
-        model_name: String,
-        api_key_location: Option<CredentialLocation>,
-    ) -> Result<Self, Error> {
-        let credentials = build_creds_caching_default(
-            api_key_location,
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
-        Ok(MistralProvider {
+    pub fn new(model_name: String, credentials: MistralCredentials) -> Self {
+        MistralProvider {
             model_name,
             credentials,
-        })
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -91,6 +82,10 @@ pub enum MistralCredentials {
     Static(SecretString),
     Dynamic(String),
     None,
+    WithFallback {
+        default: Box<MistralCredentials>,
+        fallback: Box<MistralCredentials>,
+    },
 }
 
 impl TryFrom<Credential> for MistralCredentials {
@@ -101,6 +96,12 @@ impl TryFrom<Credential> for MistralCredentials {
             Credential::Static(key) => Ok(MistralCredentials::Static(key)),
             Credential::Dynamic(key_name) => Ok(MistralCredentials::Dynamic(key_name)),
             Credential::Missing => Ok(MistralCredentials::None),
+            Credential::WithFallback { default, fallback } => {
+                Ok(MistralCredentials::WithFallback {
+                    default: Box::new((*default).try_into()?),
+                    fallback: Box::new((*fallback).try_into()?),
+                })
+            }
             _ => Err(Error::new(ErrorDetails::Config {
                 message: "Invalid api_key_location for Mistral provider".to_string(),
             })),
@@ -124,6 +125,16 @@ impl MistralCredentials {
                     .into()
                 })
             }
+            MistralCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             MistralCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
                 message: "No credentials are set".to_string(),
@@ -366,12 +377,13 @@ pub fn stream_mistral(
 
 pub(super) async fn prepare_mistral_messages<'a>(
     request: &'a ModelInferenceRequest<'_>,
+    config: OpenAIMessagesConfig<'a>,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut messages: Vec<_> = try_join_all(
         request
             .messages
             .iter()
-            .map(|msg| tensorzero_to_openai_messages(msg, PROVIDER_TYPE)),
+            .map(|msg| tensorzero_to_openai_messages(msg, config)),
     )
     .await?
     .into_iter()
@@ -431,8 +443,7 @@ fn prepare_mistral_tools<'a>(
         Some(tool_config) => match &tool_config.tool_choice {
             ToolChoice::Specific(tool_name) => {
                 let tool = tool_config
-                    .tools_available
-                    .iter()
+                    .tools_available()
                     .find(|t| t.name() == tool_name)
                     .ok_or_else(|| {
                         Error::new(ErrorDetails::ToolNotFound {
@@ -444,8 +455,7 @@ fn prepare_mistral_tools<'a>(
             }
             ToolChoice::Auto | ToolChoice::Required => {
                 let tools = tool_config
-                    .tools_available
-                    .iter()
+                    .tools_available()
                     .map(|t| MistralTool::from(OpenAITool::from(t)))
                     .collect();
                 let tool_choice = match tool_config.tool_choice {
@@ -500,6 +510,29 @@ struct MistralRequest<'a> {
     stop: Option<Cow<'a, [String]>>,
 }
 
+fn apply_inference_params(
+    _request: &mut MistralRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "reasoning_effort", None);
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "thinking_budget_tokens", None);
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
+}
+
 impl<'a> MistralRequest<'a> {
     pub async fn new(
         model: &'a str,
@@ -511,10 +544,19 @@ impl<'a> MistralRequest<'a> {
             }
             ModelInferenceRequestJsonMode::Off => None,
         };
-        let messages = prepare_mistral_messages(request).await?;
+        let messages = prepare_mistral_messages(
+            request,
+            OpenAIMessagesConfig {
+                json_mode: Some(&request.json_mode),
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: request
+                    .fetch_and_encode_input_files_before_inference,
+            },
+        )
+        .await?;
         let (tools, tool_choice) = prepare_mistral_tools(request)?;
 
-        Ok(MistralRequest {
+        let mut mistral_request = MistralRequest {
             messages,
             model,
             temperature: request.temperature,
@@ -528,7 +570,11 @@ impl<'a> MistralRequest<'a> {
             tools,
             tool_choice,
             stop: request.borrow_stop_sequences(),
-        })
+        };
+
+        apply_inference_params(&mut mistral_request, &request.inference_params_v2);
+
+        Ok(mistral_request)
     }
 }
 
@@ -536,7 +582,6 @@ impl<'a> MistralRequest<'a> {
 struct MistralUsage {
     prompt_tokens: u32,
     completion_tokens: u32,
-    total_tokens: u32,
 }
 
 impl From<MistralUsage> for Usage {
@@ -790,6 +835,7 @@ mod tests {
 
     use crate::inference::types::{FunctionType, RequestMessage, Role};
     use crate::providers::test_helpers::{WEATHER_TOOL, WEATHER_TOOL_CONFIG};
+    use tracing_test::traced_test;
 
     #[tokio::test]
     async fn test_mistral_request_new() {
@@ -851,7 +897,6 @@ mod tests {
             usage: MistralUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
 
@@ -948,7 +993,6 @@ mod tests {
             usage: MistralUsage {
                 prompt_tokens: 15,
                 completion_tokens: 25,
-                total_tokens: 40,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -1031,7 +1075,6 @@ mod tests {
             usage: MistralUsage {
                 prompt_tokens: 5,
                 completion_tokens: 0,
-                total_tokens: 5,
             },
         };
         let request_body = MistralRequest {
@@ -1084,7 +1127,6 @@ mod tests {
             usage: MistralUsage {
                 prompt_tokens: 10,
                 completion_tokens: 10,
-                total_tokens: 20,
             },
         };
         let request_body = MistralRequest {
@@ -1230,4 +1272,46 @@ mod tests {
             ErrorDetails::Config { message } if message.contains("Invalid api_key_location")
         ));
     }
+
+    #[test]
+    #[traced_test]
+    fn test_mistral_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = MistralRequest {
+            messages: vec![],
+            model: "test-model",
+            temperature: None,
+            top_p: None,
+            presence_penalty: None,
+            frequency_penalty: None,
+            max_tokens: None,
+            random_seed: None,
+            stream: false,
+            response_format: None,
+            tools: None,
+            tool_choice: None,
+            stop: None,
+        };
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort warns
+        assert!(logs_contain(
+            "Mistral does not support the inference parameter `reasoning_effort`"
+        ));
+
+        // Test that thinking_budget_tokens warns
+        assert!(logs_contain(
+            "Mistral does not support the inference parameter `thinking_budget_tokens`"
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "Mistral does not support the inference parameter `verbosity`"
+        ));
+    }
 }
