@@ -16,7 +16,7 @@ use crate::{
     inference::types::{ContentBlock, ContentBlockChatOutput},
     model::{UninitializedModelConfig, UninitializedModelProvider, UninitializedProviderConfig},
     optimization::{OptimizationJobInfo, OptimizerOutput},
-    providers::openai::{OpenAIRequestToolCall, PROVIDER_TYPE},
+    providers::openai::{OpenAIMessagesConfig, OpenAIRequestToolCall, PROVIDER_TYPE},
     stored_inference::LazyRenderedSample,
     tool::ToolCall,
 };
@@ -337,21 +337,28 @@ pub struct OpenAISupervisedRow<'a> {
 
 impl<'a> OpenAISupervisedRow<'a> {
     pub async fn from_rendered_sample(inference: &'a LazyRenderedSample) -> Result<Self, Error> {
-        let (parallel_tool_calls, tools) = match &inference.tool_params {
-            Some(tool_params) => (
-                tool_params.parallel_tool_calls.unwrap_or_default(),
-                tool_params.tools_available.iter().map(Into::into).collect(),
-            ),
-            None => (false, vec![]),
-        };
+        let parallel_tool_calls = inference
+            .tool_params
+            .parallel_tool_calls
+            .unwrap_or_default();
+        let tools = inference
+            .tool_params
+            .additional_tools
+            .as_ref()
+            .map(|tools| tools.iter().map(Into::into).collect())
+            .unwrap_or_default();
         let mut messages = prepare_openai_messages(
             inference
                 .system_input
                 .as_deref()
-                .map(super::SystemOrDeveloper::System),
+                .map(|m| super::SystemOrDeveloper::System(Cow::Borrowed(m))),
             &inference.messages,
-            None,
-            PROVIDER_TYPE,
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                // For now, this isn't configurable in SFT (we should never need to resolve a file URL here)
+                fetch_and_encode_input_files_before_inference: true,
+            },
         )
         .await?;
         let Some(output) = &inference.output else {
@@ -368,7 +375,12 @@ impl<'a> OpenAISupervisedRow<'a> {
             output.iter().map(|c| c.clone().into()).collect::<Vec<_>>();
         let final_assistant_message = tensorzero_to_openai_assistant_message(
             Cow::Owned(output_content_blocks),
-            PROVIDER_TYPE,
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                // For now, this isn't configurable in SFT (we should never need to resolve a file URL here)
+                fetch_and_encode_input_files_before_inference: true,
+            },
         )
         .await?;
         messages.push(final_assistant_message);
@@ -402,21 +414,28 @@ pub struct OpenAIReinforcementRow<'a> {
 
 impl<'a> OpenAIReinforcementRow<'a> {
     pub async fn from_rendered_sample(inference: &'a LazyRenderedSample) -> Result<Self, Error> {
-        let (parallel_tool_calls, tools) = match &inference.tool_params {
-            Some(tool_params) => (
-                tool_params.parallel_tool_calls.unwrap_or_default(),
-                tool_params.tools_available.iter().map(Into::into).collect(),
-            ),
-            None => (false, vec![]),
-        };
+        let parallel_tool_calls = inference
+            .tool_params
+            .parallel_tool_calls
+            .unwrap_or_default();
+        let tools = inference
+            .tool_params
+            .additional_tools
+            .as_ref()
+            .map(|tools| tools.iter().map(Into::into).collect())
+            .unwrap_or_default();
         let messages = prepare_openai_messages(
             inference
                 .system_input
                 .as_deref()
-                .map(super::SystemOrDeveloper::Developer),
+                .map(|m| super::SystemOrDeveloper::Developer(Cow::Borrowed(m))),
             &inference.messages,
-            None,
-            PROVIDER_TYPE,
+            OpenAIMessagesConfig {
+                json_mode: None,
+                provider_type: PROVIDER_TYPE,
+                // For now, this isn't configurable in RFT (we should never need to resolve a file URL here)
+                fetch_and_encode_input_files_before_inference: true,
+            },
         )
         .await?;
         let Some(output) = &inference.output else {
@@ -549,6 +568,9 @@ pub fn convert_to_optimizer_status(job: OpenAIFineTuningJob) -> Result<Optimizat
                     model_name: model_name.clone(),
                     api_base: None,
                     api_key_location: None,
+                    api_type: Default::default(),
+                    include_encrypted_reasoning: false,
+                    provider_tools: Vec::new(),
                 },
                 extra_headers: None,
                 extra_body: None,
@@ -587,17 +609,18 @@ pub enum OpenAIFineTuningJobStatus {
 
 #[cfg(test)]
 mod tests {
+    use super::*;
+    use serde_json::json;
+
     use crate::{
         inference::types::{
             ContentBlockChatOutput, ModelInput, ResolvedContentBlock, ResolvedRequestMessage, Role,
-            StoredInput, StoredInputMessage, StoredInputMessageContent, Text,
+            StoredInput, StoredInputMessage, StoredInputMessageContent, System, Text,
         },
         providers::openai::OpenAIContentBlock,
         stored_inference::{RenderedSample, StoredOutput},
+        tool::{DynamicToolParams, ToolCallOutput},
     };
-    use serde_json::json;
-
-    use super::*;
 
     #[tokio::test]
     async fn test_convert_to_sft_row() {
@@ -616,19 +639,21 @@ mod tests {
                 }],
             },
             stored_input: StoredInput {
-                system: Some(json!("You are a helpful assistant named Dr. M.M. Patel.")),
+                system: Some(System::Text(
+                    "You are a helpful assistant named Dr. M.M. Patel.".to_string(),
+                )),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: "What is the capital of France?".into(),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "What is the capital of France?".to_string(),
+                    })],
                 }],
             },
             output: output.clone(),
             stored_output: output.map(StoredOutput::Chat),
             episode_id: Some(uuid::Uuid::now_v7()),
             inference_id: Some(uuid::Uuid::now_v7()),
-            tool_params: None,
+            tool_params: DynamicToolParams::default(),
             output_schema: None,
             dispreferred_outputs: vec![],
             tags: HashMap::new(),
@@ -667,8 +692,6 @@ mod tests {
 
     #[tokio::test]
     async fn test_convert_to_rft_row() {
-        use crate::stored_inference::StoredOutput;
-
         let inference = RenderedSample {
             function_name: "test".to_string(),
             input: ModelInput {
@@ -681,12 +704,14 @@ mod tests {
                 }],
             },
             stored_input: StoredInput {
-                system: Some(json!("You are a helpful assistant named Dr. M.M. Patel.")),
+                system: Some(System::Text(
+                    "You are a helpful assistant named Dr. M.M. Patel.".to_string(),
+                )),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("What is the capital of France?"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "What is the capital of France?".to_string(),
+                    })],
                 }],
             },
             output: Some(vec![ContentBlockChatOutput::Text(Text {
@@ -699,7 +724,7 @@ mod tests {
             )])),
             episode_id: Some(uuid::Uuid::now_v7()),
             inference_id: Some(uuid::Uuid::now_v7()),
-            tool_params: None,
+            tool_params: DynamicToolParams::default(),
             output_schema: None,
             dispreferred_outputs: vec![],
             tags: HashMap::new(),
@@ -736,9 +761,6 @@ mod tests {
 
     #[tokio::test]
     async fn test_convert_to_rft_row_with_tool_calls() {
-        use crate::stored_inference::StoredOutput;
-        use crate::tool::ToolCallOutput;
-
         let inference = RenderedSample {
             function_name: "test".to_string(),
             input: ModelInput {
@@ -751,12 +773,12 @@ mod tests {
                 }],
             },
             stored_input: StoredInput {
-                system: Some(json!("You are a helpful assistant.")),
+                system: Some(System::Text("You are a helpful assistant.".to_string())),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("What's the weather like?"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "What's the weather like?".to_string(),
+                    })],
                 }],
             },
             output: Some(vec![
@@ -785,7 +807,7 @@ mod tests {
             ])),
             episode_id: Some(uuid::Uuid::now_v7()),
             inference_id: Some(uuid::Uuid::now_v7()),
-            tool_params: None,
+            tool_params: DynamicToolParams::default(),
             output_schema: None,
             dispreferred_outputs: vec![],
             tags: HashMap::new(),
@@ -819,7 +841,7 @@ mod tests {
         let succeeded_model = json!({
             "id": "ftjob-123",
             "status": "succeeded",
-            "fine_tuned_model": "ft:gpt-3.5-turbo:my-org:custom_suffix:id",
+            "fine_tuned_model": "ft:gpt-4.1-mini:my-org:custom_suffix:id",
             "created_at": 1620000000,
             "metadata": {},
         });
@@ -837,7 +859,7 @@ mod tests {
             "id": "ftjob-456",
             "status": "succeeded",
             "result_files": ["file-abc"],
-            "fine_tuned_model": "ft:gpt-3.5-turbo:my-org:custom_suffix:id",
+            "fine_tuned_model": "ft:gpt-4.1-mini:my-org:custom_suffix:id",
             "created_at": 1620000000,
             "metadata": {},
         });
