@@ -23,9 +23,11 @@ lazy_static! {
     pub static ref WEATHER_TOOL: ToolConfig = ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone());
     pub static ref WEATHER_TOOL_CHOICE: ToolChoice = ToolChoice::Specific("get_temperature".to_string());
     pub static ref WEATHER_TOOL_CONFIG: ToolCallConfig = ToolCallConfig {
-        tools_available: vec![ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone())],
         tool_choice: ToolChoice::Specific("get_temperature".to_string()),
-        parallel_tool_calls: None,
+        ..ToolCallConfig::with_tools_available(
+            vec![ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone())],
+            vec![],
+        )
     };
     pub static ref QUERY_TOOL_CONFIG_STATIC: Arc<StaticToolConfig> = Arc::new(StaticToolConfig {
         name: "query_articles".to_string(),
@@ -43,12 +45,15 @@ lazy_static! {
     pub static ref QUERY_TOOL: ToolConfig = ToolConfig::Static(QUERY_TOOL_CONFIG_STATIC.clone());
     pub static ref ANY_TOOL_CHOICE: ToolChoice = ToolChoice::Required;
     pub static ref MULTI_TOOL_CONFIG: ToolCallConfig = ToolCallConfig {
-        tools_available: vec![
-            ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone()),
-            ToolConfig::Static(QUERY_TOOL_CONFIG_STATIC.clone())
-        ],
         tool_choice: ToolChoice::Required,
         parallel_tool_calls: Some(true),
+        ..ToolCallConfig::with_tools_available(
+            vec![
+                ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone()),
+                ToolConfig::Static(QUERY_TOOL_CONFIG_STATIC.clone())
+            ],
+            vec![],
+        )
     };
 }
 
@@ -56,8 +61,8 @@ lazy_static! {
 pub fn get_temperature_tool_config() -> ToolCallConfig {
     let weather_tool = ToolConfig::Static(WEATHER_TOOL_CONFIG_STATIC.clone());
     ToolCallConfig {
-        tools_available: vec![weather_tool],
         tool_choice: ToolChoice::Specific("get_temperature".to_string()),
         parallel_tool_calls: Some(false),
+        ..ToolCallConfig::with_tools_available(vec![weather_tool], vec![])
     }
 }
