@@ -1,7 +1,10 @@
-use std::{borrow::Cow, sync::OnceLock, time::Duration};
+use std::{borrow::Cow, time::Duration};
 
+use crate::inference::types::chat_completion_inference_params::{
+    warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+};
 use crate::inference::types::RequestMessage;
-use crate::providers::openai::OpenAIToolChoiceString;
+use crate::providers::openai::{OpenAIMessagesConfig, OpenAIToolChoiceString};
 use futures::{future::try_join_all, StreamExt};
 use lazy_static::lazy_static;
 use reqwest_eventsource::Event;
@@ -20,7 +23,7 @@ use crate::inference::types::{
     ProviderInferenceResponseArgs,
 };
 use crate::inference::InferenceProvider;
-use crate::model::{build_creds_caching_default, Credential, CredentialLocation, ModelProvider};
+use crate::model::{Credential, ModelProvider};
 use crate::providers::helpers::{
     inject_extra_request_data_and_send, inject_extra_request_data_and_send_eventsource,
 };
@@ -67,25 +70,17 @@ pub fn default_parse_think_blocks() -> bool {
     true
 }
 
-pub static DEFAULT_CREDENTIALS: OnceLock<TogetherCredentials> = OnceLock::new();
-
 impl TogetherProvider {
     pub fn new(
         model_name: String,
-        api_key_location: Option<CredentialLocation>,
+        credentials: TogetherCredentials,
         parse_think_blocks: bool,
-    ) -> Result<Self, Error> {
-        let credentials = build_creds_caching_default(
-            api_key_location,
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
-        Ok(TogetherProvider {
+    ) -> Self {
+        TogetherProvider {
             model_name,
             credentials,
             parse_think_blocks,
-        })
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -93,15 +88,15 @@ impl TogetherProvider {
     }
 }
 
-pub fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("TOGETHER_API_KEY".to_string())
-}
-
 #[derive(Clone, Debug)]
 pub enum TogetherCredentials {
     Static(SecretString),
     Dynamic(String),
     None,
+    WithFallback {
+        default: Box<TogetherCredentials>,
+        fallback: Box<TogetherCredentials>,
+    },
 }
 
 impl TryFrom<Credential> for TogetherCredentials {
@@ -112,6 +107,12 @@ impl TryFrom<Credential> for TogetherCredentials {
             Credential::Static(key) => Ok(TogetherCredentials::Static(key)),
             Credential::Dynamic(key_name) => Ok(TogetherCredentials::Dynamic(key_name)),
             Credential::Missing => Ok(TogetherCredentials::None),
+            Credential::WithFallback { default, fallback } => {
+                Ok(TogetherCredentials::WithFallback {
+                    default: Box::new((*default).try_into()?),
+                    fallback: Box::new((*fallback).try_into()?),
+                })
+            }
             _ => Err(Error::new(ErrorDetails::Config {
                 message: "Invalid api_key_location for Together provider".to_string(),
             })),
@@ -134,6 +135,16 @@ impl TogetherCredentials {
                     },
                 )?))
             }
+            TogetherCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             TogetherCredentials::None => Err(ErrorDetails::ApiKeyMissing {
                 provider_name: PROVIDER_NAME.to_string(),
                 message: "No credentials are set".to_string(),
@@ -326,8 +337,10 @@ enum TogetherResponseFormat<'a> {
 /// presence_penalty, frequency_penalty, seed, service_tier, stop, user,
 /// or context_length_exceeded_behavior
 #[derive(Debug, Serialize)]
+#[cfg_attr(test, derive(Default))]
 struct TogetherRequest<'a> {
     messages: Vec<OpenAIRequestMessage<'a>>,
+    #[cfg_attr(test, serde(default))]
     model: &'a str,
     #[serde(skip_serializing_if = "Option::is_none")]
     temperature: Option<f32>,
@@ -352,6 +365,35 @@ struct TogetherRequest<'a> {
     parallel_tool_calls: Option<bool>,
     #[serde(skip_serializing_if = "Option::is_none")]
     stop: Option<Cow<'a, [String]>>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    reasoning_effort: Option<String>,
+}
+
+fn apply_inference_params(
+    request: &mut TogetherRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        request.reasoning_effort = reasoning_effort.clone();
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(
+            PROVIDER_NAME,
+            "thinking_budget_tokens",
+            Some("Tip: You might want to use `reasoning_effort` for this provider."),
+        );
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
 }
 
 impl<'a> TogetherRequest<'a> {
@@ -367,8 +409,17 @@ impl<'a> TogetherRequest<'a> {
             }
             ModelInferenceRequestJsonMode::Off => None,
         };
-        let messages =
-            prepare_together_messages(request.system.as_deref(), &request.messages).await?;
+        let messages = prepare_together_messages(
+            request.system.as_deref(),
+            &request.messages,
+            OpenAIMessagesConfig {
+                json_mode: Some(&request.json_mode),
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: request
+                    .fetch_and_encode_input_files_before_inference,
+            },
+        )
+        .await?;
 
         // NOTE: Together AI doesn't seem to support `tool_choice="none"`, so we simply don't include the `tools` field if that's the case
         let tool_choice = request
@@ -385,7 +436,7 @@ impl<'a> TogetherRequest<'a> {
             tool_choice = Some(OpenAIToolChoice::String(OpenAIToolChoiceString::Auto));
         }
 
-        Ok(TogetherRequest {
+        let mut together_request = TogetherRequest {
             messages,
             model,
             temperature: request.temperature,
@@ -400,18 +451,24 @@ impl<'a> TogetherRequest<'a> {
             tool_choice,
             parallel_tool_calls,
             stop: request.borrow_stop_sequences(),
-        })
+            reasoning_effort: None,
+        };
+
+        apply_inference_params(&mut together_request, &request.inference_params_v2);
+
+        Ok(together_request)
     }
 }
 
 pub async fn prepare_together_messages<'a>(
     system: Option<&'a str>,
     request_messages: &'a [RequestMessage],
+    config: OpenAIMessagesConfig<'a>,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut messages: Vec<_> = try_join_all(
         request_messages
             .iter()
-            .map(|msg| tensorzero_to_openai_messages(msg, PROVIDER_TYPE)),
+            .map(|msg| tensorzero_to_openai_messages(msg, config)),
     )
     .await?
     .into_iter()
@@ -559,6 +616,7 @@ impl<'a> TryFrom<TogetherResponseWithMetadata<'a>> for ProviderInferenceResponse
                 content.push(ContentBlockOutput::Thought(Thought {
                     text: Some(reasoning),
                     signature: None,
+                    summary: None,
                     provider_type: Some(PROVIDER_TYPE.to_string()),
                 }));
             }
@@ -684,6 +742,8 @@ fn together_to_tensorzero_chunk(
                             content.push(ContentBlockChunk::Thought(ThoughtChunk {
                                 text: Some(text),
                                 signature: None,
+                                summary_id: None,
+                                summary_text: None,
                                 id: thinking_state.get_id(),
                                 provider_type: Some(PROVIDER_TYPE.to_string()),
                             }));
@@ -783,6 +843,7 @@ mod tests {
     use std::borrow::Cow;
     use std::time::Duration;
 
+    use tracing_test::traced_test;
     use uuid::Uuid;
 
     use super::*;
@@ -892,7 +953,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -962,7 +1022,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let together_response_with_metadata = TogetherResponseWithMetadata {
@@ -988,6 +1047,7 @@ mod tests {
             ContentBlockOutput::Thought(Thought {
                 text: Some("hmmm".to_string()),
                 signature: None,
+                summary: None,
                 provider_type: Some("together".to_string()),
             })
         );
@@ -1010,7 +1070,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let together_response_with_metadata = TogetherResponseWithMetadata {
@@ -1036,6 +1095,7 @@ mod tests {
             ContentBlockOutput::Thought(Thought {
                 text: Some("hmmm".to_string()),
                 signature: None,
+                summary: None,
                 provider_type: Some("together".to_string()),
             })
         );
@@ -1064,7 +1124,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
 
@@ -1437,7 +1496,6 @@ mod tests {
             usage: Some(OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             }),
         };
         let message = together_to_tensorzero_chunk(
@@ -1506,6 +1564,8 @@ mod tests {
             vec![ContentBlockChunk::Thought(ThoughtChunk {
                 text: Some("some thinking content".to_string()),
                 signature: None,
+                summary_id: None,
+                summary_text: None,
                 id: "1".to_string(),
                 provider_type: Some("together".to_string()),
             })]
@@ -1567,4 +1627,30 @@ mod tests {
             })]
         );
     }
+
+    #[test]
+    #[traced_test]
+    fn test_together_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = TogetherRequest::default();
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort is applied correctly
+        assert_eq!(request.reasoning_effort, Some("high".to_string()));
+
+        // Test that thinking_budget_tokens warns with tip about reasoning_effort
+        assert!(logs_contain(
+            "Together does not support the inference parameter `thinking_budget_tokens`, so it will be ignored. Tip: You might want to use `reasoning_effort` for this provider."
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "Together does not support the inference parameter `verbosity`"
+        ));
+    }
 }
