@@ -1,5 +1,4 @@
 use std::borrow::Cow;
-use std::sync::OnceLock;
 
 use futures::future::try_join_all;
 use futures::{StreamExt, TryStreamExt};
@@ -19,6 +18,9 @@ use crate::endpoints::inference::InferenceCredentials;
 use crate::error::{DisplayOrDebugGateway, Error, ErrorDetails};
 use crate::http::TensorzeroHttpClient;
 use crate::inference::types::batch::{BatchRequestRow, PollBatchInferenceResponse};
+use crate::inference::types::chat_completion_inference_params::{
+    warn_inference_parameter_not_supported, ChatCompletionInferenceParamsV2,
+};
 use crate::inference::types::Thought;
 use crate::inference::types::{
     batch::StartBatchProviderInferenceResponse, ContentBlockOutput, Latency, ModelInferenceRequest,
@@ -26,11 +28,11 @@ use crate::inference::types::{
     ProviderInferenceResponse, ProviderInferenceResponseArgs,
 };
 use crate::inference::{InferenceProvider, TensorZeroEventError};
-use crate::model::{build_creds_caching_default, Credential, CredentialLocation, ModelProvider};
+use crate::model::{Credential, ModelProvider};
 use crate::providers::helpers::{
     inject_extra_request_data_and_send, inject_extra_request_data_and_send_eventsource,
 };
-use crate::providers::openai::check_api_base_suffix;
+use crate::providers::openai::{check_api_base_suffix, OpenAIMessagesConfig};
 
 const PROVIDER_NAME: &str = "vLLM";
 pub const PROVIDER_TYPE: &str = "vllm";
@@ -45,29 +47,16 @@ pub struct VLLMProvider {
     credentials: VLLMCredentials,
 }
 
-static DEFAULT_CREDENTIALS: OnceLock<VLLMCredentials> = OnceLock::new();
-
 impl VLLMProvider {
-    pub fn new(
-        model_name: String,
-        api_base: Url,
-        api_key_location: Option<CredentialLocation>,
-    ) -> Result<Self, Error> {
-        let credentials = build_creds_caching_default(
-            api_key_location,
-            default_api_key_location(),
-            PROVIDER_TYPE,
-            &DEFAULT_CREDENTIALS,
-        )?;
-
+    pub fn new(model_name: String, api_base: Url, credentials: VLLMCredentials) -> Self {
         // Check if the api_base has the `/chat/completions` suffix and warn if it does
         check_api_base_suffix(&api_base);
 
-        Ok(VLLMProvider {
+        VLLMProvider {
             model_name,
             api_base,
             credentials,
-        })
+        }
     }
 
     pub fn model_name(&self) -> &str {
@@ -75,14 +64,14 @@ impl VLLMProvider {
     }
 }
 
-fn default_api_key_location() -> CredentialLocation {
-    CredentialLocation::Env("VLLM_API_KEY".to_string())
-}
-
 #[derive(Clone, Debug)]
 pub enum VLLMCredentials {
     Static(SecretString),
     Dynamic(String),
+    WithFallback {
+        default: Box<VLLMCredentials>,
+        fallback: Box<VLLMCredentials>,
+    },
     None,
 }
 
@@ -94,6 +83,10 @@ impl TryFrom<Credential> for VLLMCredentials {
             Credential::Static(key) => Ok(VLLMCredentials::Static(key)),
             Credential::Dynamic(key_name) => Ok(VLLMCredentials::Dynamic(key_name)),
             Credential::None => Ok(VLLMCredentials::None),
+            Credential::WithFallback { default, fallback } => Ok(VLLMCredentials::WithFallback {
+                default: Box::new((*default).try_into()?),
+                fallback: Box::new((*fallback).try_into()?),
+            }),
             #[cfg(any(test, feature = "e2e_tests"))]
             Credential::Missing => Ok(VLLMCredentials::None),
             _ => Err(Error::new(ErrorDetails::Config {
@@ -102,7 +95,6 @@ impl TryFrom<Credential> for VLLMCredentials {
         }
     }
 }
-
 impl VLLMCredentials {
     fn get_api_key<'a>(
         &'a self,
@@ -118,6 +110,16 @@ impl VLLMCredentials {
                     })
                 })?))
             }
+            VLLMCredentials::WithFallback { default, fallback } => {
+                // Try default first, fall back to fallback if it fails
+                default.get_api_key(dynamic_api_keys).or_else(|_| {
+                    tracing::info!(
+                        "Default credential for {} is unavailable, attempting fallback",
+                        PROVIDER_NAME
+                    );
+                    fallback.get_api_key(dynamic_api_keys)
+                })
+            }
             VLLMCredentials::None => Ok(None),
         }
     }
@@ -295,6 +297,7 @@ impl InferenceProvider for VLLMProvider {
 /// for more details.
 /// We are not handling many features of the API here.
 #[derive(Debug, Serialize)]
+#[cfg_attr(test, derive(Default))]
 struct VLLMRequest<'a> {
     messages: Vec<OpenAIRequestMessage<'a>>,
     model: &'a str,
@@ -324,6 +327,29 @@ struct VLLMRequest<'a> {
     parallel_tool_calls: Option<bool>,
 }
 
+fn apply_inference_params(
+    _request: &mut VLLMRequest,
+    inference_params: &ChatCompletionInferenceParamsV2,
+) {
+    let ChatCompletionInferenceParamsV2 {
+        reasoning_effort,
+        thinking_budget_tokens,
+        verbosity,
+    } = inference_params;
+
+    if reasoning_effort.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "reasoning_effort", None);
+    }
+
+    if thinking_budget_tokens.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "thinking_budget_tokens", None);
+    }
+
+    if verbosity.is_some() {
+        warn_inference_parameter_not_supported(PROVIDER_NAME, "verbosity", None);
+    }
+}
+
 impl<'a> VLLMRequest<'a> {
     pub async fn new(
         model: &'a str,
@@ -343,11 +369,20 @@ impl<'a> VLLMRequest<'a> {
         } else {
             None
         };
-        let messages = prepare_vllm_messages(request).await?;
+        let messages = prepare_vllm_messages(
+            request,
+            OpenAIMessagesConfig {
+                json_mode: Some(&request.json_mode),
+                provider_type: PROVIDER_TYPE,
+                fetch_and_encode_input_files_before_inference: request
+                    .fetch_and_encode_input_files_before_inference,
+            },
+        )
+        .await?;
 
         let (tools, tool_choice, parallel_tool_calls) = prepare_openai_tools(request);
 
-        Ok(VLLMRequest {
+        let mut vllm_request = VLLMRequest {
             messages,
             model,
             temperature: request.temperature,
@@ -363,7 +398,11 @@ impl<'a> VLLMRequest<'a> {
             tools,
             tool_choice,
             parallel_tool_calls,
-        })
+        };
+
+        apply_inference_params(&mut vllm_request, &request.inference_params_v2);
+
+        Ok(vllm_request)
     }
 }
 
@@ -417,6 +456,7 @@ impl<'a> TryFrom<VLLMResponseWithMetadata<'a>> for ProviderInferenceResponse {
             content.push(ContentBlockOutput::Thought(Thought {
                 text: Some(reasoning),
                 signature: None,
+                summary: None,
                 provider_type: Some(PROVIDER_TYPE.to_string()),
             }));
         }
@@ -447,12 +487,13 @@ impl<'a> TryFrom<VLLMResponseWithMetadata<'a>> for ProviderInferenceResponse {
 
 pub(super) async fn prepare_vllm_messages<'a>(
     request: &'a ModelInferenceRequest<'_>,
+    config: OpenAIMessagesConfig<'a>,
 ) -> Result<Vec<OpenAIRequestMessage<'a>>, Error> {
     let mut messages: Vec<_> = try_join_all(
         request
             .messages
             .iter()
-            .map(|msg| tensorzero_to_openai_messages(msg, PROVIDER_TYPE)),
+            .map(|msg| tensorzero_to_openai_messages(msg, config)),
     )
     .await?
     .into_iter()
@@ -628,7 +669,6 @@ mod tests {
             usage: OpenAIUsage {
                 prompt_tokens: 10,
                 completion_tokens: 20,
-                total_tokens: 30,
             },
         };
         let generic_request = ModelInferenceRequest {
@@ -689,31 +729,27 @@ mod tests {
     #[traced_test]
     fn test_vllm_provider_new_api_base_check() {
         let model_name = "test-model".to_string();
-        let api_key_location = Some(CredentialLocation::None);
 
         // Valid cases (should not warn)
         let _ = VLLMProvider::new(
             model_name.clone(),
             Url::parse("http://localhost:1234/v1/").unwrap(),
-            api_key_location.clone(),
-        )
-        .unwrap();
+            VLLMCredentials::None,
+        );
 
         let _ = VLLMProvider::new(
             model_name.clone(),
             Url::parse("http://localhost:1234/v1").unwrap(),
-            api_key_location.clone(),
-        )
-        .unwrap();
+            VLLMCredentials::None,
+        );
 
         // Invalid cases (should warn)
         let invalid_url_1 = Url::parse("http://localhost:1234/chat/completions").unwrap();
         let _ = VLLMProvider::new(
             model_name.clone(),
             invalid_url_1.clone(),
-            api_key_location.clone(),
-        )
-        .unwrap();
+            VLLMCredentials::None,
+        );
         assert!(logs_contain("automatically appends `/chat/completions`"));
         assert!(logs_contain(invalid_url_1.as_ref()));
 
@@ -721,9 +757,8 @@ mod tests {
         let _ = VLLMProvider::new(
             model_name.clone(),
             invalid_url_2.clone(),
-            api_key_location.clone(),
-        )
-        .unwrap();
+            VLLMCredentials::None,
+        );
         assert!(logs_contain("automatically appends `/chat/completions`"));
         assert!(logs_contain(invalid_url_2.as_ref()));
     }
@@ -771,9 +806,9 @@ mod tests {
         let parallel_tool_calls = vllm_request.parallel_tool_calls.unwrap();
         assert!(parallel_tool_calls);
         let tool_config = ToolCallConfig {
-            tools_available: vec![],
             tool_choice: ToolChoice::Required,
             parallel_tool_calls: Some(true),
+            ..Default::default()
         };
 
         // Test no tools but a tool choice and make sure tool choice output is None
@@ -805,4 +840,32 @@ mod tests {
         assert!(vllm_request.tool_choice.is_none());
         assert!(vllm_request.parallel_tool_calls.is_none());
     }
+
+    #[test]
+    #[traced_test]
+    fn test_vllm_apply_inference_params_called() {
+        let inference_params = ChatCompletionInferenceParamsV2 {
+            reasoning_effort: Some("high".to_string()),
+            thinking_budget_tokens: Some(1024),
+            verbosity: Some("low".to_string()),
+        };
+        let mut request = VLLMRequest::default();
+
+        apply_inference_params(&mut request, &inference_params);
+
+        // Test that reasoning_effort warns
+        assert!(logs_contain(
+            "vLLM does not support the inference parameter `reasoning_effort`"
+        ));
+
+        // Test that thinking_budget_tokens warns
+        assert!(logs_contain(
+            "vLLM does not support the inference parameter `thinking_budget_tokens`"
+        ));
+
+        // Test that verbosity warns
+        assert!(logs_contain(
+            "vLLM does not support the inference parameter `verbosity`"
+        ));
+    }
 }
