@@ -1,26 +1,35 @@
 use std::future::IntoFuture;
 use std::net::SocketAddr;
 use std::sync::Arc;
+use std::time::Duration;
 
 use crate::db::postgres::PostgresConnectionInfo;
 use crate::endpoints::openai_compatible::RouterExt;
-use axum::extract::{rejection::JsonRejection, FromRequest, Json, Request};
+use axum::extract::{rejection::JsonRejection, DefaultBodyLimit, FromRequest, Json, Request};
 use axum::Router;
+use moka::sync::Cache;
 use serde::de::DeserializeOwned;
 use sqlx::postgres::PgPoolOptions;
+use tensorzero_auth::postgres::AuthResult;
 use tokio::runtime::Handle;
 use tokio::sync::oneshot::Sender;
 use tokio_util::sync::CancellationToken;
+use tokio_util::task::TaskTracker;
 use tracing::instrument;
 
 use crate::config::{Config, ConfigFileGlob};
+use crate::db::clickhouse::clickhouse_client::ClickHouseClientType;
 use crate::db::clickhouse::migration_manager::{self, RunMigrationManagerArgs};
 use crate::db::clickhouse::ClickHouseConnectionInfo;
+use crate::db::feedback::FeedbackQueries;
 use crate::endpoints;
 use crate::error::{Error, ErrorDetails};
 use crate::howdy::setup_howdy;
 use crate::http::TensorzeroHttpClient;
 
+#[cfg(test)]
+use crate::db::clickhouse::ClickHouseClient;
+
 /// Represents an active gateway (either standalone or embedded)
 /// The contained `app_state` can be freely cloned and dropped.
 /// However, dropping the `GatewayHandle` itself will wait for any
@@ -54,7 +63,7 @@ impl Drop for GatewayHandle {
             .batcher_join_handle();
         // Drop our `ClickHouseConnectionInfo`, so that we stop holding on to the `Arc<BatchSender>`
         // This allows the batch writer task to exit (once all of the remaining `ClickhouseConnectionInfo`s are dropped)
-        self.app_state.clickhouse_connection_info = ClickHouseConnectionInfo::Disabled;
+        self.app_state.clickhouse_connection_info = ClickHouseConnectionInfo::new_disabled();
         if let Some(handle) = handle {
             tracing::info!("Waiting for ClickHouse batch writer to finish");
             // This could block forever if:
@@ -76,6 +85,20 @@ impl Drop for GatewayHandle {
             });
             tracing::info!("ClickHouse batch writer finished");
         }
+        self.app_state.deferred_tasks.close();
+        // The 'wait' future will resolve immediately if the pool is empty.
+        // Closing the pool doesn't block more futures from being added, so checking
+        // if it's empty doesn't introduce any new race conditions (it's still possible
+        // for some existing tokio task to spawn something in this pool after 'wait' resolves).
+        // This check makes it easier to write tests (since we don't need to use a multi-threaded runtime),
+        // as well as reducing the number of log messages in the common case.
+        if !self.app_state.deferred_tasks.is_empty() {
+            tokio::task::block_in_place(|| {
+                tracing::info!("Waiting for deferred tasks to finish");
+                Handle::current().block_on(self.app_state.deferred_tasks.wait());
+                tracing::info!("Deferred tasks finished");
+            });
+        }
     }
 }
 
@@ -88,6 +111,11 @@ pub struct AppStateData {
     pub http_client: TensorzeroHttpClient,
     pub clickhouse_connection_info: ClickHouseConnectionInfo,
     pub postgres_connection_info: PostgresConnectionInfo,
+    /// Holds any background tasks that we want to wait on during shutdown
+    /// We wait for these tasks to finish when `GatewayHandle` is dropped
+    pub deferred_tasks: TaskTracker,
+    /// Optional cache for TensorZero API key authentication
+    pub auth_cache: Option<Cache<String, AuthResult>>,
     // Prevent `AppStateData` from being directly constructed outside of this module
     // This ensures that `AppStateData` is only ever constructed via explicit `new` methods,
     // which can ensure that we update global state.
@@ -95,15 +123,35 @@ pub struct AppStateData {
 }
 pub type AppState = axum::extract::State<AppStateData>;
 
+/// Creates an auth cache based on the configuration.
+/// Returns None if auth is disabled or cache is disabled.
+fn create_auth_cache_from_config(config: &Config) -> Option<Cache<String, AuthResult>> {
+    if !config.gateway.auth.enabled {
+        return None;
+    }
+
+    let default_cache_config = Default::default();
+    let cache_config = config
+        .gateway
+        .auth
+        .cache
+        .as_ref()
+        .unwrap_or(&default_cache_config);
+
+    if !cache_config.enabled {
+        return None;
+    }
+
+    Some(
+        Cache::builder()
+            .time_to_live(Duration::from_millis(cache_config.ttl_ms))
+            .build(),
+    )
+}
+
 impl GatewayHandle {
     pub async fn new(config: Arc<Config>) -> Result<Self, Error> {
         let clickhouse_url = std::env::var("TENSORZERO_CLICKHOUSE_URL").ok();
-        if clickhouse_url.is_none()
-            && std::env::var("CLICKHOUSE_URL").is_ok()
-            && config.gateway.observability.enabled.is_none()
-        {
-            return Err(ErrorDetails::ClickHouseConfiguration { message: "`CLICKHOUSE_URL` is deprecated and no longer accepted. Please set `TENSORZERO_CLICKHOUSE_URL`".to_string() }.into());
-        }
         let postgres_url = std::env::var("TENSORZERO_POSTGRES_URL").ok();
         Self::new_with_databases(config, clickhouse_url, postgres_url).await
     }
@@ -116,12 +164,13 @@ impl GatewayHandle {
         let clickhouse_connection_info = setup_clickhouse(&config, clickhouse_url, false).await?;
         let postgres_connection_info = setup_postgres(&config, postgres_url).await?;
         let http_client = TensorzeroHttpClient::new()?;
-        Ok(Self::new_with_database_and_http_client(
+        Self::new_with_database_and_http_client(
             config,
             clickhouse_connection_info,
             postgres_connection_info,
             http_client,
-        ))
+        )
+        .await
     }
 
     /// # Panics
@@ -130,47 +179,96 @@ impl GatewayHandle {
     pub fn new_unit_test_data(config: Arc<Config>, test_options: GatewayHandleTestOptions) -> Self {
         let http_client = TensorzeroHttpClient::new().unwrap();
         let clickhouse_connection_info =
-            ClickHouseConnectionInfo::new_mock(test_options.clickhouse_healthy);
+            ClickHouseConnectionInfo::new_mock(test_options.clickhouse_client);
         let postgres_connection_info =
             PostgresConnectionInfo::new_mock(test_options.postgres_healthy);
-        Self::new_with_database_and_http_client(
-            config,
-            clickhouse_connection_info,
-            postgres_connection_info,
-            http_client,
-        )
-    }
-
-    pub fn new_with_database_and_http_client(
-        config: Arc<Config>,
-        clickhouse_connection_info: ClickHouseConnectionInfo,
-        postgres_connection_info: PostgresConnectionInfo,
-        http_client: TensorzeroHttpClient,
-    ) -> Self {
         let cancel_token = CancellationToken::new();
-        setup_howdy(
-            &config,
-            clickhouse_connection_info.clone(),
-            cancel_token.clone(),
-        );
+        let auth_cache = create_auth_cache_from_config(&config);
         Self {
             app_state: AppStateData {
                 config,
                 http_client,
                 clickhouse_connection_info,
                 postgres_connection_info,
+                deferred_tasks: TaskTracker::new(),
+                auth_cache,
                 _private: (),
             },
             cancel_token,
             _private: (),
         }
     }
+
+    #[cfg(feature = "pyo3")]
+    pub fn new_dummy(http_client: TensorzeroHttpClient) -> Self {
+        let config = Arc::new(Config::new_dummy_for_pyo3());
+        let clickhouse_connection_info = ClickHouseConnectionInfo::new_fake();
+        #[cfg(test)]
+        let postgres_connection_info = PostgresConnectionInfo::new_mock(true);
+        #[cfg(not(test))]
+        let postgres_connection_info = PostgresConnectionInfo::new_disabled();
+        let cancel_token = CancellationToken::new();
+        let auth_cache = create_auth_cache_from_config(&config);
+        Self {
+            app_state: AppStateData {
+                config,
+                http_client,
+                clickhouse_connection_info,
+                postgres_connection_info,
+                deferred_tasks: TaskTracker::new(),
+                auth_cache,
+                _private: (),
+            },
+            cancel_token,
+            _private: (),
+        }
+    }
+
+    pub async fn new_with_database_and_http_client(
+        config: Arc<Config>,
+        clickhouse_connection_info: ClickHouseConnectionInfo,
+        postgres_connection_info: PostgresConnectionInfo,
+        http_client: TensorzeroHttpClient,
+    ) -> Result<Self, Error> {
+        let cancel_token = CancellationToken::new();
+        setup_howdy(
+            &config,
+            clickhouse_connection_info.clone(),
+            cancel_token.clone(),
+        );
+        for (function_name, function_config) in &config.functions {
+            function_config
+                .experimentation()
+                .setup(
+                    Arc::new(clickhouse_connection_info.clone())
+                        as Arc<dyn FeedbackQueries + Send + Sync>,
+                    function_name,
+                    &postgres_connection_info,
+                    cancel_token.clone(),
+                )
+                .await?;
+        }
+        let auth_cache = create_auth_cache_from_config(&config);
+        Ok(Self {
+            app_state: AppStateData {
+                config,
+                http_client,
+                clickhouse_connection_info,
+                postgres_connection_info,
+                deferred_tasks: TaskTracker::new(),
+                auth_cache,
+                _private: (),
+            },
+            cancel_token,
+            _private: (),
+        })
+    }
 }
 
 pub async fn setup_clickhouse_without_config(
     clickhouse_url: String,
 ) -> Result<ClickHouseConnectionInfo, Error> {
-    setup_clickhouse(&Config::default(), Some(clickhouse_url), true).await
+    setup_clickhouse(&Config::new_empty().await?, Some(clickhouse_url), true).await
 }
 
 pub async fn setup_clickhouse(
@@ -220,11 +318,11 @@ pub async fn setup_clickhouse(
     };
 
     // Run ClickHouse migrations (if any) if we have a production ClickHouse connection
-    if let ClickHouseConnectionInfo::Production { .. } = &clickhouse_connection_info {
+    if clickhouse_connection_info.client_type() == ClickHouseClientType::Production {
         migration_manager::run(RunMigrationManagerArgs {
             clickhouse: &clickhouse_connection_info,
-            skip_completed_migrations: true,
-            manual_run: false,
+            is_manual_run: false,
+            disable_automatic_migrations: config.gateway.observability.disable_automatic_migrations,
         })
         .await?;
     }
@@ -236,9 +334,17 @@ pub async fn setup_postgres(
     postgres_url: Option<String>,
 ) -> Result<PostgresConnectionInfo, Error> {
     let Some(postgres_url) = postgres_url else {
+        // Check if rate limiting is configured but Postgres is not available
+        if config.rate_limiting.enabled() && !config.rate_limiting.rules().is_empty() {
+            return Err(Error::new(ErrorDetails::Config {
+                message: "Rate limiting is configured but PostgreSQL is not available. Rate limiting requires PostgreSQL to be configured. Please set the TENSORZERO_POSTGRES_URL environment variable or disable rate limiting.".to_string(),
+            }));
+        }
         return Ok(PostgresConnectionInfo::Disabled);
     };
 
+    // TODO - decide how we should handle apply `connection_pool_size` to two pools
+    // Hopefully, sqlx does a stable release before we actually start using `alpha_pool`
     let pool = PgPoolOptions::new()
         .max_connections(config.postgres.connection_pool_size)
         .connect(&postgres_url)
@@ -248,7 +354,18 @@ pub async fn setup_postgres(
                 message: err.to_string(),
             })
         })?;
-    let connection_info = PostgresConnectionInfo::new_with_pool(pool);
+
+    let alpha_pool = sqlx_alpha::postgres::PgPoolOptions::new()
+        .max_connections(config.postgres.connection_pool_size)
+        .connect(&postgres_url)
+        .await
+        .map_err(|err| {
+            Error::new(ErrorDetails::PostgresConnectionInitialization {
+                message: err.to_string(),
+            })
+        })?;
+
+    let connection_info = PostgresConnectionInfo::new_with_pool(pool, Some(alpha_pool));
     connection_info.check_migrations().await?;
     Ok(connection_info)
 }
@@ -331,7 +448,7 @@ pub async fn start_openai_compatible_gateway(
     let config = if let Some(config_file) = config_file {
         Arc::new(Config::load_and_verify_from_path(&ConfigFileGlob::new(config_file)?).await?)
     } else {
-        Arc::new(Config::default())
+        Arc::new(Config::new_empty().await?)
     };
     let gateway_handle =
         GatewayHandle::new_with_databases(config, clickhouse_url, postgres_url).await?;
@@ -339,6 +456,10 @@ pub async fn start_openai_compatible_gateway(
     let router = Router::new()
         .register_openai_compatible_routes()
         .fallback(endpoints::fallback::handle_404)
+        .layer(DefaultBodyLimit::max(100 * 1024 * 1024)) // increase the default body limit from 2MB to 100MB
+        .layer(axum::middleware::from_fn(
+            crate::observability::warn_early_drop::warn_on_early_connection_drop,
+        ))
         .with_state(gateway_handle.app_state.clone());
 
     let (sender, recv) = tokio::sync::oneshot::channel::<()>();
@@ -346,6 +467,8 @@ pub async fn start_openai_compatible_gateway(
         let _ = recv.await;
     };
 
+    // TODO(https://github.com/tensorzero/tensorzero/issues/3983): Audit this callsite
+    #[expect(clippy::disallowed_methods)]
     tokio::spawn(
         axum::serve(listener, router)
             .with_graceful_shutdown(shutdown_fut)
@@ -362,7 +485,7 @@ pub async fn start_openai_compatible_gateway(
 
 #[cfg(test)]
 pub struct GatewayHandleTestOptions {
-    pub clickhouse_healthy: bool,
+    pub clickhouse_client: Arc<dyn ClickHouseClient>,
     pub postgres_healthy: bool,
 }
 
@@ -382,6 +505,7 @@ mod tests {
                 enabled: Some(false),
                 async_writes: false,
                 batch_writes: Default::default(),
+                disable_automatic_migrations: false,
             },
             bind_address: None,
             debug: false,
@@ -391,6 +515,8 @@ mod tests {
             unstable_error_json: false,
             unstable_disable_feedback_target_validation: false,
             disable_pseudonymous_usage_analytics: false,
+            fetch_and_encode_input_files_before_inference: false,
+            auth: Default::default(),
         };
 
         let config = Box::leak(Box::new(Config {
@@ -399,10 +525,10 @@ mod tests {
         }));
 
         let clickhouse_connection_info = setup_clickhouse(config, None, false).await.unwrap();
-        assert!(matches!(
-            clickhouse_connection_info,
-            ClickHouseConnectionInfo::Disabled
-        ));
+        assert_eq!(
+            clickhouse_connection_info.client_type(),
+            ClickHouseClientType::Disabled
+        );
         assert!(!logs_contain(
             "Missing environment variable TENSORZERO_CLICKHOUSE_URL"
         ));
@@ -413,7 +539,9 @@ mod tests {
                 enabled: None,
                 async_writes: false,
                 batch_writes: Default::default(),
+                disable_automatic_migrations: false,
             },
+            fetch_and_encode_input_files_before_inference: false,
             unstable_error_json: false,
             ..Default::default()
         };
@@ -422,10 +550,10 @@ mod tests {
             ..Default::default()
         }));
         let clickhouse_connection_info = setup_clickhouse(config, None, false).await.unwrap();
-        assert!(matches!(
-            clickhouse_connection_info,
-            ClickHouseConnectionInfo::Disabled
-        ));
+        assert_eq!(
+            clickhouse_connection_info.client_type(),
+            ClickHouseClientType::Disabled
+        );
         assert!(!logs_contain(
             "Missing environment variable TENSORZERO_CLICKHOUSE_URL"
         ));
@@ -440,6 +568,7 @@ mod tests {
                 enabled: Some(true),
                 async_writes: false,
                 batch_writes: Default::default(),
+                disable_automatic_migrations: false,
             },
             bind_address: None,
             debug: false,
@@ -449,6 +578,8 @@ mod tests {
             unstable_error_json: false,
             unstable_disable_feedback_target_validation: false,
             disable_pseudonymous_usage_analytics: false,
+            fetch_and_encode_input_files_before_inference: false,
+            auth: Default::default(),
         };
 
         let config = Box::leak(Box::new(Config {
@@ -467,6 +598,7 @@ mod tests {
                 enabled: Some(true),
                 async_writes: false,
                 batch_writes: Default::default(),
+                disable_automatic_migrations: false,
             },
             bind_address: None,
             debug: false,
@@ -476,6 +608,8 @@ mod tests {
             unstable_error_json: false,
             unstable_disable_feedback_target_validation: false,
             disable_pseudonymous_usage_analytics: false,
+            fetch_and_encode_input_files_before_inference: false,
+            auth: Default::default(),
         };
         let config = Box::leak(Box::new(Config {
             gateway: gateway_config,
@@ -496,6 +630,7 @@ mod tests {
                 enabled: Some(true),
                 async_writes: false,
                 batch_writes: Default::default(),
+                disable_automatic_migrations: false,
             },
             bind_address: None,
             debug: false,
@@ -505,6 +640,8 @@ mod tests {
             unstable_error_json: false,
             unstable_disable_feedback_target_validation: false,
             disable_pseudonymous_usage_analytics: false,
+            fetch_and_encode_input_files_before_inference: false,
+            auth: Default::default(),
         };
         let config = Config {
             gateway: gateway_config,
