@@ -1,4 +1,3 @@
-#![allow(clippy::print_stdout)]
 use futures::StreamExt;
 use reqwest::{Client, StatusCode};
 use reqwest_eventsource::{Event, RequestBuilderExt};
@@ -114,7 +113,7 @@ async fn e2e_test_best_of_n_dummy_candidates_dummy_judge_inner(
                 {
                     "role": "user",
                     "content": [
-                        {"type": "text", "value": format!("Please write me a sentence about Megumin making an explosion: {random_input}")},
+                        {"type": "text", "text": format!("Please write me a sentence about Megumin making an explosion: {random_input}")},
                     ]
                 }
             ]
@@ -264,7 +263,7 @@ async fn e2e_test_best_of_n_dummy_candidates_real_judge() {
                 {
                     "role": "user",
                     "content": [
-                        {"type": "text", "value": "Please write me a sentence about Megumin making an explosion."},
+                        {"type": "text", "text": "Please write me a sentence about Megumin making an explosion."},
                         {"type": "unknown", "model_provider_name": "tensorzero::model_name::json::provider_name::json", "data": {"type": "text", "text": "My extra json-model input", "my": {"other": "keys"}}},
                         {"type": "unknown", "model_provider_name": "tensorzero::model_name::gemini-2.0-flash-001::provider_name::gcp_vertex_gemini", "data": {"text": "My extra gemini text"}}
                     ]
@@ -345,16 +344,7 @@ async fn e2e_test_best_of_n_dummy_candidates_real_judge() {
                     ]
                   }
                 ],
-                "tools": null,
-                "toolConfig": null,
                 "generationConfig": {
-                  "stopSequences": null,
-                  "temperature": null,
-                  "maxOutputTokens": null,
-                  "topP": null,
-                  "presencePenalty": null,
-                  "frequencyPenalty": null,
-                  "seed": null,
                   "responseMimeType": "application/json",
                   "responseSchema": {
                     "type": "object",
@@ -575,7 +565,7 @@ async fn e2e_test_best_of_n_json_real_judge() {
             "messages": [
                 {
                     "role": "user",
-                    "content": [{"type": "text", "value": "What's the first word in the typical output of one's first program. Answer as a json object with a single field 'answer' containing the string."}]
+                    "content": [{"type": "text", "text": "What's the first word in the typical output of one's first program. Answer as a json object with a single field 'answer' containing the string."}]
                 }
             ]
         }
@@ -743,16 +733,7 @@ async fn e2e_test_best_of_n_json_real_judge() {
                   ]
                 }
               ],
-              "tools": null,
-              "toolConfig": null,
               "generationConfig": {
-                "stopSequences": null,
-                "temperature": null,
-                "topP": null,
-                "presencePenalty": null,
-                "frequencyPenalty": null,
-                "maxOutputTokens": null,
-                "seed": null,
                 "responseMimeType": "application/json",
                 "responseSchema": {
                     "type": "object",
@@ -855,7 +836,7 @@ async fn e2e_test_best_of_n_json_real_judge_implicit_tool() {
             "messages": [
                 {
                     "role": "user",
-                    "content": [{"type": "text", "value": "What's the first word in the typical output of one's first program. Answer as a json object with a single field 'answer' containing the string."}]
+                    "content": [{"type": "text", "text": "What's the first word in the typical output of one's first program. Answer as a json object with a single field 'answer' containing the string."}]
                 }
             ]
         }
@@ -1024,7 +1005,12 @@ async fn e2e_test_best_of_n_json_real_judge_implicit_tool() {
                 ],
                 "max_tokens": 4096,
                 "stream": false,
-                "system": "You are an assistant tasked with re-ranking candidate answers to the following problem:\n------\nYou are a helpful and friendly assistant named AskJeeves\n------\nThe messages below are the conversation history between the user and the assistant along with a final message giving a set of candidate responses.\nPlease evaluate the following candidate responses and provide your reasoning along with the index of the best candidate in the following JSON format:\n{\n    \"thinking\": \"your reasoning here\",\n    \"answer_choice\": int  // Range: 0 to 1\n}\nIn the \"thinking\" block:\nFirst, you should analyze each response itself against the conversation history and determine if it is a good response or not.\nThen you should think out loud about which is best and most faithful to instructions.\nIn the \"answer_choice\" block: you should output the index of the best response.",
+                "system": [
+                    {
+                        "type": "text",
+                        "text": "You are an assistant tasked with re-ranking candidate answers to the following problem:\n------\nYou are a helpful and friendly assistant named AskJeeves\n------\nThe messages below are the conversation history between the user and the assistant along with a final message giving a set of candidate responses.\nPlease evaluate the following candidate responses and provide your reasoning along with the index of the best candidate in the following JSON format:\n{\n    \"thinking\": \"your reasoning here\",\n    \"answer_choice\": int  // Range: 0 to 1\n}\nIn the \"thinking\" block:\nFirst, you should analyze each response itself against the conversation history and determine if it is a good response or not.\nThen you should think out loud about which is best and most faithful to instructions.\nIn the \"answer_choice\" block: you should output the index of the best response."
+                    }
+                ],
                 "tool_choice": {
                     "type": "tool",
                     "name": "respond",
@@ -1166,16 +1152,8 @@ async fn e2e_test_best_of_n_judge_extra_body() {
                     ]
                   }
                 ],
-                "tools": null,
-                "toolConfig": null,
                 "generationConfig": {
-                    "stopSequences": null,
                     "temperature": 0.123,
-                    "maxOutputTokens": null,
-                    "topP": null,
-                    "presencePenalty": null,
-                    "frequencyPenalty": null,
-                    "seed": null,
                     "responseMimeType": "application/json",
                     "responseSchema": {
                         "type": "object",
