@@ -5,16 +5,27 @@ use futures::StreamExt;
 use reqwest::{Client, StatusCode};
 use reqwest_eventsource::{Event, RequestBuilderExt};
 use serde_json::{json, Value};
+use tensorzero::{
+    test_helpers::make_embedded_gateway_with_config, ClientInferenceParams, ClientInput,
+    ClientInputMessage, ClientInputMessageContent, File, InferenceOutput, InferenceResponse, Role,
+    UrlFile,
+};
+use url::Url;
 use uuid::Uuid;
 
 use crate::{
     common::get_gateway_endpoint,
-    providers::common::{E2ETestProvider, E2ETestProviders},
+    providers::common::{E2ETestProvider, E2ETestProviders, DEEPSEEK_PAPER_PDF, FERRIS_PNG},
 };
-use tensorzero_core::db::clickhouse::test_helpers::{
-    get_clickhouse, select_chat_inference_clickhouse, select_model_inference_clickhouse,
+use tensorzero_core::{
+    db::clickhouse::test_helpers::{
+        get_clickhouse, select_chat_inference_clickhouse, select_model_inference_clickhouse,
+    },
+    inference::types::{ContentBlockChatOutput, TextKind},
 };
 
+use super::common::ModelTestProvider;
+
 crate::generate_provider_tests!(get_providers);
 crate::generate_batch_inference_tests!(get_providers);
 
@@ -43,7 +54,7 @@ async fn get_providers() -> E2ETestProviders {
     let pdf_providers = vec![E2ETestProvider {
         supports_batch_inference: false,
         variant_name: "anthropic".to_string(),
-        model_name: "anthropic::claude-3-5-sonnet-20241022".into(),
+        model_name: "anthropic::claude-sonnet-4-5-20250929".into(),
         model_provider_name: "anthropic".into(),
         credentials: HashMap::new(),
     }];
@@ -112,6 +123,31 @@ async fn get_providers() -> E2ETestProviders {
         credentials: HashMap::new(),
     }];
 
+    let provider_type_default_credentials_providers = vec![E2ETestProvider {
+        supports_batch_inference: false,
+        variant_name: "anthropic".to_string(),
+        model_name: "claude-3-haiku-20240307".into(),
+        model_provider_name: "anthropic".into(),
+        credentials: HashMap::new(),
+    }];
+
+    let provider_type_default_credentials_shorthand_providers = vec![E2ETestProvider {
+        supports_batch_inference: false,
+        variant_name: "anthropic-shorthand".to_string(),
+        model_name: "anthropic::claude-3-haiku-20240307".into(),
+        model_provider_name: "anthropic".into(),
+        credentials: HashMap::new(),
+    }];
+
+    let credential_fallbacks = vec![ModelTestProvider {
+        provider_type: "anthropic".into(),
+        model_info: HashMap::from([(
+            "model_name".to_string(),
+            "claude-3-haiku-20240307".to_string(),
+        )]),
+        use_modal_headers: false,
+    }];
+
     E2ETestProviders {
         simple_inference: standard_providers.clone(),
         bad_auth_extra_headers,
@@ -120,6 +156,9 @@ async fn get_providers() -> E2ETestProviders {
         embeddings: vec![],
         inference_params_inference: standard_providers.clone(),
         inference_params_dynamic_credentials: inference_params_dynamic_providers,
+        provider_type_default_credentials: provider_type_default_credentials_providers,
+        provider_type_default_credentials_shorthand:
+            provider_type_default_credentials_shorthand_providers,
         tool_use_inference: standard_providers.clone(),
         tool_multi_turn_inference: standard_providers.clone(),
         dynamic_tool_use_inference: standard_providers.clone(),
@@ -129,6 +168,7 @@ async fn get_providers() -> E2ETestProviders {
         image_inference: image_providers,
         pdf_inference: pdf_providers,
         shorthand_inference: shorthand_providers.clone(),
+        credential_fallbacks,
     }
 }
 
@@ -144,7 +184,7 @@ async fn test_thinking_rejected_128k() {
     // since we want to test the current Anthropic behavior.
     let random = Uuid::now_v7();
     let payload = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "input":{
             "messages": [
                 {
@@ -155,6 +195,7 @@ async fn test_thinking_rejected_128k() {
         "params": {
             "chat_completion": {
                 "max_tokens": 128000,
+                "thinking_budget_tokens": 1024,
             }
         },
         "stream": false,
@@ -230,11 +271,6 @@ async fn test_thinking_inference_extra_header_128k() {
                     "content": "Output a haiku that ends in the word 'my_custom_stop'"
                 }
             ]},
-        "params": {
-            "chat_completion": {
-                "max_tokens": 128000,
-            }
-        },
         "extra_headers": [
             {
                 "model_provider_name": "tensorzero::model_name::anthropic::claude-3-7-sonnet-20250219::provider_name::anthropic",
@@ -243,17 +279,6 @@ async fn test_thinking_inference_extra_header_128k() {
             }
         ],
         "extra_body": [
-            {
-                "model_provider_name": "tensorzero::model_name::anthropic::claude-3-7-sonnet-20250219::provider_name::anthropic",
-                "pointer": "/thinking",
-                // We use a budget tokens of 1024 to make sure that it doesn't think for too long,
-                // since 'stop_sequences' does not seem to apply to thinking. We set 'max_tokens'
-                // to 128k in 'test_thinking_128k'
-                "value": {
-                    "type": "enabled",
-                    "budget_tokens": 1024,
-                }
-            },
             {
                 "model_provider_name": "tensorzero::model_name::anthropic::claude-3-7-sonnet-20250219::provider_name::anthropic",
                 "pointer": "/stop_sequences",
@@ -262,7 +287,15 @@ async fn test_thinking_inference_extra_header_128k() {
                 ]
             }
         ],
-        "stream": false,
+        // We use a budget tokens of 1024 to make sure that it doesn't think for too long,
+        // since 'stop_sequences' does not seem to apply to thinking. We set 'max_tokens'
+        // to 128k in 'test_thinking_128k'
+        "params": {
+            "chat_completion": {
+                "max_tokens": 128000,
+                "thinking_budget_tokens": 1024,
+            }
+        },
     });
 
     let response = test_thinking_helper(&client, &payload).await;
@@ -280,12 +313,14 @@ async fn test_thinking_inference_extra_header_128k() {
         .await
         .unwrap();
 
-    // Check that 'extra_body' was written out correctly to ClickHouse (we don't store 'extra_headers'))
+    // Check that thinking_budget_tokens was stored correctly
+    let inference_params: Value =
+        serde_json::from_str(result["inference_params"].as_str().unwrap()).unwrap();
+    println!("inference_params: {inference_params:?}");
     assert_eq!(
-        result["extra_body"],
-        r#"[{"model_provider_name":"tensorzero::model_name::anthropic::claude-3-7-sonnet-20250219::provider_name::anthropic","pointer":"/thinking","value":{"type":"enabled","budget_tokens":1024}},{"model_provider_name":"tensorzero::model_name::anthropic::claude-3-7-sonnet-20250219::provider_name::anthropic","pointer":"/stop_sequences","value":["my_custom_stop"]}]"#
+        inference_params["chat_completion"]["thinking_budget_tokens"],
+        1024
     );
-    // We don't check anything else in the database, as we already do that in lots of places.
 }
 
 #[tokio::test]
@@ -307,6 +342,7 @@ async fn test_thinking_128k() {
             ]},
         "params": {
             "chat_completion": {
+                "thinking_budget_tokens": 1024,
                 "max_tokens": 128000,
             }
         },
@@ -405,7 +441,7 @@ async fn test_thinking_signature() {
         "messages": [
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "Hi I'm visiting Brooklyn from Brazil. What's the weather?"}]
+                "content": [{"type": "text", "text": "Hi I'm visiting Brooklyn from Brazil. What's the weather?"}]
             }
         ]
     });
@@ -440,9 +476,9 @@ async fn test_thinking_signature() {
     let inference_id_result = Uuid::parse_str(inference_id_result).unwrap();
     assert_eq!(inference_id_result, inference_id);
     let model_name = result.get("model_name").unwrap().as_str().unwrap();
-    assert_eq!(model_name, "claude-3-7-sonnet-20250219-thinking");
+    assert_eq!(model_name, "anthropic::claude-3-7-sonnet-20250219");
     let model_provider_name = result.get("model_provider_name").unwrap().as_str().unwrap();
-    assert_eq!(model_provider_name, "anthropic-extra-body");
+    assert_eq!(model_provider_name, "anthropic");
     let raw_request = result.get("raw_request").unwrap().as_str().unwrap();
     assert!(raw_request.to_lowercase().contains("weather"));
     // Check that raw_request is valid JSON
@@ -477,11 +513,16 @@ async fn test_thinking_signature() {
     println!("New messages: {new_messages:?}");
 
     let payload = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "episode_id": episode_id,
         "input": {
             "messages": new_messages
         },
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024,
+            }
+        },
         "stream": false,
     });
 
@@ -514,7 +555,7 @@ async fn test_redacted_thinking() {
     let episode_id = Uuid::now_v7();
 
     let payload = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "episode_id": episode_id,
         "input": {
             "messages": [
@@ -528,6 +569,11 @@ async fn test_redacted_thinking() {
                     "content": "What is the capital of Japan?"
                 }
             ]},
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024,
+            }
+        },
         "stream": false,
     });
 
@@ -582,11 +628,11 @@ async fn test_redacted_thinking() {
         "messages": [
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "ANTHROPIC_MAGIC_STRING_TRIGGER_REDACTED_THINKING_46C9A13E193C177646C7398A98432ECCCE4C1253D5E2D82641AC0E52CC2876CB"}]
+                "content": [{"type": "text", "text": "ANTHROPIC_MAGIC_STRING_TRIGGER_REDACTED_THINKING_46C9A13E193C177646C7398A98432ECCCE4C1253D5E2D82641AC0E52CC2876CB"}]
             },
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "What is the capital of Japan?"}]
+                "content": [{"type": "text", "text": "What is the capital of Japan?"}]
             }
         ]
     });
@@ -610,7 +656,7 @@ async fn test_redacted_thinking() {
     assert_eq!(retrieved_episode_id, episode_id);
     // Check the variant name
     let variant_name = result.get("variant_name").unwrap().as_str().unwrap();
-    assert_eq!(variant_name, "claude-3-7-sonnet-20250219-thinking");
+    assert_eq!(variant_name, "anthropic::claude-3-7-sonnet-20250219");
     // Check the processing time
     let processing_time_ms = result.get("processing_time_ms").unwrap().as_u64().unwrap();
     assert!(processing_time_ms > 0);
@@ -623,9 +669,9 @@ async fn test_redacted_thinking() {
     let inference_id_result = Uuid::parse_str(inference_id_result).unwrap();
     assert_eq!(inference_id_result, inference_id);
     let model_name = result.get("model_name").unwrap().as_str().unwrap();
-    assert_eq!(model_name, "claude-3-7-sonnet-20250219-thinking");
+    assert_eq!(model_name, "anthropic::claude-3-7-sonnet-20250219");
     let model_provider_name = result.get("model_provider_name").unwrap().as_str().unwrap();
-    assert_eq!(model_provider_name, "anthropic-extra-body");
+    assert_eq!(model_provider_name, "anthropic");
     let raw_request = result.get("raw_request").unwrap().as_str().unwrap();
     assert!(raw_request.to_lowercase().contains("japan"));
     // Check that raw_request is valid JSON
@@ -660,11 +706,16 @@ async fn test_redacted_thinking() {
     }));
 
     let payload = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "episode_id": episode_id,
         "input": {
             "messages": new_messages
         },
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024,
+            }
+        },
         "stream": false,
     });
 
@@ -692,7 +743,7 @@ async fn test_streaming_thinking() {
     let episode_id = Uuid::now_v7();
 
     let payload = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "episode_id": episode_id,
         "input": {
             "system": "Always thinking before responding",
@@ -723,6 +774,11 @@ async fn test_streaming_thinking() {
                 "strict": true,
             }
         ],
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024,
+            }
+        },
         "stream": true,
     });
 
@@ -781,7 +837,11 @@ async fn test_streaming_thinking() {
             }
         }
     }
-    assert_eq!(content_blocks.len(), 3);
+    assert!(
+        content_blocks.len() == 2 || content_blocks.len() == 3,
+        "Expected 2 or 3 content blocks, got {}",
+        content_blocks.len()
+    );
     assert_eq!(content_block_signatures.len(), 1);
     let inference_id = Uuid::parse_str(&inference_id.unwrap()).unwrap();
     // Sleep for 1 second to allow time for data to be inserted into ClickHouse (trailing writes from API)
@@ -803,7 +863,7 @@ async fn test_streaming_thinking() {
             "messages": [
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "What is the capital of Japan?"}]
+                "content": [{"type": "text", "text": "What is the capital of Japan?"}]
             }
         ]}
     );
@@ -813,11 +873,23 @@ async fn test_streaming_thinking() {
     let clickhouse_content_blocks: Vec<Value> =
         serde_json::from_str(clickhouse_content_blocks).unwrap();
     println!("Got content blocks: {clickhouse_content_blocks:?}");
-    // We should reconstruct thee blocks - a thought block, tool call, and text block
-    assert_eq!(clickhouse_content_blocks.len(), 3);
+    assert!(
+        clickhouse_content_blocks.len() == 2 || clickhouse_content_blocks.len() == 3,
+        "Expected 2 or 3 content blocks in ClickHouse, got {}",
+        clickhouse_content_blocks.len()
+    );
     assert_eq!(clickhouse_content_blocks[0]["type"], "thought");
-    assert_eq!(clickhouse_content_blocks[1]["type"], "text");
-    assert_eq!(clickhouse_content_blocks[2]["type"], "tool_call");
+
+    let tool_call_index = clickhouse_content_blocks.len() - 1;
+    let has_text_block = clickhouse_content_blocks.len() == 3;
+
+    if has_text_block {
+        assert_eq!(clickhouse_content_blocks[1]["type"], "text");
+    }
+    assert_eq!(
+        clickhouse_content_blocks[tool_call_index]["type"],
+        "tool_call"
+    );
 
     assert_eq!(
         clickhouse_content_blocks[0],
@@ -828,12 +900,17 @@ async fn test_streaming_thinking() {
             "_internal_provider_type": "anthropic",
         })
     );
-    assert_eq!(clickhouse_content_blocks[1]["text"], content_blocks["1"]);
 
-    let tool_call_id = clickhouse_content_blocks[2]["id"].as_str().unwrap();
+    if has_text_block {
+        assert_eq!(clickhouse_content_blocks[1]["text"], content_blocks["1"]);
+    }
+
+    let tool_call_id = clickhouse_content_blocks[tool_call_index]["id"]
+        .as_str()
+        .unwrap();
 
     assert_eq!(
-        clickhouse_content_blocks[2]["raw_arguments"],
+        clickhouse_content_blocks[tool_call_index]["raw_arguments"],
         content_blocks[tool_call_id]
     );
 
@@ -842,7 +919,7 @@ async fn test_streaming_thinking() {
     // Call Anthropic again with our reconstructed blocks, and make sure that it accepts the signed thought block
 
     let good_input = json!({
-        "model_name": "claude-3-7-sonnet-20250219-thinking",
+        "model_name": "anthropic::claude-3-7-sonnet-20250219",
         "episode_id": episode_id,
         "input": {
             "system": "Always thinking before responding",
@@ -853,23 +930,7 @@ async fn test_streaming_thinking() {
                 },
                 {
                     "role": "assistant",
-                    "content": [
-                        {
-                            "type": "thought",
-                            "text": content_blocks["0"],
-                            "signature": content_block_signatures["0"],
-                        },
-                        {
-                            "type": "text",
-                            "text": content_blocks["1"]
-                        },
-                        {
-                            "type": "tool_call",
-                            "name": "get_capital",
-                            "id": tool_call_id,
-                            "raw_arguments": content_blocks[tool_call_id].to_string(),
-                        },
-                    ]
+                    "content": clickhouse_content_blocks
                 },
                 {
                     "role": "user",
@@ -904,6 +965,11 @@ async fn test_streaming_thinking() {
                 "strict": true,
             }
         ],
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024,
+            }
+        },
         "stream": false,
     });
 
@@ -945,3 +1011,155 @@ async fn test_streaming_thinking() {
         "Unexpected error: {resp}"
     );
 }
+
+#[tokio::test]
+async fn test_forward_image_url() {
+    let temp_dir = tempfile::tempdir().unwrap();
+    let config = format!(
+        r#"
+    [object_storage]
+    type = "filesystem"
+    path = "{}"
+
+    [gateway]
+    fetch_and_encode_input_files_before_inference = false
+    "#,
+        temp_dir.path().to_string_lossy()
+    );
+
+    let client = make_embedded_gateway_with_config(&config).await;
+
+    let response = client.inference(ClientInferenceParams {
+        model_name: Some("anthropic::claude-3-haiku-20240307".to_string()),
+        input: ClientInput {
+            messages: vec![ClientInputMessage {
+                role: Role::User,
+                content: vec![ClientInputMessageContent::Text(TextKind::Text { text: "Describe the contents of the image".to_string() }),
+                ClientInputMessageContent::File(File::Url(UrlFile {
+                    url: Url::parse("https://raw.githubusercontent.com/tensorzero/tensorzero/ff3e17bbd3e32f483b027cf81b54404788c90dc1/tensorzero-internal/tests/e2e/providers/ferris.png").unwrap(),
+                    mime_type: Some(mime::IMAGE_PNG)
+                })),
+                ],
+            }],
+            ..Default::default()
+        },
+        ..Default::default()
+    }).await.unwrap();
+
+    let InferenceOutput::NonStreaming(response) = response else {
+        panic!("Expected non-streaming inference response");
+    };
+
+    // Sleep for 1 second to allow writing to ClickHouse
+    tokio::time::sleep(std::time::Duration::from_secs(1)).await;
+    let clickhouse = get_clickhouse().await;
+
+    let model_inference = select_model_inference_clickhouse(&clickhouse, response.inference_id())
+        .await
+        .unwrap();
+
+    let raw_request = model_inference
+        .get("raw_request")
+        .unwrap()
+        .as_str()
+        .unwrap();
+    assert_eq!(raw_request, "{\"model\":\"claude-3-haiku-20240307\",\"messages\":[{\"role\":\"user\",\"content\":[{\"type\":\"text\",\"text\":\"Describe the contents of the image\"},{\"type\":\"image\",\"source\":{\"type\":\"url\",\"url\":\"https://raw.githubusercontent.com/tensorzero/tensorzero/ff3e17bbd3e32f483b027cf81b54404788c90dc1/tensorzero-internal/tests/e2e/providers/ferris.png\"}}]}],\"max_tokens\":4096,\"stream\":false}");
+
+    let file_path =
+        "observability/files/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png";
+
+    // Check that the file exists on the filesystem
+    let result = std::fs::read(temp_dir.path().join(file_path)).unwrap();
+    assert_eq!(result, FERRIS_PNG);
+
+    println!("Got response: {response:#?}");
+
+    let InferenceResponse::Chat(response) = response else {
+        panic!("Expected chat inference response");
+    };
+    let text_block = &response.content[0];
+    let ContentBlockChatOutput::Text(text) = text_block else {
+        panic!("Expected text content block");
+    };
+    assert!(
+        text.text.to_lowercase().contains("cartoon")
+            || text.text.to_lowercase().contains("crab")
+            || text.text.to_lowercase().contains("animal"),
+        "Content should contain 'cartoon' or 'crab' or 'animal': {text:?}"
+    );
+}
+
+#[tokio::test]
+async fn test_forward_file_url() {
+    let temp_dir = tempfile::tempdir().unwrap();
+    let config = format!(
+        r#"
+    [object_storage]
+    type = "filesystem"
+    path = "{}"
+
+    [gateway]
+    fetch_and_encode_input_files_before_inference = false
+    "#,
+        temp_dir.path().to_string_lossy()
+    );
+
+    let client = make_embedded_gateway_with_config(&config).await;
+
+    let response = client.inference(ClientInferenceParams {
+        model_name: Some("anthropic::claude-sonnet-4-5-20250929".to_string()),
+        input: ClientInput {
+            messages: vec![ClientInputMessage {
+                role: Role::User,
+                content: vec![ClientInputMessageContent::Text(TextKind::Text { text: "Describe the contents of the PDF".to_string() }),
+                ClientInputMessageContent::File(File::Url(UrlFile {
+                    url: Url::parse("https://raw.githubusercontent.com/tensorzero/tensorzero/ac37477d56deaf6e0585a394eda68fd4f9390cab/tensorzero-core/tests/e2e/providers/deepseek_paper.pdf").unwrap(),
+                    mime_type: Some(mime::APPLICATION_PDF)
+                })),
+                ],
+            }],
+            ..Default::default()
+        },
+        ..Default::default()
+    }).await.unwrap();
+
+    let InferenceOutput::NonStreaming(response) = response else {
+        panic!("Expected non-streaming inference response");
+    };
+
+    // Sleep for 1 second to allow writing to ClickHouse
+    tokio::time::sleep(std::time::Duration::from_secs(1)).await;
+    let clickhouse = get_clickhouse().await;
+
+    let model_inference = select_model_inference_clickhouse(&clickhouse, response.inference_id())
+        .await
+        .unwrap();
+
+    let raw_request = model_inference
+        .get("raw_request")
+        .unwrap()
+        .as_str()
+        .unwrap();
+    assert_eq!(raw_request, "{\"model\":\"claude-sonnet-4-5-20250929\",\"messages\":[{\"role\":\"user\",\"content\":[{\"type\":\"text\",\"text\":\"Describe the contents of the PDF\"},{\"type\":\"document\",\"source\":{\"type\":\"url\",\"url\":\"https://raw.githubusercontent.com/tensorzero/tensorzero/ac37477d56deaf6e0585a394eda68fd4f9390cab/tensorzero-core/tests/e2e/providers/deepseek_paper.pdf\"}}]}],\"max_tokens\":64000,\"stream\":false}");
+
+    let file_path =
+        "observability/files/3e127d9a726f6be0fd81d73ccea97d96ec99419f59650e01d49183cd3be999ef.pdf";
+
+    // Check that the file exists on the filesystem
+    let result = std::fs::read(temp_dir.path().join(file_path)).unwrap();
+    assert_eq!(result, DEEPSEEK_PAPER_PDF);
+
+    println!("Got response: {response:#?}");
+
+    let InferenceResponse::Chat(response) = response else {
+        panic!("Expected chat inference response");
+    };
+    let text_block = &response.content[0];
+    let ContentBlockChatOutput::Text(text) = text_block else {
+        panic!("Expected text content block");
+    };
+    assert!(
+        text.text.to_lowercase().contains("deepseek"),
+        "Content should contain 'deepseek': {text:?}"
+    );
+}
