@@ -92,6 +92,8 @@ async fn get_providers() -> E2ETestProviders {
         embeddings: vec![],
         inference_params_inference: standard_providers.clone(),
         inference_params_dynamic_credentials: vec![],
+        provider_type_default_credentials: vec![],
+        provider_type_default_credentials_shorthand: vec![],
         tool_use_inference: standard_providers.clone(),
         tool_multi_turn_inference: standard_providers.clone(),
         dynamic_tool_use_inference: standard_providers.clone(),
@@ -102,6 +104,8 @@ async fn get_providers() -> E2ETestProviders {
         pdf_inference: standard_providers.clone(),
 
         shorthand_inference: vec![],
+        // AWS bedrock only works with SDK credentials
+        credential_fallbacks: vec![],
     }
 }
 
@@ -164,7 +168,7 @@ async fn test_inference_with_explicit_region() {
         "messages": [
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "Hello, world!"}]
+                "content": [{"type": "text", "text": "Hello, world!"}]
             }
         ]
     });
@@ -250,7 +254,7 @@ async fn test_inference_with_explicit_broken_region() {
         .await
         .unwrap();
 
-    assert_eq!(response.status(), StatusCode::BAD_GATEWAY);
+    assert_eq!(response.status(), StatusCode::INTERNAL_SERVER_ERROR);
 
     let response_json = response.json::<Value>().await.unwrap();
 
@@ -271,7 +275,7 @@ async fn test_inference_with_empty_system() {
             "messages": [
                 {
                     "role": "user",
-                    "content": [{"type": "text", "arguments": {"topic": "artificial intelligence"}}]
+                    "content": [{"type": "template", "name": "user", "arguments": {"topic": "artificial intelligence"}}]
                 }
             ]},
         "stream": false,
@@ -293,3 +297,101 @@ async fn test_inference_with_empty_system() {
     assert_eq!(content_block_type, "text");
     content_block.get("text").unwrap().as_str().unwrap();
 }
+
+#[tokio::test]
+async fn test_inference_with_thinking_budget_tokens() {
+    let client = Client::new();
+    let episode_id = Uuid::now_v7();
+
+    let payload = json!({
+        "function_name": "basic_test",
+        "variant_name": "aws-bedrock-thinking",
+        "episode_id": episode_id,
+        "input": {
+            "system": {"assistant_name": "Bedrock Thinker"},
+            "messages": [
+                {
+                    "role": "user",
+                    "content": "Share a short fun fact."
+                }
+            ]
+        },
+        "stream": false,
+        "params": {
+            "chat_completion": {
+                "thinking_budget_tokens": 1024
+            }
+        }
+    });
+
+    let response = client
+        .post(get_gateway_endpoint("/inference"))
+        .json(&payload)
+        .send()
+        .await
+        .unwrap();
+
+    assert_eq!(response.status(), StatusCode::OK);
+    let response_json = response.json::<Value>().await.unwrap();
+    let content_blocks = response_json
+        .get("content")
+        .and_then(Value::as_array)
+        .expect("content must be an array");
+    assert!(
+        content_blocks.iter().any(|block| {
+            block
+                .get("type")
+                .and_then(Value::as_str)
+                .is_some_and(|t| t == "thought")
+        }),
+        "response should include at least one thought block: {response_json:#?}"
+    );
+    assert!(
+        content_blocks.iter().any(|block| {
+            block
+                .get("type")
+                .and_then(Value::as_str)
+                .is_some_and(|t| t == "text")
+        }),
+        "response should include at least one text block: {response_json:#?}"
+    );
+
+    let inference_id = response_json
+        .get("inference_id")
+        .and_then(Value::as_str)
+        .unwrap();
+    let inference_id = Uuid::parse_str(inference_id).unwrap();
+
+    tokio::time::sleep(std::time::Duration::from_secs(1)).await;
+
+    let clickhouse = get_clickhouse().await;
+    let model_inference = select_model_inference_clickhouse(&clickhouse, inference_id)
+        .await
+        .unwrap();
+
+    let raw_request = model_inference
+        .get("raw_request")
+        .and_then(Value::as_str)
+        .unwrap();
+
+    let raw_request_json: Value =
+        serde_json::from_str(raw_request).expect("raw_request should be valid JSON");
+
+    let thinking = raw_request_json
+        .get("additionalModelRequestFields")
+        .and_then(|fields| fields.get("thinking"))
+        .expect("Expected `thinking` block to be forwarded to AWS Bedrock");
+
+    let thinking_type = thinking
+        .get("type")
+        .and_then(Value::as_str)
+        .expect("Expected thinking type");
+    assert_eq!(thinking_type, "enabled");
+
+    let budget_tokens = thinking
+        .get("budget_tokens")
+        .and_then(Value::as_i64)
+        .or_else(|| thinking.get("budgetTokens").and_then(Value::as_i64))
+        .expect("Expected thinking budget tokens");
+    assert_eq!(budget_tokens, 1024);
+}
