@@ -1,5 +1,3 @@
-#![allow(clippy::print_stdout)]
-
 use std::collections::HashMap;
 
 use http::StatusCode;
@@ -12,6 +10,8 @@ use crate::{
     providers::common::{E2ETestProvider, E2ETestProviders},
 };
 
+use super::common::ModelTestProvider;
+
 crate::generate_provider_tests!(get_providers);
 crate::generate_batch_inference_tests!(get_providers);
 
@@ -137,6 +137,16 @@ async fn get_providers() -> E2ETestProviders {
         credentials: HashMap::new(),
     }];
 
+    let credential_fallbacks = vec![ModelTestProvider {
+        provider_type: "gcp_vertex_gemini".to_string(),
+        model_info: HashMap::from([
+            ("model_id".to_string(), "gemini-2.0-flash-001".to_string()),
+            ("location".to_string(), "us-central1".to_string()),
+            ("project_id".to_string(), "tensorzero-public".to_string()),
+        ]),
+        use_modal_headers: false,
+    }];
+
     E2ETestProviders {
         simple_inference: standard_providers.clone(),
         extra_body_inference: extra_body_providers,
@@ -145,6 +155,8 @@ async fn get_providers() -> E2ETestProviders {
         embeddings: vec![],
         inference_params_inference: standard_providers.clone(),
         inference_params_dynamic_credentials: vec![],
+        provider_type_default_credentials: vec![],
+        provider_type_default_credentials_shorthand: vec![],
         tool_use_inference: tool_providers.clone(),
         tool_multi_turn_inference: tool_providers.clone(),
         dynamic_tool_use_inference: tool_providers.clone(),
@@ -154,6 +166,7 @@ async fn get_providers() -> E2ETestProviders {
         image_inference: image_providers.clone(),
         pdf_inference: pdf_providers,
         shorthand_inference: shorthand_providers.clone(),
+        credential_fallbacks,
     }
 }
 
@@ -243,7 +256,7 @@ async fn test_gcp_vertex_gemini_bad_model_id() {
     let response_json: Value = response.json().await.unwrap();
     println!("API response: {response_json:#?}");
 
-    assert_eq!(status, StatusCode::BAD_GATEWAY);
+    assert_eq!(status, StatusCode::INTERNAL_SERVER_ERROR);
     let error = response_json.get("error").unwrap().as_str().unwrap();
     assert!(error.contains("Model or endpoint not found. You may be specifying the wrong one of these. Standard GCP models should use a `model_id` and not an `endpoint_id`, while fine-tuned models should use an `endpoint_id`."));
 }
