@@ -70,3 +70,28 @@ pub fn get_extra_headers() -> UnfilteredInferenceExtraHeaders {
     }
     UnfilteredInferenceExtraHeaders { extra_headers }
 }
+
+/// Get extra headers for the test-model/test-provider configuration
+/// used in credential fallback tests.
+pub fn get_test_model_extra_headers() -> UnfilteredInferenceExtraHeaders {
+    let mut headers = Vec::new();
+    if let Ok(modal_key) = env::var("MODAL_KEY") {
+        headers.push(InferenceExtraHeader::Provider {
+            model_provider_name: "tensorzero::model_name::test-model::provider_name::test-provider"
+                .to_string(),
+            name: "Modal-Key".to_string(),
+            kind: ExtraHeaderKind::Value(modal_key),
+        });
+    }
+    if let Ok(modal_secret) = env::var("MODAL_SECRET") {
+        headers.push(InferenceExtraHeader::Provider {
+            model_provider_name: "tensorzero::model_name::test-model::provider_name::test-provider"
+                .to_string(),
+            name: "Modal-Secret".to_string(),
+            kind: ExtraHeaderKind::Value(modal_secret),
+        });
+    }
+    UnfilteredInferenceExtraHeaders {
+        extra_headers: headers,
+    }
+}
