@@ -3,17 +3,20 @@ use object_store::path::Path;
 use serde_json::json;
 use std::collections::HashMap;
 use tensorzero::{
-    ChatInferenceDatapoint, Datapoint, JsonInferenceDatapoint, Role, StorageKind, StoragePath,
-    StoredChatInference, StoredInference, StoredJsonInference, Tool,
+    JsonInferenceDatapoint, Role, StorageKind, StoragePath, StoredChatInferenceDatabase,
+    StoredChatInferenceDatapoint, StoredDatapoint, StoredInferenceDatabase, StoredJsonInference,
+    Tool,
 };
-use tensorzero_core::inference::types::file::Base64FileMetadata;
+use tensorzero_core::inference::types::file::ObjectStoragePointer;
 use tensorzero_core::inference::types::stored_input::StoredFile;
 use tensorzero_core::inference::types::stored_input::{
     StoredInput, StoredInputMessage, StoredInputMessageContent,
 };
-use tensorzero_core::inference::types::{ResolvedContentBlock, ResolvedRequestMessage};
+use tensorzero_core::inference::types::{
+    Arguments, ResolvedContentBlock, ResolvedRequestMessage, System,
+};
 use tensorzero_core::{
-    inference::types::{ContentBlockChatOutput, JsonInferenceOutput, Text},
+    inference::types::{ContentBlockChatOutput, JsonInferenceOutput, Template, Text},
     tool::{ToolCallConfigDatabaseInsert, ToolCallOutput, ToolChoice},
 };
 use tracing_test::traced_test;
@@ -25,7 +28,7 @@ pub async fn test_render_samples_empty() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     // Test with an empty stored inferences array.
-    let stored_inferences: Vec<StoredInference> = vec![];
+    let stored_inferences: Vec<StoredInferenceDatabase> = vec![];
     let rendered_inferences = client
         .experimental_render_samples(stored_inferences, HashMap::new())
         .await
@@ -40,16 +43,16 @@ pub async fn test_render_samples_empty() {
 pub async fn test_render_samples_no_function() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
+    let stored_inferences = vec![StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
         function_name: "basic_test".to_string(),
         variant_name: "dummy".to_string(),
         input: StoredInput {
             system: None,
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: vec![],
@@ -76,16 +79,16 @@ pub async fn test_render_samples_no_function() {
 pub async fn test_render_samples_no_variant() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
+    let stored_inferences = vec![StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
         function_name: "basic_test".to_string(),
         variant_name: "dummy".to_string(),
         input: StoredInput {
             system: None,
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: vec![],
@@ -120,16 +123,19 @@ pub async fn test_render_samples_no_variant() {
 pub async fn test_render_samples_missing_variable() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
+    let stored_inferences = vec![StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
         function_name: "basic_test".to_string(),
         variant_name: "dummy".to_string(),
         input: StoredInput {
-            system: Some(json!({"foo": "bar"})),
+            system: Some(System::Template(Arguments(serde_json::Map::from_iter([(
+                "foo".to_string(),
+                "bar".into(),
+            )])))),
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: vec![],
@@ -159,16 +165,18 @@ pub async fn test_render_samples_normal() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let stored_inferences = vec![
-        StoredInference::Chat(StoredChatInference {
+        StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
             function_name: "basic_test".to_string(),
             variant_name: "dummy".to_string(),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("Hello, world!"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "Hello, world!".to_string(),
+                    })],
                 }],
             },
             output: vec![],
@@ -179,16 +187,19 @@ pub async fn test_render_samples_normal() {
             dispreferred_outputs: vec![],
             tags: HashMap::new(),
         }),
-        StoredInference::Json(StoredJsonInference {
+        StoredInferenceDatabase::Json(StoredJsonInference {
             function_name: "json_success".to_string(),
             variant_name: "dummy".to_string(),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!({"country": "Japan"}),
-                    }],
+                    content: vec![StoredInputMessageContent::Template(Template {
+                        name: "user".to_string(),
+                        arguments: Arguments(serde_json::Map::from_iter(vec![("country".to_string(), json!("Japan"))])),
+                    })],
                 }],
             },
             output: JsonInferenceOutput {
@@ -205,16 +216,18 @@ pub async fn test_render_samples_normal() {
             }],
             tags: HashMap::new(),
         }),
-        StoredInference::Chat(StoredChatInference {
+        StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
             function_name: "weather_helper".to_string(),
             variant_name: "dummy".to_string(),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("Hello, world!"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "Hello, world!".to_string(),
+                    })],
                 }],
             },
             output: vec![ContentBlockChatOutput::ToolCall(ToolCallOutput {
@@ -242,33 +255,35 @@ pub async fn test_render_samples_normal() {
             })]],
             tags: HashMap::new(),
         }),
-        StoredInference::Chat(StoredChatInference {
+        StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
             function_name: "basic_test".to_string(),
             variant_name: "gpt-4o-mini-2024-07-18".to_string(),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: json!("What is this a picture of?"),
-                        },
-                        StoredInputMessageContent::File(Box::new(StoredFile {
-                            file: Base64FileMetadata {
-                                url: None,
+                        StoredInputMessageContent::Text(Text {
+                            text: "What is this a picture of?".to_string(),
+                        }),
+                        StoredInputMessageContent::File(Box::new(StoredFile(
+                            ObjectStoragePointer {
+                                source_url: None,
                                 mime_type: mime::IMAGE_PNG,
-                            },
-                            storage_path: StoragePath {
-                                kind: StorageKind::S3Compatible {
-                                    bucket_name: Some("tensorzero-e2e-test-images".to_string()),
-                                    region: Some("us-east-1".to_string()),
-                                    prefix: String::new(),
-                                    endpoint: None,
-                                    allow_http: None,
+                                storage_path: StoragePath {
+                                    kind: StorageKind::S3Compatible {
+                                        bucket_name: Some("tensorzero-e2e-test-images".to_string()),
+                                        region: Some("us-east-1".to_string()),
+                                        prefix: String::new(),
+                                        endpoint: None,
+                                        allow_http: None,
+                                    },
+                                    path: Path::from("observability/images/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"),
                                 },
-                                path: Path::from("observability/images/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"),
                             },
-                        })),
+                        ))),
                     ],
                 }],
             },
@@ -315,7 +330,6 @@ pub async fn test_render_samples_normal() {
 
     // Check other fields
     assert!(first_inference.output.as_ref().unwrap().is_empty());
-    assert!(first_inference.tool_params.is_some());
     assert!(first_inference.output_schema.is_none());
 
     // Check the second rendered inference
@@ -354,7 +368,6 @@ pub async fn test_render_samples_normal() {
     };
     assert_eq!(output_text.text, "{}");
     // Check other fields
-    assert!(second_inference.tool_params.is_none());
     assert!(second_inference.output_schema.is_some());
 
     // Check the third rendered inference
@@ -396,7 +409,6 @@ pub async fn test_render_samples_normal() {
     };
     assert_eq!(output_text.text, "Hello, world!");
     // Check other fields
-    assert!(third_inference.tool_params.is_some());
     assert!(third_inference.output_schema.is_none());
 
     // Check the fourth rendered inference
@@ -424,13 +436,12 @@ pub async fn test_render_samples_normal() {
     };
 
     // Check that the base64 string is > 1000 chars
-    assert!(file.file.data.len() > 1000);
+    assert!(file.data.len() > 1000);
 
     // Check the output
     assert_eq!(fourth_inference.output.as_ref().unwrap().len(), 0);
 
     // Check other fields
-    assert!(fourth_inference.tool_params.is_some());
     assert!(fourth_inference.output_schema.is_none());
 }
 
@@ -439,33 +450,33 @@ pub async fn test_render_samples_normal() {
 pub async fn test_render_samples_template_no_schema() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let stored_inferences = vec![StoredInference::Chat(StoredChatInference {
+    let stored_inferences = vec![StoredInferenceDatabase::Chat(StoredChatInferenceDatabase {
         function_name: "basic_test_template_no_schema".to_string(),
         variant_name: "test".to_string(),
         timestamp: Utc::now(),
         input: StoredInput {
-            system: Some("My system message".into()),
+            system: Some(System::Text("My system message".to_string())),
             messages: vec![
                 StoredInputMessage {
                     role: Role::User,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: "First user message".into(),
-                        },
-                        StoredInputMessageContent::Text {
-                            value: "Second user message".into(),
-                        },
+                        StoredInputMessageContent::Text(Text {
+                            text: "First user message".to_string(),
+                        }),
+                        StoredInputMessageContent::Text(Text {
+                            text: "Second user message".to_string(),
+                        }),
                     ],
                 },
                 StoredInputMessage {
                     role: Role::Assistant,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: "First assistant message".into(),
-                        },
-                        StoredInputMessageContent::Text {
-                            value: "Second assistant message".into(),
-                        },
+                        StoredInputMessageContent::Text(Text {
+                            text: "First assistant message".to_string(),
+                        }),
+                        StoredInputMessageContent::Text(Text {
+                            text: "Second assistant message".to_string(),
+                        }),
                     ],
                 },
             ],
@@ -531,7 +542,6 @@ pub async fn test_render_samples_template_no_schema() {
 
     // Check other fields
     assert!(first_inference.output.as_ref().unwrap().is_empty());
-    assert!(first_inference.tool_params.is_some());
     assert!(first_inference.output_schema.is_none());
 }
 
@@ -544,7 +554,7 @@ pub async fn test_render_datapoints_empty() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     // Test with an empty datapoints array.
-    let datapoints: Vec<Datapoint> = vec![];
+    let datapoints: Vec<StoredDatapoint> = vec![];
     let rendered_samples = client
         .experimental_render_samples(datapoints, HashMap::new())
         .await
@@ -559,18 +569,19 @@ pub async fn test_render_datapoints_empty() {
 pub async fn test_render_datapoints_no_function() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
+    let datapoints = vec![StoredDatapoint::Chat(StoredChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
         function_name: "basic_test".to_string(),
+        name: None,
         id: Uuid::now_v7(),
         episode_id: Some(Uuid::now_v7()),
         input: StoredInput {
             system: None,
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: Some(vec![]),
@@ -580,6 +591,7 @@ pub async fn test_render_datapoints_no_function() {
         is_deleted: false,
         source_inference_id: None,
         staled_at: None,
+        updated_at: "2025-10-13T20:17:36Z".to_string(),
         is_custom: false,
     })];
 
@@ -598,18 +610,19 @@ pub async fn test_render_datapoints_no_function() {
 pub async fn test_render_datapoints_no_variant() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
+    let datapoints = vec![StoredDatapoint::Chat(StoredChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
         function_name: "basic_test".to_string(),
+        name: None,
         id: Uuid::now_v7(),
         episode_id: Some(Uuid::now_v7()),
         input: StoredInput {
             system: None,
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: Some(vec![]),
@@ -619,6 +632,7 @@ pub async fn test_render_datapoints_no_variant() {
         is_deleted: false,
         source_inference_id: None,
         staled_at: None,
+        updated_at: "2025-10-13T20:17:36Z".to_string(),
         is_custom: false,
     })];
 
@@ -645,18 +659,22 @@ pub async fn test_render_datapoints_no_variant() {
 pub async fn test_render_datapoints_missing_variable() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
+    let datapoints = vec![StoredDatapoint::Chat(StoredChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
         function_name: "basic_test".to_string(),
+        name: None,
         id: Uuid::now_v7(),
         episode_id: Some(Uuid::now_v7()),
         input: StoredInput {
-            system: Some(json!({"foo": "bar"})),
+            system: Some(System::Template(Arguments(serde_json::Map::from_iter([(
+                "foo".to_string(),
+                "bar".into(),
+            )])))),
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!("Hello, world!"),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "Hello, world!".to_string(),
+                })],
             }],
         },
         output: Some(vec![]),
@@ -666,6 +684,7 @@ pub async fn test_render_datapoints_missing_variable() {
         is_deleted: false,
         source_inference_id: None,
         staled_at: None,
+        updated_at: "2025-10-13T20:17:36Z".to_string(),
         is_custom: false,
     })];
 
@@ -687,18 +706,21 @@ pub async fn test_render_datapoints_normal() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
     let datapoints = vec![
-        Datapoint::Chat(ChatInferenceDatapoint {
+        StoredDatapoint::Chat(StoredChatInferenceDatapoint {
             dataset_name: "test_dataset".to_string(),
             function_name: "basic_test".to_string(),
+            name: None,
             id: Uuid::now_v7(),
             episode_id: Some(Uuid::now_v7()),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("Hello, world!"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "Hello, world!".to_string(),
+                    })],
                 }],
             },
             output: Some(vec![]),
@@ -708,20 +730,25 @@ pub async fn test_render_datapoints_normal() {
             is_deleted: false,
             source_inference_id: None,
             staled_at: None,
+            updated_at: "2025-10-13T20:17:36Z".to_string(),
             is_custom: false,
         }),
-        Datapoint::Json(JsonInferenceDatapoint {
+        StoredDatapoint::Json(JsonInferenceDatapoint {
             dataset_name: "test_dataset".to_string(),
             function_name: "json_success".to_string(),
+            name: None,
             id: Uuid::now_v7(),
             episode_id: Some(Uuid::now_v7()),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!({"country": "Japan"}),
-                    }],
+                    content: vec![StoredInputMessageContent::Template(Template {
+                        name: "user".to_string(),
+                        arguments: Arguments(serde_json::Map::from_iter(vec![("country".to_string(), json!("Japan"))])),
+                    })],
                 }],
             },
             output: Some(JsonInferenceOutput {
@@ -734,20 +761,24 @@ pub async fn test_render_datapoints_normal() {
             is_deleted: false,
             source_inference_id: None,
             staled_at: None,
+            updated_at: "2025-10-13T20:17:36Z".to_string(),
             is_custom: false,
         }),
-        Datapoint::Chat(ChatInferenceDatapoint {
+        StoredDatapoint::Chat(StoredChatInferenceDatapoint {
             dataset_name: "test_dataset".to_string(),
             function_name: "weather_helper".to_string(),
+            name: None,
             id: Uuid::now_v7(),
             episode_id: Some(Uuid::now_v7()),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("Hello, world!"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "Hello, world!".to_string(),
+                    })],
                 }],
             },
             output: Some(vec![ContentBlockChatOutput::ToolCall(ToolCallOutput {
@@ -772,37 +803,41 @@ pub async fn test_render_datapoints_normal() {
             is_deleted: false,
             source_inference_id: None,
             staled_at: None,
+            updated_at: "2025-10-13T20:17:36Z".to_string(),
             is_custom: false,
         }),
-        Datapoint::Chat(ChatInferenceDatapoint {
+        StoredDatapoint::Chat(StoredChatInferenceDatapoint {
             dataset_name: "test_dataset".to_string(),
             function_name: "basic_test".to_string(),
+            name: None,
             id: Uuid::now_v7(),
             episode_id: Some(Uuid::now_v7()),
             input: StoredInput {
-                system: Some(json!({"assistant_name": "Dr. Mehta"})),
+                system: Some(System::Template(Arguments(serde_json::Map::from_iter([
+                    ("assistant_name".to_string(), "Dr. Mehta".into()),
+                ])))),
                 messages: vec![StoredInputMessage {
                     role: Role::User,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: json!("What is this a picture of?"),
-                        },
-                        StoredInputMessageContent::File(Box::new(StoredFile {
-                            file: Base64FileMetadata {
-                                url: None,
+                        StoredInputMessageContent::Text(Text {
+                            text: "What is this a picture of?".to_string(),
+                        }),
+                        StoredInputMessageContent::File(Box::new(StoredFile(
+                            ObjectStoragePointer {
+                                source_url: None,
                                 mime_type: mime::IMAGE_PNG,
-                            },
-                            storage_path: StoragePath {
-                                kind: StorageKind::S3Compatible {
-                                    bucket_name: Some("tensorzero-e2e-test-images".to_string()),
-                                    region: Some("us-east-1".to_string()),
-                                    prefix: String::new(),
-                                    endpoint: None,
-                                    allow_http: None,
+                                storage_path: StoragePath {
+                                    kind: StorageKind::S3Compatible {
+                                        bucket_name: Some("tensorzero-e2e-test-images".to_string()),
+                                        region: Some("us-east-1".to_string()),
+                                        prefix: String::new(),
+                                        endpoint: None,
+                                        allow_http: None,
+                                    },
+                                    path: Path::from("observability/images/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"),
                                 },
-                                path: Path::from("observability/images/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"),
                             },
-                        })),
+                        ))),
                     ],
                 }],
             },
@@ -813,6 +848,7 @@ pub async fn test_render_datapoints_normal() {
             is_deleted: false,
             source_inference_id: None,
             staled_at: None,
+            updated_at: "2025-10-13T20:17:36Z".to_string(),
             is_custom: false,
         }),
     ];
@@ -850,7 +886,6 @@ pub async fn test_render_datapoints_normal() {
 
     // Check other fields
     assert!(first_sample.output.as_ref().unwrap().is_empty());
-    assert!(first_sample.tool_params.is_some());
     assert!(first_sample.output_schema.is_none());
 
     // Check the second rendered sample (json_success)
@@ -882,7 +917,6 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(output_text.text, "{}");
 
     // Check other fields
-    assert!(second_sample.tool_params.is_none());
     assert!(second_sample.output_schema.is_some());
 
     // Check the third rendered sample (weather_helper with tool call)
@@ -917,7 +951,6 @@ pub async fn test_render_datapoints_normal() {
     assert_eq!(tool_call.arguments, Some(json!({"location": "Tokyo"})));
 
     // Check other fields
-    assert!(third_sample.tool_params.is_some());
     assert!(third_sample.output_schema.is_none());
 
     // Check the fourth rendered sample (basic_test with image)
@@ -945,13 +978,12 @@ pub async fn test_render_datapoints_normal() {
     };
 
     // Check that the base64 string is > 1000 chars
-    assert!(file.file.data.len() > 1000);
+    assert!(file.data.len() > 1000);
 
     // Check the output
     assert_eq!(fourth_sample.output.as_ref().unwrap().len(), 0);
 
     // Check other fields
-    assert!(fourth_sample.tool_params.is_some());
     assert!(fourth_sample.output_schema.is_none());
 }
 
@@ -960,34 +992,35 @@ pub async fn test_render_datapoints_normal() {
 pub async fn test_render_datapoints_template_no_schema() {
     let client = tensorzero::test_helpers::make_embedded_gateway().await;
 
-    let datapoints = vec![Datapoint::Chat(ChatInferenceDatapoint {
+    let datapoints = vec![StoredDatapoint::Chat(StoredChatInferenceDatapoint {
         dataset_name: "test_dataset".to_string(),
         function_name: "basic_test_template_no_schema".to_string(),
+        name: None,
         id: Uuid::now_v7(),
         episode_id: Some(Uuid::now_v7()),
         input: StoredInput {
-            system: Some("My system message".into()),
+            system: Some(System::Text("My system message".to_string())),
             messages: vec![
                 StoredInputMessage {
                     role: Role::User,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: "First user message".into(),
-                        },
-                        StoredInputMessageContent::Text {
-                            value: "Second user message".into(),
-                        },
+                        StoredInputMessageContent::Text(Text {
+                            text: "First user message".to_string(),
+                        }),
+                        StoredInputMessageContent::Text(Text {
+                            text: "Second user message".to_string(),
+                        }),
                     ],
                 },
                 StoredInputMessage {
                     role: Role::Assistant,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: "First assistant message".into(),
-                        },
-                        StoredInputMessageContent::Text {
-                            value: "Second assistant message".into(),
-                        },
+                        StoredInputMessageContent::Text(Text {
+                            text: "First assistant message".to_string(),
+                        }),
+                        StoredInputMessageContent::Text(Text {
+                            text: "Second assistant message".to_string(),
+                        }),
                     ],
                 },
             ],
@@ -999,6 +1032,7 @@ pub async fn test_render_datapoints_template_no_schema() {
         is_deleted: false,
         source_inference_id: None,
         staled_at: None,
+        updated_at: "2025-10-13T20:17:36Z".to_string(),
         is_custom: false,
     })];
 
@@ -1055,6 +1089,5 @@ pub async fn test_render_datapoints_template_no_schema() {
 
     // Check other fields
     assert!(first_sample.output.as_ref().unwrap().is_empty());
-    assert!(first_sample.tool_params.is_some());
     assert!(first_sample.output_schema.is_none());
 }
