@@ -9,23 +9,22 @@ use uuid::Uuid;
 use super::use_mock_inference_provider;
 use tensorzero::{
     ClientInferenceParams, ClientInput, ClientInputMessage, ClientInputMessageContent,
-    InferenceOutput, InferenceOutputSource, LaunchOptimizationWorkflowParams, RenderedSample, Role,
+    DynamicToolParams, InferenceOutput, InferenceOutputSource, LaunchOptimizationWorkflowParams,
+    RenderedSample, Role, System,
 };
 use tensorzero_core::{
     config::{Config, ConfigFileGlob, UninitializedVariantConfig},
-    db::clickhouse::{
-        test_helpers::{
-            get_clickhouse, select_chat_inference_clickhouse, select_json_inference_clickhouse,
-            select_model_inferences_clickhouse, CLICKHOUSE_URL,
-        },
-        ClickhouseFormat,
+    db::clickhouse::test_helpers::{
+        get_clickhouse, select_chat_inference_clickhouse, select_json_inference_clickhouse,
+        select_model_inferences_clickhouse, CLICKHOUSE_URL,
     },
     http::TensorzeroHttpClient,
     inference::types::{
-        ContentBlockChatOutput, ContentBlockChunk, JsonInferenceOutput, ModelInput,
+        Arguments, ContentBlockChatOutput, ContentBlockChunk, JsonInferenceOutput, ModelInput,
         ResolvedContentBlock, ResolvedRequestMessage, StoredContentBlock, StoredInput,
         StoredInputMessage, StoredInputMessageContent, StoredRequestMessage, Text, TextKind, Usage,
     },
+    model_table::ProviderTypeDefaultCredentials,
     optimization::{
         dicl::UninitializedDiclOptimizationConfig, JobHandle, OptimizationJobInfo, Optimizer,
         OptimizerOutput, UninitializedOptimizerConfig, UninitializedOptimizerInfo,
@@ -88,7 +87,10 @@ pub async fn test_dicl_optimization_chat() {
         }),
     };
 
-    let optimizer_info = uninitialized_optimizer_info.load().await.unwrap();
+    let optimizer_info = uninitialized_optimizer_info
+        .load(&ProviderTypeDefaultCredentials::default())
+        .await
+        .unwrap();
     let client = TensorzeroHttpClient::new().unwrap();
     let test_examples = get_pinocchio_examples(false);
     let val_examples = None; // No validation examples needed for this test
@@ -128,7 +130,14 @@ pub async fn test_dicl_optimization_chat() {
 
     let mut status;
     loop {
-        status = job_handle.poll(&client, &credentials).await.unwrap();
+        status = job_handle
+            .poll(
+                &client,
+                &credentials,
+                &ProviderTypeDefaultCredentials::default(),
+            )
+            .await
+            .unwrap();
         println!("Status: `{status:?}` Handle: `{job_handle}`");
         if matches!(status, OptimizationJobInfo::Completed { .. }) {
             break;
@@ -165,9 +174,9 @@ pub async fn test_dicl_optimization_chat() {
     };
 
     // Validate that the returned config matches our input
-    assert_eq!(dicl_config.embedding_model.as_ref(), embedding_model);
+    assert_eq!(&dicl_config.embedding_model, &embedding_model);
     assert_eq!(dicl_config.k, k);
-    assert_eq!(dicl_config.model.as_ref(), model);
+    assert_eq!(&dicl_config.model, &model);
 
     // Test DICL variant inference by creating a temporary config
     let (config_path, _temp_dir) = create_dicl_test_files(
@@ -193,7 +202,10 @@ pub async fn test_dicl_optimization_chat() {
 
     // Test inference with the DICL variant using Pinocchio pattern
     let input = ClientInput {
-        system: Some(serde_json::json!({"assistant_name": "Pinocchio"})),
+        system: Some(System::Template(Arguments(serde_json::Map::from_iter([(
+            "assistant_name".to_string(),
+            "Pinocchio".into(),
+        )])))),
         messages: vec![ClientInputMessage {
             role: Role::User,
             content: vec![ClientInputMessageContent::Text(TextKind::Text {
@@ -355,7 +367,10 @@ pub async fn test_dicl_optimization_json() {
         }),
     };
 
-    let optimizer_info = uninitialized_optimizer_info.load().await.unwrap();
+    let optimizer_info = uninitialized_optimizer_info
+        .load(&ProviderTypeDefaultCredentials::default())
+        .await
+        .unwrap();
 
     let client = TensorzeroHttpClient::new().unwrap();
     let test_examples = get_pinocchio_examples(true);
@@ -396,7 +411,14 @@ pub async fn test_dicl_optimization_json() {
 
     let mut status;
     loop {
-        status = job_handle.poll(&client, &credentials).await.unwrap();
+        status = job_handle
+            .poll(
+                &client,
+                &credentials,
+                &ProviderTypeDefaultCredentials::default(),
+            )
+            .await
+            .unwrap();
         println!("Status: `{status:?}` Handle: `{job_handle}`");
         if matches!(status, OptimizationJobInfo::Completed { .. }) {
             break;
@@ -433,9 +455,9 @@ pub async fn test_dicl_optimization_json() {
     };
 
     // Validate that the returned config matches our input
-    assert_eq!(dicl_config.embedding_model.as_ref(), embedding_model);
+    assert_eq!(&dicl_config.embedding_model, &embedding_model);
     assert_eq!(dicl_config.k, k, "k value should match input");
-    assert_eq!(dicl_config.model.as_ref(), model);
+    assert_eq!(&dicl_config.model, &model);
 
     // Test DICL variant inference by creating a temporary config
     let (config_path, _temp_dir) = create_dicl_test_files(
@@ -461,7 +483,10 @@ pub async fn test_dicl_optimization_json() {
 
     // Test inference with the DICL variant using Pinocchio pattern
     let input = ClientInput {
-        system: Some(serde_json::json!({"assistant_name": "Pinocchio"})),
+        system: Some(System::Template(Arguments(serde_json::Map::from_iter([(
+            "assistant_name".to_string(),
+            "Pinocchio".into(),
+        )])))),
         messages: vec![ClientInputMessage {
             role: Role::User,
             content: vec![ClientInputMessageContent::Text(TextKind::Text {
@@ -621,6 +646,7 @@ fn create_inference_params(
         extra_body: Default::default(),
         extra_headers: Default::default(),
         internal_dynamic_variant_config: None,
+        otlp_traces_extra_headers: Default::default(),
     }
 }
 
@@ -795,7 +821,7 @@ async fn validate_inference_clickhouse(
         "messages": [
             {
                 "role": "user",
-                "content": [{"type": "text", "value": "Who was the author of the Harry Potter series?"}]
+                "content": [{"type": "text", "text": "Who was the author of the Harry Potter series?"}]
             }
         ]
     });
@@ -1045,7 +1071,6 @@ pub async fn run_dicl_workflow_with_client(client: &tensorzero::Client) {
         limit: Some(10),
         offset: None,
         val_fraction: None,
-        format: ClickhouseFormat::JsonEachRow,
         // We always mock the client tests since this is tested above
         optimizer_config: UninitializedOptimizerInfo {
             inner: UninitializedOptimizerConfig::Dicl(UninitializedDiclOptimizationConfig {
@@ -1164,19 +1189,21 @@ fn create_pinocchio_example(
             }],
         },
         stored_input: StoredInput {
-            system: system.clone(),
+            system: system
+                .as_ref()
+                .map(|s| System::Template(Arguments(s.as_object().unwrap().to_owned()))),
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: json!(question),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: question.to_string(),
+                })],
             }],
         },
         output: Some(output),
         stored_output: Some(stored_output),
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
-        tool_params: None,
+        tool_params: DynamicToolParams::default(),
         output_schema: if is_json_function {
             Some(json!({
                 "type": "object",
