@@ -1,7 +1,8 @@
 #![expect(clippy::panic, clippy::print_stdout, clippy::unwrap_used)]
 use base64::Engine;
-use serde_json::json;
 use std::collections::HashMap;
+use std::sync::Arc;
+use tensorzero_core::rate_limiting::ScopeInfo;
 use tokio::time::{sleep, Duration};
 use url::Url;
 use uuid::Uuid;
@@ -11,27 +12,27 @@ use tracing_subscriber::{self, EnvFilter};
 use tensorzero::{InferenceOutputSource, LaunchOptimizationWorkflowParams, RenderedSample, Role};
 use tensorzero_core::{
     cache::CacheOptions,
-    config::{Config, ConfigFileGlob, ProviderTypesConfig},
+    config::{provider_types::ProviderTypesConfig, Config, ConfigFileGlob},
     db::{
-        clickhouse::{test_helpers::CLICKHOUSE_URL, ClickHouseConnectionInfo, ClickhouseFormat},
+        clickhouse::{test_helpers::CLICKHOUSE_URL, ClickHouseConnectionInfo},
         postgres::PostgresConnectionInfo,
     },
     endpoints::inference::InferenceClients,
     http::TensorzeroHttpClient,
     inference::types::{
-        file::Base64FileMetadata,
-        resolved_input::FileWithPath,
         storage::{StorageKind, StoragePath},
         stored_input::StoredFile,
-        Base64File, ContentBlock, ContentBlockChatOutput, FunctionType, ModelInferenceRequest,
-        ModelInput, RequestMessage, ResolvedContentBlock, ResolvedRequestMessage, StoredInput,
-        StoredInputMessage, StoredInputMessageContent, Text,
+        ContentBlock, ContentBlockChatOutput, FunctionType, ModelInferenceRequest, ModelInput,
+        ObjectStorageFile, ObjectStoragePointer, RequestMessage, ResolvedContentBlock,
+        ResolvedRequestMessage, StoredInput, StoredInputMessage, StoredInputMessageContent, System,
+        Text,
     },
+    model_table::ProviderTypeDefaultCredentials,
     optimization::{
         JobHandle, OptimizationJobInfo, Optimizer, OptimizerOutput, UninitializedOptimizerInfo,
     },
     stored_inference::StoredOutput,
-    tool::{Tool, ToolCall, ToolCallConfigDatabaseInsert, ToolCallOutput, ToolChoice, ToolResult},
+    tool::{DynamicToolParams, Tool, ToolCall, ToolCallOutput, ToolChoice, ToolResult},
     variant::JsonMode,
 };
 
@@ -70,7 +71,7 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
 
     let optimizer_info = test_case
         .get_optimizer_info(use_mock_inference_provider())
-        .load()
+        .load(&ProviderTypeDefaultCredentials::default())
         .await
         .unwrap();
 
@@ -122,7 +123,14 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
         .unwrap();
     let mut status;
     loop {
-        status = job_handle.poll(&client, &credentials).await.unwrap();
+        status = job_handle
+            .poll(
+                &client,
+                &credentials,
+                &ProviderTypeDefaultCredentials::default(),
+            )
+            .await
+            .unwrap();
         println!("Status: `{status:?}` Handle: `{job_handle}`");
         if matches!(status, OptimizationJobInfo::Completed { .. }) {
             break;
@@ -146,7 +154,11 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
     match output {
         OptimizerOutput::Model(model_config) => {
             let model_config = model_config
-                .load("test-fine-tuned-model", &ProviderTypesConfig::default())
+                .load(
+                    "test-fine-tuned-model",
+                    &ProviderTypesConfig::default(),
+                    &ProviderTypeDefaultCredentials::default(),
+                )
                 .await
                 .unwrap();
             // Test the model configuration
@@ -164,31 +176,23 @@ pub async fn run_test_case(test_case: &impl OptimizationTestCase) {
                 system: Some(system),
                 messages,
                 inference_id: Uuid::now_v7(),
-                tool_config: None,
-                temperature: None,
-                top_p: None,
-                max_tokens: None,
-                presence_penalty: None,
-                frequency_penalty: None,
-                seed: None,
-                stop_sequences: None,
-                stream: false,
                 json_mode: JsonMode::Off.into(),
                 function_type: FunctionType::Chat,
-                output_schema: None,
-                extra_body: Default::default(),
-                extra_headers: Default::default(),
-                extra_cache_key: None,
+                ..Default::default()
             };
             let clients = InferenceClients {
-                http_client: &client,
-                clickhouse_connection_info: &ClickHouseConnectionInfo::Disabled,
-                postgres_connection_info: &PostgresConnectionInfo::Disabled,
-                credentials: &HashMap::new(),
-                cache_options: &CacheOptions::default(),
-                tags: &Default::default(),
-                rate_limiting_config: &Default::default(),
-                otlp_config: &Default::default(),
+                http_client: client.clone(),
+                clickhouse_connection_info: ClickHouseConnectionInfo::new_disabled(),
+                postgres_connection_info: PostgresConnectionInfo::Disabled,
+                credentials: Arc::new(HashMap::new()),
+                cache_options: CacheOptions::default(),
+                tags: Arc::new(Default::default()),
+                rate_limiting_config: Arc::new(Default::default()),
+                otlp_config: Default::default(),
+                deferred_tasks: tokio_util::task::TaskTracker::new(),
+                scope_info: ScopeInfo {
+                    tags: Arc::new(HashMap::new()),
+                },
             };
             // We didn't produce a real model, so there's nothing to test
             if use_mock_inference_provider() {
@@ -222,7 +226,6 @@ pub async fn run_workflow_test_case_with_tensorzero_client(
         limit: Some(10),
         offset: None,
         val_fraction: None,
-        format: ClickhouseFormat::JsonEachRow,
         // We always mock the client tests since this is tested above
         optimizer_config: test_case.get_optimizer_info(true),
     };
@@ -285,19 +288,19 @@ fn generate_text_example() -> RenderedSample {
             }],
         },
         stored_input: StoredInput {
-            system: Some(json!(system_prompt)),
+            system: Some(System::Text(system_prompt.clone())),
             messages: vec![StoredInputMessage {
                 role: Role::User,
-                content: vec![StoredInputMessageContent::Text {
-                    value: "What is the capital of France?".into(),
-                }],
+                content: vec![StoredInputMessageContent::Text(Text {
+                    text: "What is the capital of France?".to_string(),
+                })],
             }],
         },
         output: Some(output.clone()),
         stored_output: Some(StoredOutput::Chat(output)),
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
-        tool_params: None,
+        tool_params: DynamicToolParams::default(),
         output_schema: None,
         dispreferred_outputs: vec![vec![ContentBlockChatOutput::Text(Text {
             text: "The capital of France is Marseille.".to_string(),
@@ -376,20 +379,20 @@ fn generate_tool_call_example() -> RenderedSample {
             ],
         },
         stored_input: StoredInput {
-            system: Some(json!(system_prompt)),
+            system: Some(System::Text(system_prompt.clone())),
             messages: vec![
                 StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("What is the weather in Paris?"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "What is the weather in Paris?".to_string(),
+                    })],
                 },
                 StoredInputMessage {
                     role: Role::Assistant,
                     content: vec![
-                        StoredInputMessageContent::Text {
-                            value: json!("Let me look that up for you."),
-                        },
+                        StoredInputMessageContent::Text(Text {
+                            text: "Let me look that up for you.".to_string(),
+                        }),
                         StoredInputMessageContent::ToolCall(ToolCall {
                             name: "get_weather".to_string(),
                             arguments: serde_json::json!({
@@ -413,22 +416,23 @@ fn generate_tool_call_example() -> RenderedSample {
                 },
                 StoredInputMessage {
                     role: Role::Assistant,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("The weather in Paris is sunny, 25 degrees Celsius."),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "The weather in Paris is sunny, 25 degrees Celsius.".to_string(),
+                    })],
                 },
                 StoredInputMessage {
                     role: Role::User,
-                    content: vec![StoredInputMessageContent::Text {
-                        value: json!("What is the weather in London?"),
-                    }],
+                    content: vec![StoredInputMessageContent::Text(Text {
+                        text: "What is the weather in London?".to_string(),
+                    })],
                 },
             ],
         },
         output: Some(tool_call_output.clone()),
         stored_output: Some(StoredOutput::Chat(tool_call_output)),
-        tool_params: Some(ToolCallConfigDatabaseInsert {
-            tools_available: vec![Tool {
+        tool_params: DynamicToolParams {
+            allowed_tools: None,
+            additional_tools: Some(vec![Tool {
                 name: "get_weather".to_string(),
                 description: "Get the weather for a location".to_string(),
                 parameters: serde_json::json!({
@@ -442,10 +446,11 @@ fn generate_tool_call_example() -> RenderedSample {
                     "required": ["location"]
                 }),
                 strict: false,
-            }],
-            tool_choice: ToolChoice::Auto,
+            }]),
+            tool_choice: Some(ToolChoice::Auto),
             parallel_tool_calls: None,
-        }),
+            provider_tools: None,
+        },
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
         output_schema: None,
@@ -472,48 +477,48 @@ fn generate_image_example() -> RenderedSample {
                     ResolvedContentBlock::Text(Text {
                         text: "What is the main color of this image?".to_string(),
                     }),
-                    ResolvedContentBlock::File(Box::new(FileWithPath {
-                        file: Base64File {
-                            url: None,
+                    ResolvedContentBlock::File(Box::new(ObjectStorageFile {
+                        file: ObjectStoragePointer {
+                            source_url: None,
                             mime_type: mime::IMAGE_PNG,
-                            data: base64::prelude::BASE64_STANDARD.encode(FERRIS_PNG),
-                        },
-                        storage_path: StoragePath {
-                            kind: StorageKind::Disabled,
-                            path: object_store::path::Path::parse(
-                                "observability/files/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"
-                            ).unwrap(),
+                            storage_path: StoragePath {
+                                kind: StorageKind::Disabled,
+                                path: object_store::path::Path::parse(
+                                    "observability/files/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"
+                                ).unwrap(),
+                            },
                         },
+                        data: base64::prelude::BASE64_STANDARD.encode(FERRIS_PNG),
                     })),
                 ],
             }],
         },
         stored_input: StoredInput {
-            system: Some(json!(system_prompt)),
+            system: Some(System::Text(system_prompt.clone())),
             messages: vec![StoredInputMessage {
                 role: Role::User,
                 content: vec![
-                    StoredInputMessageContent::Text {
-                        value: json!("What is the main color of this image?"),
-                    },
-                    StoredInputMessageContent::File(Box::new(StoredFile {
-                        file: Base64FileMetadata {
-                            url: None,
+                    StoredInputMessageContent::Text(Text {
+                        text: "What is the main color of this image?".to_string(),
+                    }),
+                    StoredInputMessageContent::File(Box::new(StoredFile(
+                        ObjectStoragePointer {
+                            source_url: None,
                             mime_type: mime::IMAGE_PNG,
+                            storage_path: StoragePath {
+                                kind: StorageKind::Disabled,
+                                path: object_store::path::Path::parse(
+                                    "observability/files/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"
+                                ).unwrap(),
+                            },
                         },
-                        storage_path: StoragePath {
-                            kind: StorageKind::Disabled,
-                            path: object_store::path::Path::parse(
-                                "observability/files/08bfa764c6dc25e658bab2b8039ddb494546c3bc5523296804efc4cab604df5d.png"
-                            ).unwrap(),
-                        },
-                    })),
+                    ))),
                 ],
             }],
         },
         output: Some(output.clone()),
         stored_output: Some(StoredOutput::Chat(output)),
-        tool_params: None,
+        tool_params: DynamicToolParams::default(),
         episode_id: Some(Uuid::now_v7()),
         inference_id: Some(Uuid::now_v7()),
         output_schema: None,
